@@ -59,10 +59,6 @@ dxJoint::dxJoint( dxWorld *w ) :
     w->nj++;
     feedback = 0;
 
-    // joint damping
-    use_damping = false;
-    damping_coefficient = 0.0;
-
     // Moved here by OSRF
     // Default to negative value, which means the current global value
     // will be used. If set non-negative, then this joint-specific 
