@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Open Source Robotics Foundation
+ * Copyright (C) 2012-2016 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -42,26 +42,28 @@ using namespace gazebo;
 using namespace common;
 
 /////////////////////////////////////////////////
-struct Vector3Hash : std::unary_function<const math::Vector3, std::size_t>
+struct Vector3Hash : std::unary_function<const ignition::math::Vector3d,
+  std::size_t>
 {
-  std::size_t operator()(const math::Vector3 _v) const
+  std::size_t operator()(const ignition::math::Vector3d &_v) const
   {
     std::size_t seed = 0;
-    boost::hash_combine(seed, _v.x);
-    boost::hash_combine(seed, _v.y);
-    boost::hash_combine(seed, _v.z);
+    boost::hash_combine(seed, _v.X());
+    boost::hash_combine(seed, _v.Y());
+    boost::hash_combine(seed, _v.Z());
     return seed;
   }
 };
 
 /////////////////////////////////////////////////
-struct Vector2dHash : std::unary_function<const math::Vector2d, std::size_t>
+struct Vector2dHash : std::unary_function<const ignition::math::Vector2d,
+  std::size_t>
 {
-  std::size_t operator()(const math::Vector2d _v) const
+  std::size_t operator()(const ignition::math::Vector2d &_v) const
   {
     std::size_t seed = 0;
-    boost::hash_combine(seed, _v.x);
-    boost::hash_combine(seed, _v.y);
+    boost::hash_combine(seed, _v.X());
+    boost::hash_combine(seed, _v.Y());
     return seed;
   }
 };
@@ -120,7 +122,8 @@ Mesh *ColladaLoader::Load(const std::string &_filename)
   {
     TiXmlElement *unitXml = assetXml->FirstChildElement("unit");
     if (unitXml && unitXml->Attribute("meter"))
-      this->dataPtr->meter = math::parseFloat(unitXml->Attribute("meter"));
+      this->dataPtr->meter = ignition::math::parseFloat(
+          unitXml->Attribute("meter"));
   }
 
   Mesh *mesh = new Mesh();
@@ -153,19 +156,19 @@ void ColladaLoader::LoadScene(Mesh *_mesh)
   TiXmlElement *nodeXml = visSceneXml->FirstChildElement("node");
   while (nodeXml)
   {
-    this->LoadNode(nodeXml, _mesh, math::Matrix4::IDENTITY);
+    this->LoadNode(nodeXml, _mesh, ignition::math::Matrix4d::Identity);
     nodeXml = nodeXml->NextSiblingElement("node");
   }
 }
 
 /////////////////////////////////////////////////
 void ColladaLoader::LoadNode(TiXmlElement *_elem, Mesh *_mesh,
-    const math::Matrix4 &_transform)
+    const ignition::math::Matrix4d &_transform)
 {
   TiXmlElement *nodeXml;
   TiXmlElement *instGeomXml;
 
-  math::Matrix4 transform = this->LoadNodeTransform(_elem);
+  ignition::math::Matrix4d transform = this->LoadNodeTransform(_elem);
   transform = _transform * transform;
 
   if (_elem->Attribute("name"))
@@ -262,16 +265,16 @@ void ColladaLoader::LoadNode(TiXmlElement *_elem, Mesh *_mesh,
 }
 
 /////////////////////////////////////////////////
-math::Matrix4 ColladaLoader::LoadNodeTransform(TiXmlElement *_elem)
+ignition::math::Matrix4d ColladaLoader::LoadNodeTransform(TiXmlElement *_elem)
 {
-  math::Matrix4 transform(math::Matrix4::IDENTITY);
+  ignition::math::Matrix4d transform(ignition::math::Matrix4d::Identity);
 
   if (_elem->FirstChildElement("matrix"))
   {
     std::string matrixStr = _elem->FirstChildElement("matrix")->GetText();
     std::istringstream iss(matrixStr);
     std::vector<double> values(16);
-    for (unsigned int i = 0; i < 16; i++)
+    for (unsigned int i = 0; i < 16; ++i)
       iss >> values[i];
     transform.Set(values[0], values[1], values[2], values[3],
         values[4], values[5], values[6], values[7],
@@ -283,27 +286,29 @@ math::Matrix4 ColladaLoader::LoadNodeTransform(TiXmlElement *_elem)
     if (_elem->FirstChildElement("translate"))
     {
       std::string transStr = _elem->FirstChildElement("translate")->GetText();
-      math::Vector3 translate;
-      translate = boost::lexical_cast<math::Vector3>(transStr);
+      ignition::math::Vector3d translate;
+      translate = boost::lexical_cast<ignition::math::Vector3d>(transStr);
       // translate *= this->dataPtr->meter;
-      transform.SetTranslate(translate);
+      transform.Translate(translate);
     }
 
     TiXmlElement *rotateXml = _elem->FirstChildElement("rotate");
     while (rotateXml)
     {
-      math::Matrix3 mat;
-      math::Vector3 axis;
+      ignition::math::Matrix3d mat;
+      ignition::math::Vector3d axis;
       double angle;
 
       std::string rotateStr = rotateXml->GetText();
       std::istringstream iss(rotateStr);
 
-      iss >> axis.x >> axis.y >> axis.z;
+      iss >> axis.X() >> axis.Y() >> axis.Z();
       iss >> angle;
-      mat.SetFromAxis(axis, GZ_DTOR(angle));
+      mat.Axis(axis, IGN_DTOR(angle));
+      ignition::math::Matrix4d mat4(ignition::math::Matrix4d::Identity);
+      mat4 = mat;
 
-      transform = transform * mat;
+      transform = transform * mat4;
 
       rotateXml = rotateXml->NextSiblingElement("rotate");
     }
@@ -311,10 +316,10 @@ math::Matrix4 ColladaLoader::LoadNodeTransform(TiXmlElement *_elem)
     if (_elem->FirstChildElement("scale"))
     {
       std::string scaleStr = _elem->FirstChildElement("scale")->GetText();
-      math::Vector3 scale;
-      scale = boost::lexical_cast<math::Vector3>(scaleStr);
-      math::Matrix4 scaleMat;
-      scaleMat.SetScale(scale);
+      ignition::math::Vector3d scale;
+      scale = boost::lexical_cast<ignition::math::Vector3d>(scaleStr);
+      ignition::math::Matrix4d scaleMat;
+      scaleMat.Scale(scale);
       transform = transform * scaleMat;
     }
   }
@@ -324,7 +329,8 @@ math::Matrix4 ColladaLoader::LoadNodeTransform(TiXmlElement *_elem)
 
 /////////////////////////////////////////////////
 void ColladaLoader::LoadController(TiXmlElement *_contrXml,
-      TiXmlElement *_skelXml, const math::Matrix4 &_transform, Mesh *_mesh)
+      TiXmlElement *_skelXml,
+      const ignition::math::Matrix4d &_transform, Mesh *_mesh)
 {
   Skeleton *skeleton = new Skeleton(this->LoadSkeletonNodes(_skelXml, NULL));
   _mesh->SetSkeleton(skeleton);
@@ -338,12 +344,12 @@ void ColladaLoader::LoadController(TiXmlElement *_contrXml,
   TiXmlElement *skinXml = _contrXml->FirstChildElement("skin");
   std::string geomURL = skinXml->Attribute("source");
 
-  math::Matrix4 bindTrans;
+  ignition::math::Matrix4d bindTrans;
   std::string matrixStr =
         skinXml->FirstChildElement("bind_shape_matrix")->GetText();
   std::istringstream iss(matrixStr);
   std::vector<double> values(16);
-  for (unsigned int i = 0; i < 16; i++)
+  for (unsigned int i = 0; i < 16; ++i)
     iss >> values[i];
   bindTrans.Set(values[0], values[1], values[2], values[3],
                 values[4], values[5], values[6], values[7],
@@ -395,18 +401,26 @@ void ColladaLoader::LoadController(TiXmlElement *_contrXml,
   std::vector<std::string> strs;
   boost::split(strs, posesStr, boost::is_any_of("   "));
 
-  for (unsigned int i = 0; i < joints.size(); i++)
+  for (unsigned int i = 0; i < joints.size(); ++i)
   {
     unsigned int id = i * 16;
-    math::Matrix4 mat;
-    mat.Set(math::parseFloat(strs[id +  0]), math::parseFloat(strs[id +  1]),
-            math::parseFloat(strs[id +  2]), math::parseFloat(strs[id +  3]),
-            math::parseFloat(strs[id +  4]), math::parseFloat(strs[id +  5]),
-            math::parseFloat(strs[id +  6]), math::parseFloat(strs[id +  7]),
-            math::parseFloat(strs[id +  8]), math::parseFloat(strs[id +  9]),
-            math::parseFloat(strs[id + 10]), math::parseFloat(strs[id + 11]),
-            math::parseFloat(strs[id + 12]), math::parseFloat(strs[id + 13]),
-            math::parseFloat(strs[id + 14]), math::parseFloat(strs[id + 15]));
+    ignition::math::Matrix4d mat;
+    mat.Set(ignition::math::parseFloat(strs[id +  0]),
+            ignition::math::parseFloat(strs[id +  1]),
+            ignition::math::parseFloat(strs[id +  2]),
+            ignition::math::parseFloat(strs[id +  3]),
+            ignition::math::parseFloat(strs[id +  4]),
+            ignition::math::parseFloat(strs[id +  5]),
+            ignition::math::parseFloat(strs[id +  6]),
+            ignition::math::parseFloat(strs[id +  7]),
+            ignition::math::parseFloat(strs[id +  8]),
+            ignition::math::parseFloat(strs[id +  9]),
+            ignition::math::parseFloat(strs[id + 10]),
+            ignition::math::parseFloat(strs[id + 11]),
+            ignition::math::parseFloat(strs[id + 12]),
+            ignition::math::parseFloat(strs[id + 13]),
+            ignition::math::parseFloat(strs[id + 14]),
+            ignition::math::parseFloat(strs[id + 15]));
 
     skeleton->GetNodeByName(joints[i])->SetInverseBindTransform(mat);
   }
@@ -442,7 +456,7 @@ void ColladaLoader::LoadController(TiXmlElement *_contrXml,
   boost::split(wStrs, wString, boost::is_any_of("   "));
 
   std::vector<float> weights;
-  for (unsigned int i = 0; i < wStrs.size(); i++)
+  for (unsigned int i = 0; i < wStrs.size(); ++i)
     weights.push_back(math::parseFloat(wStrs[i]));
 
   std::string cString = vertWeightsXml->FirstChildElement("vcount")->GetText();
@@ -456,18 +470,18 @@ void ColladaLoader::LoadController(TiXmlElement *_contrXml,
   std::vector<unsigned int> vCount;
   std::vector<unsigned int> v;
 
-  for (unsigned int i = 0; i < vCountStrs.size(); i++)
+  for (unsigned int i = 0; i < vCountStrs.size(); ++i)
     vCount.push_back(math::parseInt(vCountStrs[i]));
 
-  for (unsigned int i = 0; i < vStrs.size(); i++)
+  for (unsigned int i = 0; i < vStrs.size(); ++i)
     v.push_back(math::parseInt(vStrs[i]));
 
   skeleton->SetNumVertAttached(vCount.size());
 
   unsigned int vIndex = 0;
-  for (unsigned int i = 0; i < vCount.size(); i++)
+  for (unsigned int i = 0; i < vCount.size(); ++i)
   {
-    for (unsigned int j = 0; j < vCount[i]; j++)
+    for (unsigned int j = 0; j < vCount[i]; ++j)
     {
       skeleton->AddVertNodeWeight(i, joints[v[vIndex + jOffset]],
                                     weights[v[vIndex + wOffset]]);
@@ -545,11 +559,11 @@ void ColladaLoader::LoadAnimationSet(TiXmlElement *_xml, Skeleton *_skel)
         if (sep == '(')
         {
           std::string idx1Str = idxStr.substr(0, 1);
-          idx1 = math::parseInt(idx1Str);
+          idx1 = ignition::math::parseInt(idx1Str);
           if (idxStr.length() > 4)
           {
             std::string idx2Str = idxStr.substr(3, 1);
-            idx2 = math::parseInt(idx2Str);
+            idx2 = ignition::math::parseInt(idx2Str);
           }
         }
 
@@ -578,7 +592,7 @@ void ColladaLoader::LoadAnimationSet(TiXmlElement *_xml, Skeleton *_skel)
       boost::split(timeStrs, timeStr, boost::is_any_of("   "));
 
       std::vector<double> times;
-      for (unsigned int i = 0; i < timeStrs.size(); i++)
+      for (unsigned int i = 0; i < timeStrs.size(); ++i)
         times.push_back(math::parseFloat(timeStrs[i]));
 
       TiXmlElement *output = frameTransXml->FirstChildElement("float_array");
@@ -587,16 +601,17 @@ void ColladaLoader::LoadAnimationSet(TiXmlElement *_xml, Skeleton *_skel)
       boost::split(outputStrs, outputStr, boost::is_any_of("   "));
 
       std::vector<double> values;
-      for (unsigned int i = 0; i < outputStrs.size(); i++)
+      for (unsigned int i = 0; i < outputStrs.size(); ++i)
         values.push_back(math::parseFloat(outputStrs[i]));
 
       TiXmlElement *accessor =
         frameTransXml->FirstChildElement("technique_common");
       accessor = accessor->FirstChildElement("accessor");
 
-      unsigned int stride = math::parseInt(accessor->Attribute("stride"));
+      unsigned int stride =
+        ignition::math::parseInt(accessor->Attribute("stride"));
 
-      for (unsigned int i = 0; i < times.size(); i++)
+      for (unsigned int i = 0; i < times.size(); ++i)
       {
         if (animation[targetBone].find(times[i]) == animation[targetBone].end())
           animation[targetBone][times[i]] =
@@ -604,7 +619,7 @@ void ColladaLoader::LoadAnimationSet(TiXmlElement *_xml, Skeleton *_skel)
 
         std::vector<NodeTransform> *frame = &animation[targetBone][times[i]];
 
-        for (unsigned int j = 0; j < (*frame).size(); j++)
+        for (unsigned int j = 0; j < (*frame).size(); ++j)
         {
           NodeTransform *nt = &((*frame)[j]);
           if (nt->GetSID() == targetTrans)
@@ -634,8 +649,8 @@ void ColladaLoader::LoadAnimationSet(TiXmlElement *_xml, Skeleton *_skel)
     for (RawNodeAnim::iterator niter = iter->second.begin();
           niter != iter->second.end(); ++niter)
     {
-      math::Matrix4 transform(math::Matrix4::IDENTITY);
-      for (unsigned int i = 0; i < niter->second.size(); i++)
+      ignition::math::Matrix4d transform(ignition::math::Matrix4d::Identity);
+      for (unsigned int i = 0; i < niter->second.size(); ++i)
       {
         niter->second[i].RecalculateMatrix();
         transform = transform * niter->second[i]();
@@ -676,14 +691,14 @@ SkeletonNode* ColladaLoader::LoadSkeletonNodes(TiXmlElement *_xml,
 void ColladaLoader::SetSkeletonNodeTransform(TiXmlElement *_elem,
       SkeletonNode *_node)
 {
-  math::Matrix4 transform(math::Matrix4::IDENTITY);
+  ignition::math::Matrix4d transform(ignition::math::Matrix4d::Identity);
 
   if (_elem->FirstChildElement("matrix"))
   {
     std::string matrixStr = _elem->FirstChildElement("matrix")->GetText();
     std::istringstream iss(matrixStr);
     std::vector<double> values(16);
-    for (unsigned int i = 0; i < 16; i++)
+    for (unsigned int i = 0; i < 16; ++i)
       iss >> values[i];
     transform.Set(values[0], values[1], values[2], values[3],
         values[4], values[5], values[6], values[7],
@@ -701,10 +716,10 @@ void ColladaLoader::SetSkeletonNodeTransform(TiXmlElement *_elem,
     if (_elem->FirstChildElement("translate"))
     {
       std::string transStr = _elem->FirstChildElement("translate")->GetText();
-      math::Vector3 translate;
-      translate = boost::lexical_cast<math::Vector3>(transStr);
+      ignition::math::Vector3d translate;
+      translate = boost::lexical_cast<ignition::math::Vector3d>(transStr);
       // translate *= this->dataPtr->meter;
-      transform.SetTranslate(translate);
+      transform.Translate(translate);
 
       NodeTransform nt(transform);
       if (_elem->FirstChildElement("translate")->Attribute("sid"))
@@ -717,18 +732,18 @@ void ColladaLoader::SetSkeletonNodeTransform(TiXmlElement *_elem,
     TiXmlElement *rotateXml = _elem->FirstChildElement("rotate");
     while (rotateXml)
     {
-      math::Matrix3 mat;
-      math::Vector3 axis;
+      ignition::math::Matrix3d mat;
+      ignition::math::Vector3d axis;
       double angle;
 
       std::string rotateStr = rotateXml->GetText();
       std::istringstream iss(rotateStr);
 
-      iss >> axis.x >> axis.y >> axis.z;
+      iss >> axis.X() >> axis.Y() >> axis.Z();
       iss >> angle;
-      mat.SetFromAxis(axis, GZ_DTOR(angle));
+      mat.Axis(axis, IGN_DTOR(angle));
 
-      math::Matrix4 mat4(math::Matrix4::IDENTITY);
+      ignition::math::Matrix4d mat4(ignition::math::Matrix4d::Identity);
       mat4 = mat;
       NodeTransform nt(mat4);
       if (rotateXml->Attribute("sid"))
@@ -737,7 +752,7 @@ void ColladaLoader::SetSkeletonNodeTransform(TiXmlElement *_elem,
       nt.SetSourceValues(axis, angle);
       _node->AddRawTransform(nt);
 
-      transform = transform * mat;
+      transform = transform * mat4;
 
       rotateXml = rotateXml->NextSiblingElement("rotate");
     }
@@ -745,10 +760,10 @@ void ColladaLoader::SetSkeletonNodeTransform(TiXmlElement *_elem,
     if (_elem->FirstChildElement("scale"))
     {
       std::string scaleStr = _elem->FirstChildElement("scale")->GetText();
-      math::Vector3 scale;
-      scale = boost::lexical_cast<math::Vector3>(scaleStr);
-      math::Matrix4 scaleMat;
-      scaleMat.SetScale(scale);
+      ignition::math::Vector3d scale;
+      scale = boost::lexical_cast<ignition::math::Vector3d>(scaleStr);
+      ignition::math::Matrix4d scaleMat;
+      scaleMat.Scale(scale);
 
       NodeTransform nt(scaleMat);
       if (_elem->FirstChildElement("matrix")->Attribute("sid"))
@@ -766,7 +781,7 @@ void ColladaLoader::SetSkeletonNodeTransform(TiXmlElement *_elem,
 
 /////////////////////////////////////////////////
 void ColladaLoader::LoadGeometry(TiXmlElement *_xml,
-                                 const math::Matrix4 &_transform, Mesh *_mesh)
+    const ignition::math::Matrix4d &_transform, Mesh *_mesh)
 {
   TiXmlElement *meshXml = _xml->FirstChildElement("mesh");
   TiXmlElement *childXml;
@@ -836,9 +851,9 @@ TiXmlElement *ColladaLoader::GetElementId(TiXmlElement *_parent,
 
 /////////////////////////////////////////////////
 void ColladaLoader::LoadVertices(const std::string &_id,
-    const math::Matrix4 &_transform,
-    std::vector<math::Vector3> &_verts,
-    std::vector<math::Vector3> &_norms)
+    const ignition::math::Matrix4d &_transform,
+    std::vector<ignition::math::Vector3d> &_verts,
+    std::vector<ignition::math::Vector3d> &_norms)
 {
   std::map<unsigned int, unsigned int> vertDup;
   std::map<unsigned int, unsigned int> normDup;
@@ -847,9 +862,9 @@ void ColladaLoader::LoadVertices(const std::string &_id,
 
 /////////////////////////////////////////////////
 void ColladaLoader::LoadVertices(const std::string &_id,
-    const math::Matrix4 &_transform,
-    std::vector<math::Vector3> &_verts,
-    std::vector<math::Vector3> &_norms,
+    const ignition::math::Matrix4d &_transform,
+    std::vector<ignition::math::Vector3d> &_verts,
+    std::vector<ignition::math::Vector3d> &_norms,
     std::map<unsigned int, unsigned int> &_vertDups,
     std::map<unsigned int, unsigned int> &_normDups)
 {
@@ -882,8 +897,8 @@ void ColladaLoader::LoadVertices(const std::string &_id,
 
 /////////////////////////////////////////////////
 void ColladaLoader::LoadPositions(const std::string &_id,
-    const math::Matrix4 &_transform,
-    std::vector<math::Vector3> &_values,
+    const ignition::math::Matrix4d &_transform,
+    std::vector<ignition::math::Vector3d> &_values,
     std::map<unsigned int, unsigned int> &_duplicates)
 {
   if (this->dataPtr->positionIds.find(_id) != this->dataPtr->positionIds.end())
@@ -931,7 +946,8 @@ void ColladaLoader::LoadPositions(const std::string &_id,
   }
   std::string valueStr = floatArrayXml->GetText();
 
-  boost::unordered_map<math::Vector3, unsigned int, Vector3Hash> unique;
+  boost::unordered_map<ignition::math::Vector3d,
+    unsigned int, Vector3Hash> unique;
 
   std::vector<std::string> strs;
   std::vector<std::string>::iterator iter, end;
@@ -940,8 +956,10 @@ void ColladaLoader::LoadPositions(const std::string &_id,
   end = strs.end();
   for (iter = strs.begin(); iter != end; iter += 3)
   {
-    math::Vector3 vec(math::parseFloat(*iter), math::parseFloat(*(iter+1)),
-        math::parseFloat(*(iter+2)));
+    ignition::math::Vector3d vec(math::parseFloat(*iter),
+        ignition::math::parseFloat(*(iter+1)),
+        ignition::math::parseFloat(*(iter+2)));
+
     vec = _transform * vec;
     _values.push_back(vec);
 
@@ -958,8 +976,8 @@ void ColladaLoader::LoadPositions(const std::string &_id,
 
 /////////////////////////////////////////////////
 void ColladaLoader::LoadNormals(const std::string &_id,
-    const math::Matrix4 &_transform,
-    std::vector<math::Vector3> &_values,
+    const ignition::math::Matrix4d &_transform,
+    std::vector<ignition::math::Vector3d> &_values,
     std::map<unsigned int, unsigned int> &_duplicates)
 {
   if (this->dataPtr->normalIds.find(_id) != this->dataPtr->normalIds.end())
@@ -969,8 +987,8 @@ void ColladaLoader::LoadNormals(const std::string &_id,
     return;
   }
 
-  math::Matrix4 rotMat = _transform;
-  rotMat.SetTranslate(math::Vector3::Zero);
+  ignition::math::Matrix4d rotMat = _transform;
+  rotMat.Translate(ignition::math::Vector3d::Zero);
 
   TiXmlElement *normalsXml = this->GetElementId("source", _id);
   if (!normalsXml)
@@ -1009,14 +1027,15 @@ void ColladaLoader::LoadNormals(const std::string &_id,
     return;
   }
 
-  boost::unordered_map<math::Vector3, unsigned int, Vector3Hash> unique;
+  boost::unordered_map<ignition::math::Vector3d,
+    unsigned int, Vector3Hash> unique;
 
   std::string valueStr = floatArrayXml->GetText();
   std::istringstream iss(valueStr);
   do
   {
-    math::Vector3 vec;
-    iss >> vec.x >> vec.y >> vec.z;
+    ignition::math::Vector3d vec;
+    iss >> vec.X() >> vec.Y() >> vec.Z();
     if (iss)
     {
       vec = rotMat * vec;
@@ -1037,7 +1056,7 @@ void ColladaLoader::LoadNormals(const std::string &_id,
 
 /////////////////////////////////////////////////
 void ColladaLoader::LoadTexCoords(const std::string &_id,
-    std::vector<math::Vector2d> &_values,
+    std::vector<ignition::math::Vector2d> &_values,
     std::map<unsigned int, unsigned int> &_duplicates)
 {
   if (this->dataPtr->texcoordIds.find(_id) != this->dataPtr->texcoordIds.end())
@@ -1156,7 +1175,8 @@ void ColladaLoader::LoadTexCoords(const std::string &_id,
   if (totCount == 0)
     return;
 
-  boost::unordered_map<math::Vector2d, unsigned int, Vector2dHash> unique;
+  boost::unordered_map<ignition::math::Vector2d,
+    unsigned int, Vector2dHash> unique;
 
   // Read the raw texture values, and split them on spaces.
   std::string valueStr = floatArrayXml->GetText();
@@ -1167,7 +1187,7 @@ void ColladaLoader::LoadTexCoords(const std::string &_id,
   for (int i = 0; i < totCount; i += stride)
   {
     // We only handle 2D texture coordinates right now.
-    math::Vector2d vec(boost::lexical_cast<double>(values[i]),
+    ignition::math::Vector2d vec(boost::lexical_cast<double>(values[i]),
           1.0 - boost::lexical_cast<double>(values[i+1]));
     _values.push_back(vec);
 
@@ -1351,7 +1371,7 @@ void ColladaLoader::LoadColorOrTexture(TiXmlElement *_elem,
 
 /////////////////////////////////////////////////
 void ColladaLoader::LoadPolylist(TiXmlElement *_polylistXml,
-    const math::Matrix4 &_transform,
+    const ignition::math::Matrix4d &_transform,
     Mesh *_mesh)
 {
   // This function parses polylist types in collada into
@@ -1388,9 +1408,9 @@ void ColladaLoader::LoadPolylist(TiXmlElement *_polylistXml,
 
   TiXmlElement *polylistInputXml = _polylistXml->FirstChildElement("input");
 
-  std::vector<math::Vector3> verts;
-  std::vector<math::Vector3> norms;
-  std::vector<math::Vector2d> texcoords;
+  std::vector<ignition::math::Vector3d> verts;
+  std::vector<ignition::math::Vector3d> norms;
+  std::vector<ignition::math::Vector2d> texcoords;
 
   const unsigned int VERTEX = 0;
   const unsigned int NORMAL = 1;
@@ -1405,9 +1425,9 @@ void ColladaLoader::LoadPolylist(TiXmlElement *_polylistXml,
   std::map<unsigned int, unsigned int> normalDupMap;
   std::map<unsigned int, unsigned int> positionDupMap;
 
-  math::Matrix4 bindShapeMat(math::Matrix4::IDENTITY);
+  ignition::math::Matrix4d bindShapeMat(ignition::math::Matrix4d::Identity);
   if (_mesh->HasSkeleton())
-    bindShapeMat = _mesh->GetSkeleton()->GetBindShapeTransform();
+    bindShapeMat = _mesh->GetSkeleton()->BindShapeTransform();
 
   // read input elements
   std::map<const unsigned int, int> inputs;
@@ -1423,25 +1443,25 @@ void ColladaLoader::LoadPolylist(TiXmlElement *_polylistXml,
           positionDupMap, normalDupMap);
       if (norms.size() > count)
         combinedVertNorms = true;
-      inputs[VERTEX] = math::parseInt(offset);
+      inputs[VERTEX] = ignition::math::parseInt(offset);
       hasVertices = true;
     }
     else if (semantic == "NORMAL")
     {
       this->LoadNormals(source, _transform, norms, normalDupMap);
       combinedVertNorms = false;
-      inputs[NORMAL] = math::parseInt(offset);
+      inputs[NORMAL] = ignition::math::parseInt(offset);
       hasNormals = true;
     }
     else if (semantic == "TEXCOORD")
     {
       this->LoadTexCoords(source, texcoords, texDupMap);
-      inputs[TEXCOORD] = math::parseInt(offset);
+      inputs[TEXCOORD] = ignition::math::parseInt(offset);
       hasTexcoords = true;
     }
     else
     {
-      inputs[otherSemantics++] = math::parseInt(offset);
+      inputs[otherSemantics++] = ignition::math::parseInt(offset);
       gzwarn << "Polylist input semantic: '" << semantic << "' is currently"
           << " not supported" << std::endl;
     }
@@ -1495,9 +1515,9 @@ void ColladaLoader::LoadPolylist(TiXmlElement *_polylistXml,
         if (j == 2)
           triangle_index = (k)*inputs.size();
 
-        for (unsigned int i = 0; i < inputs.size(); i++)
+        for (unsigned int i = 0; i < inputs.size(); ++i)
         {
-          values[i] = math::parseInt(strs_iter[triangle_index+i]);
+          values[i] = ignition::math::parseInt(strs_iter[triangle_index+i]);
           /*gzerr << "debug parsing "
                 << " poly-i[" << l
                 << "] tri-end-index[" << k
@@ -1599,7 +1619,7 @@ void ColladaLoader::LoadPolylist(TiXmlElement *_polylistXml,
             if (_mesh->HasSkeleton())
             {
               subMesh->SetVertex(newVertIndex, bindShapeMat *
-                  subMesh->GetVertex(newVertIndex));
+                  subMesh->Vertex(newVertIndex));
               Skeleton *skel = _mesh->GetSkeleton();
               for (unsigned int i = 0;
                   i < skel->GetNumVertNodeWeights(daeVertIndex); ++i)
@@ -1632,8 +1652,8 @@ void ColladaLoader::LoadPolylist(TiXmlElement *_polylistXml,
               inputRemappedTexcoordIndex =
                   texDupMap[inputRemappedTexcoordIndex];
             }
-            subMesh->AddTexCoord(texcoords[inputRemappedTexcoordIndex].x,
-                texcoords[inputRemappedTexcoordIndex].y);
+            subMesh->AddTexCoord(texcoords[inputRemappedTexcoordIndex].X(),
+                texcoords[inputRemappedTexcoordIndex].Y());
             input.texcoordIndex = inputRemappedTexcoordIndex;
           }
 
@@ -1655,7 +1675,7 @@ void ColladaLoader::LoadPolylist(TiXmlElement *_polylistXml,
 
 /////////////////////////////////////////////////
 void ColladaLoader::LoadTriangles(TiXmlElement *_trianglesXml,
-                                  const math::Matrix4 &_transform,
+                                  const ignition::math::Matrix4d &_transform,
                                   Mesh *_mesh)
 {
   SubMesh *subMesh = new SubMesh;
@@ -1687,9 +1707,9 @@ void ColladaLoader::LoadTriangles(TiXmlElement *_trianglesXml,
 
   TiXmlElement *trianglesInputXml = _trianglesXml->FirstChildElement("input");
 
-  std::vector<math::Vector3> verts;
-  std::vector<math::Vector3> norms;
-  std::vector<math::Vector2d> texcoords;
+  std::vector<ignition::math::Vector3d> verts;
+  std::vector<ignition::math::Vector3d> norms;
+  std::vector<ignition::math::Vector2d> texcoords;
 
   const unsigned int VERTEX = 0;
   const unsigned int NORMAL = 1;
@@ -1718,26 +1738,26 @@ void ColladaLoader::LoadTriangles(TiXmlElement *_trianglesXml,
           positionDupMap, normalDupMap);
       if (norms.size() > count)
         combinedVertNorms = true;
-      inputs[VERTEX] = math::parseInt(offset);
+      inputs[VERTEX] = ignition::math::parseInt(offset);
       hasVertices = true;
     }
     else if (semantic == "NORMAL")
     {
       this->LoadNormals(source, _transform, norms, normalDupMap);
       combinedVertNorms = false;
-      inputs[NORMAL] = math::parseInt(offset);
+      inputs[NORMAL] = ignition::math::parseInt(offset);
       hasNormals = true;
     }
     else if (semantic == "TEXCOORD" && !hasTexcoords)
     {
       // we currently only support one set of UVs
       this->LoadTexCoords(source, texcoords, texDupMap);
-      inputs[TEXCOORD] = math::parseInt(offset);
+      inputs[TEXCOORD] = ignition::math::parseInt(offset);
       hasTexcoords = true;
     }
     else
     {
-      inputs[otherSemantics++] = math::parseInt(offset);
+      inputs[otherSemantics++] = ignition::math::parseInt(offset);
       gzwarn << "Triangle input semantic: '" << semantic << "' is currently"
           << " not supported" << std::endl;
     }
@@ -1796,7 +1816,7 @@ void ColladaLoader::LoadTriangles(TiXmlElement *_trianglesXml,
   for (unsigned int j = 0; j < strs.size(); j += offsetSize)
   {
     for (unsigned int i = 0; i < offsetSize; ++i)
-      values[i] = math::parseInt(strs[j+i]);
+      values[i] = ignition::math::parseInt(strs[j+i]);
 
     unsigned int daeVertIndex = 0;
     bool addIndex = !hasVertices;
@@ -1911,8 +1931,8 @@ void ColladaLoader::LoadTriangles(TiXmlElement *_trianglesXml,
         unsigned int inputRemappedTexcoordIndex = values[inputs[TEXCOORD]];
         if (texDupMap.find(inputRemappedTexcoordIndex) != texDupMap.end())
           inputRemappedTexcoordIndex = texDupMap[inputRemappedTexcoordIndex];
-        subMesh->AddTexCoord(texcoords[inputRemappedTexcoordIndex].x,
-            texcoords[inputRemappedTexcoordIndex].y);
+        subMesh->AddTexCoord(texcoords[inputRemappedTexcoordIndex].X(),
+            texcoords[inputRemappedTexcoordIndex].Y());
         input.texcoordIndex = inputRemappedTexcoordIndex;
       }
 
@@ -1932,7 +1952,7 @@ void ColladaLoader::LoadTriangles(TiXmlElement *_trianglesXml,
 
 /////////////////////////////////////////////////
 void ColladaLoader::LoadLines(TiXmlElement *_xml,
-    const math::Matrix4 &_transform,
+    const ignition::math::Matrix4d &_transform,
     Mesh *_mesh)
 {
   SubMesh *subMesh = new SubMesh;
@@ -1943,8 +1963,8 @@ void ColladaLoader::LoadLines(TiXmlElement *_xml,
   // std::string semantic = inputXml->Attribute("semantic");
   std::string source = inputXml->Attribute("source");
 
-  std::vector<math::Vector3> verts;
-  std::vector<math::Vector3> norms;
+  std::vector<ignition::math::Vector3d> verts;
+  std::vector<ignition::math::Vector3d> norms;
   this->LoadVertices(source, _transform, verts, norms);
 
   TiXmlElement *pXml = _xml->FirstChildElement("p");
@@ -1974,7 +1994,8 @@ float ColladaLoader::LoadFloat(TiXmlElement *_elem)
 
   if (_elem->FirstChildElement("float"))
   {
-    value = math::parseFloat(_elem->FirstChildElement("float")->GetText());
+    value =
+      ignition::math::parseFloat(_elem->FirstChildElement("float")->GetText());
   }
 
   return value;
