@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Open Source Robotics Foundation
+ * Copyright (C) 2012-2016 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -23,9 +23,10 @@ using namespace gui;
 event::EventT<void (std::string, std::string)> Events::createEntity;
 event::EventT<void (bool)> Events::moveMode;
 event::EventT<void (std::string)> Events::manipMode;
-event::EventT<void (std::string, std::string, std::string, bool)>
+event::EventT<void (std::string, std::string, std::string, bool, bool)>
     Events::alignMode;
 event::EventT<void (bool)> Events::fullScreen;
+event::EventT<void (bool)> Events::showToolbars;
 event::EventT<void ()> Events::fps;
 event::EventT<void ()> Events::orbit;
 event::EventT<void (std::string)> Events::keyPress;
@@ -35,3 +36,7 @@ event::EventT<void (int)> Events::inputStepSize;
 event::EventT<void (const std::string &)> Events::follow;
 event::EventT<void (bool)> Events::leftPaneVisibility;
 event::EventT<void ()> Events::mainWindowReady;
+event::EventT<void (const std::string &)> Events::editModel;
+event::EventT<void (const std::string &)> Events::windowMode;
+event::EventT<void (const std::string &, const math::Vector3 &)>
+    Events::scaleEntity;
