@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Open Source Robotics Foundation
+ * Copyright (C) 2012-2016 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,10 +14,14 @@
  * limitations under the License.
  *
 */
+#include <boost/bind.hpp>
 
 #include "gazebo/math/Angle.hh"
+
+#include "gazebo/common/Color.hh"
+
+#include "gazebo/gui/Conversions.hh"
 #include "gazebo/gui/building/ImportImageDialog.hh"
-#include "gazebo/gui/building/BuildingItem.hh"
 #include "gazebo/gui/building/GridLines.hh"
 #include "gazebo/gui/building/EditorItem.hh"
 #include "gazebo/gui/building/RectItem.hh"
@@ -203,7 +207,10 @@ void EditorView::contextMenuEvent(QContextMenuEvent *_event)
 /////////////////////////////////////////////////
 void EditorView::wheelEvent(QWheelEvent *_event)
 {
-  int numSteps = (_event->delta()/8) / 15;
+  int wheelIncr = 120;
+  int sign = (_event->delta() > 0) ? 1 : -1;
+  int delta = std::max(std::abs(_event->delta()), wheelIncr) * sign;
+  int numSteps = delta / wheelIncr;
 
   QMatrix mat = matrix();
   QPointF mousePosition = _event->pos();
@@ -348,8 +355,8 @@ void EditorView::mouseMoveEvent(QMouseEvent *_event)
                   distanceToClose)
               {
                 pf = anotherWall->GetStartPoint();
-                this->snapGrabberOther = anotherWall->grabbers[0];
-                this->snapGrabberCurrent = wallSegmentItem->grabbers[1];
+                this->snapGrabberOther = anotherWall->Grabbers()[0];
+                this->snapGrabberCurrent = wallSegmentItem->Grabbers()[1];
                 this->snapToGrabber = true;
                 break;
               }
@@ -357,8 +364,8 @@ void EditorView::mouseMoveEvent(QMouseEvent *_event)
                   distanceToClose)
               {
                 pf = anotherWall->GetEndPoint();
-                this->snapGrabberOther = anotherWall->grabbers[1];
-                this->snapGrabberCurrent = wallSegmentItem->grabbers[1];
+                this->snapGrabberOther = anotherWall->Grabbers()[1];
+                this->snapGrabberCurrent = wallSegmentItem->Grabbers()[1];
                 this->snapToGrabber = true;
                 break;
               }
@@ -447,12 +454,12 @@ void EditorView::mouseMoveEvent(QMouseEvent *_event)
         double distance = fabs(deltaLineMouse2) / sqrt(deltaLine2);
         if (distance > 30 || t > 1.0 || t < 0.0)
         {
-          editorItem->setParentItem(NULL);
-          wallSegmentItem->setZValue(wallSegmentItem->zValueIdle);
+          editorItem->DetachFromParent();
+          wallSegmentItem->setZValue(wallSegmentItem->ZValueIdle());
           editorItem->SetPositionOnWall(0);
           editorItem->SetAngleOnWall(0);
-          this->buildingMaker->DetachManip(this->itemToVisualMap[editorItem],
-                this->itemToVisualMap[wallSegmentItem]);
+          this->buildingMaker->DetachFromParent(
+              this->itemToVisualMap[editorItem]);
           editorItem->SetRotation(editorItem->GetRotation()
             - this->mousePressRotation);
           editorItem->SetPosition(mousePoint);
@@ -490,7 +497,7 @@ void EditorView::mouseMoveEvent(QMouseEvent *_event)
               scenePos)))
           {
             editorItem->setParentItem(wallSegmentItem);
-            wallSegmentItem->setZValue(wallSegmentItem->zValueIdle+5);
+            wallSegmentItem->setZValue(wallSegmentItem->ZValueIdle()+5);
             this->buildingMaker->AttachManip(
                 this->itemToVisualMap[editorItem],
                 this->itemToVisualMap[wallSegmentItem]);
@@ -571,7 +578,7 @@ void EditorView::mouseDoubleClickEvent(QMouseEvent *_event)
         this->currentMouseItem);
     this->itemToVisualMap.erase(wallSegmentItem);
 
-    this->UnlinkGrabbers(wallSegmentItem->grabbers[0]);
+    this->UnlinkGrabbers(wallSegmentItem->Grabbers()[0]);
 
     this->scene()->removeItem(this->currentMouseItem);
     delete this->currentMouseItem;
@@ -630,8 +637,8 @@ void EditorView::DeleteItem(EditorItem *_item)
       }
     }
 
-    this->UnlinkGrabbers(wallSegmentItem->grabbers[0]);
-    this->UnlinkGrabbers(wallSegmentItem->grabbers[1]);
+    this->UnlinkGrabbers(wallSegmentItem->Grabbers()[0]);
+    this->UnlinkGrabbers(wallSegmentItem->Grabbers()[1]);
 
     this->wallSegmentList.erase(std::remove(this->wallSegmentList.begin(),
         this->wallSegmentList.end(), dynamic_cast<WallSegmentItem *>(_item)),
@@ -700,16 +707,16 @@ void EditorView::DrawWall(const QPoint &_pos)
               <= distanceToClose)
           {
             wallSegmentItem->SetStartPoint(anotherWall->GetStartPoint());
-            this->LinkGrabbers(anotherWall->grabbers[0],
-                wallSegmentItem->grabbers[0]);
+            this->LinkGrabbers(anotherWall->Grabbers()[0],
+                wallSegmentItem->Grabbers()[0]);
             break;
           }
           else if (QVector2D(pointStart - anotherWall->GetEndPoint()).length()
               <= distanceToClose)
           {
             wallSegmentItem->SetStartPoint(anotherWall->GetEndPoint());
-            this->LinkGrabbers(anotherWall->grabbers[1],
-                wallSegmentItem->grabbers[0]);
+            this->LinkGrabbers(anotherWall->Grabbers()[1],
+                wallSegmentItem->Grabbers()[0]);
             break;
           }
         }
@@ -752,7 +759,7 @@ void EditorView::DrawWall(const QPoint &_pos)
     // Start from previous segment's end
     QPointF newPointStart = wallSegmentItem->GetEndPoint();
     QPointF newPointEnd = newPointStart + QPointF(1, 0);
-    GrabberHandle *grabberStart = wallSegmentItem->grabbers[1];
+    GrabberHandle *grabberStart = wallSegmentItem->Grabbers()[1];
 
     wallSegmentItem = new WallSegmentItem(newPointStart,
         newPointEnd, this->levelDefaultHeight);
@@ -762,7 +769,7 @@ void EditorView::DrawWall(const QPoint &_pos)
     this->scene()->addItem(wallSegmentItem);
     this->currentMouseItem = wallSegmentItem;
 
-    this->LinkGrabbers(grabberStart, wallSegmentItem->grabbers[0]);
+    this->LinkGrabbers(grabberStart, wallSegmentItem->Grabbers()[0]);
   }
 
   // 3D counterpart
@@ -1101,15 +1108,14 @@ void EditorView::OnAddLevel()
     // start / end
     for (int g = 0; g < 2; ++g)
     {
-      for (unsigned int i = 0; i < oldWall->grabbers[g]->linkedGrabbers.size();
-          ++i)
+      for (auto oldGrabber : oldWall->Grabbers()[g]->LinkedGrabbers())
       {
         WallSegmentItem *parentItem = dynamic_cast<WallSegmentItem*>(
-            oldWall->grabbers[g]->linkedGrabbers[i]->parentItem());
-        int index = oldWall->grabbers[g]->linkedGrabbers[i]->GetIndex();
+            oldGrabber->parentItem());
+        int index = oldGrabber->Index();
 
-        newWall->grabbers[g]->linkedGrabbers.push_back(
-            clonedWallMap[parentItem]->grabbers[index]);
+        newWall->Grabbers()[g]->PushLinkedGrabber(
+            clonedWallMap[parentItem]->Grabbers()[index]);
       }
     }
   }
@@ -1279,13 +1285,14 @@ void EditorView::OnOpenLevelInspector()
   FloorItem *floorItem = this->levels[this->currentLevel]->floorItem;
   if (floorItem)
   {
-    this->levelInspector->floorWidget->show();
-    this->levelInspector->SetFloorColor(floorItem->Get3dColor());
-    this->levelInspector->SetFloorTexture(floorItem->Get3dTexture());
+    this->levelInspector->ShowFloorWidget(true);
+    this->levelInspector->SetColor(Conversions::Convert(
+        floorItem->Get3dColor()));
+    this->levelInspector->SetTexture(floorItem->Get3dTexture().toStdString());
   }
   else
   {
-    this->levelInspector->floorWidget->hide();
+    this->levelInspector->ShowFloorWidget(false);
   }
   this->levelInspector->move(QCursor::pos());
   this->levelInspector->show();
@@ -1297,13 +1304,13 @@ void EditorView::OnLevelApply()
   LevelInspectorDialog *dialog =
       qobject_cast<LevelInspectorDialog *>(QObject::sender());
 
-  std::string newLevelName = dialog->GetLevelName();
+  std::string newLevelName = dialog->LevelName();
   this->levels[this->currentLevel]->name = newLevelName;
   FloorItem *floorItem = this->levels[this->currentLevel]->floorItem;
   if (floorItem)
   {
-    floorItem->Set3dTexture(dialog->GetFloorTexture());
-    floorItem->Set3dColor(dialog->GetFloorColor());
+    floorItem->Set3dTexture(QString::fromStdString(dialog->Texture()));
+    floorItem->Set3dColor(Conversions::Convert(dialog->Color()));
     floorItem->Set3dTransparency(0.4);
     floorItem->FloorChanged();
   }
@@ -1324,7 +1331,7 @@ void EditorView::CancelDrawMode()
 
       if (wallSegmentItem)
       {
-        this->UnlinkGrabbers(wallSegmentItem->grabbers[0]);
+        this->UnlinkGrabbers(wallSegmentItem->Grabbers()[0]);
       }
       this->scene()->removeItem(this->currentMouseItem);
       delete this->currentMouseItem;
@@ -1410,29 +1417,27 @@ void EditorView::LinkGrabbers(GrabberHandle *_grabber1,
   if (_grabber1 && _grabber2 && _grabber1 != _grabber2)
   {
     // if _grabber2 is not yet linked to _grabber1
-    if (std::find(_grabber1->linkedGrabbers.begin(),
-                  _grabber1->linkedGrabbers.end(), _grabber2) ==
-                  _grabber1->linkedGrabbers.end())
+    auto linked1 = _grabber1->LinkedGrabbers();
+    auto linked2 = _grabber2->LinkedGrabbers();
+    if (std::find(linked1.begin(), linked1.end(), _grabber2) == linked1.end())
     {
       // Add _grabber2 so it moves when _grabber1 is moved
-      _grabber1->linkedGrabbers.push_back(_grabber2);
+      _grabber1->PushLinkedGrabber(_grabber2);
       // also link _grabber1 to all grabbers already linked to _grabber2
-      for (unsigned int i2 = 0; i2 < _grabber2->linkedGrabbers.size(); ++i2)
+      for (auto it : linked2)
       {
-        this->LinkGrabbers(_grabber1, _grabber2->linkedGrabbers[i2]);
+        this->LinkGrabbers(_grabber1, it);
       }
     }
     // if _grabber1 is not yet linked to _grabber2
-    if (std::find(_grabber2->linkedGrabbers.begin(),
-                  _grabber2->linkedGrabbers.end(), _grabber1) ==
-                  _grabber2->linkedGrabbers.end())
+    if (std::find(linked2.begin(), linked2.end(), _grabber1) == linked2.end())
     {
       // Add _grabber1 so it moves when _grabber2 is moved
-      _grabber2->linkedGrabbers.push_back(_grabber1);
+      _grabber2->PushLinkedGrabber(_grabber1);
       // also link _grabber2 to all grabbers already linked to _grabber1
-      for (unsigned int i1 = 0; i1 < _grabber1->linkedGrabbers.size(); ++i1)
+      for (auto it : linked1)
       {
-        this->LinkGrabbers(_grabber2, _grabber1->linkedGrabbers[i1]);
+        this->LinkGrabbers(_grabber2, it);
       }
     }
   }
@@ -1445,12 +1450,9 @@ void EditorView::UnlinkGrabbers(GrabberHandle *_grabber1,
   // If only one grabber, erase it from all grabbers it is linked
   if (!_grabber2)
   {
-    for (unsigned int i = 0; i < _grabber1->linkedGrabbers.size(); ++i)
+    for (auto linked : _grabber1->LinkedGrabbers())
     {
-      _grabber1->linkedGrabbers[i]->linkedGrabbers.erase(
-          std::remove(_grabber1->linkedGrabbers[i]->linkedGrabbers.begin(),
-          _grabber1->linkedGrabbers[i]->linkedGrabbers.end(), _grabber1),
-          _grabber1->linkedGrabbers[i]->linkedGrabbers.end());
+      linked->EraseLinkedGrabber(_grabber1);
     }
   }
 
