@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Open Source Robotics Foundation
+ * Copyright (C) 2012-2016 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,6 +14,12 @@
  * limitations under the License.
  *
 */
+#ifdef _WIN32
+  // Ensure that Winsock2.h is included before Windows.h, which can get
+  // pulled in by anybody (e.g., Boost).
+  #include <Winsock2.h>
+#endif
+
 #include <boost/thread/recursive_mutex.hpp>
 #include <sstream>
 #include <limits>
@@ -83,9 +89,9 @@ void Actor::Load(sdf::ElementPtr _sdf)
     double angle = (2 * i * M_PI) / pointNum;
     double x = radius * sin(angle);
     double y = radius * cos(angle);
-    if (math::equal(x, 0.0))
+    if (ignition::math::equal(x, 0.0))
       x = 0;
-    if (math::equal(y, 0.0))
+    if (ignition::math::equal(y, 0.0))
       y = 0;
     std::cerr << x << " " << y << " 0 0 0 " << angle << "\n";
   }   */
@@ -100,19 +106,25 @@ void Actor::Load(sdf::ElementPtr _sdf)
     /// create the link sdfs for the model
     NodeMap nodes = this->skeleton->GetNodes();
 
+    /// self_collide should be added to prevent error messages
+    // _sdf->GetElement("self_collide")->Set(false);
+
     sdf::ElementPtr linkSdf;
     linkSdf = _sdf->GetElement("link");
     linkSdf->GetAttribute("name")->Set(actorName + "_pose");
     linkSdf->GetElement("gravity")->Set(false);
+    linkSdf->GetElement("self_collide")->Set(false);
     sdf::ElementPtr linkPose = linkSdf->GetElement("pose");
 
-//    this->AddSphereInertia(linkSdf, math::Pose(), 1.0, 0.01);
-//    this->AddSphereCollision(linkSdf, actorName + "_pose_col",
-//                                             math::Pose(), 0.02);
-    // this->AddBoxVisual(linkSdf, actorName + "_pose_vis", math::Pose(),
-    //                   math::Vector3(0.05, 0.05, 0.05), "Gazebo/White",
-    //                   Color::White);
-    this->AddActorVisual(linkSdf, actorName + "_visual", math::Pose());
+    // this->AddSphereInertia(linkSdf, ignition::math::Pose3d(), 1.0, 0.01);
+    // this->AddSphereCollision(linkSdf, actorName + "_pose_col",
+    //                          ignition::math::Pose3d(), 0.02);
+    // this->AddBoxVisual(linkSdf, actorName + "_pose_vis",
+    // ignition::math::Pose3d(),
+    // ignition::math::Vector3d(0.05, 0.05, 0.05), "Gazebo/White",
+    // Color::White);
+    this->AddActorVisual(linkSdf, actorName + "_visual",
+        ignition::math::Pose3d::Zero);
     std::string actorLinkName = actorName + "::" + actorName + "_pose";
     this->visualName = actorLinkName + "::"
                              + actorName + "_visual";
@@ -125,60 +137,67 @@ void Actor::Load(sdf::ElementPtr _sdf)
 
       linkSdf->GetAttribute("name")->Set(bone->GetName());
       linkSdf->GetElement("gravity")->Set(false);
+      linkSdf->GetElement("self_collide")->Set(false);
       linkPose = linkSdf->GetElement("pose");
-      math::Pose pose(bone->GetModelTransform().GetTranslation(),
-                      bone->GetModelTransform().GetRotation());
+      ignition::math::Pose3d pose(bone->ModelTransform().Translation(),
+                                  bone->ModelTransform().Rotation());
       if (bone->IsRootNode())
-        pose = math::Pose();
+        pose = ignition::math::Pose3d::Zero;
       linkPose->Set(pose);
 
       /// FIXME hardcoded inertia of a sphere with mass 1.0 and radius 0.01
-      this->AddSphereInertia(linkSdf, math::Pose(), 1.0, 0.01);
+      this->AddSphereInertia(linkSdf, ignition::math::Pose3d::Zero, 1.0, 0.01);
 
       /// FIXME hardcoded collision to sphere with radius 0.02
       this->AddSphereCollision(linkSdf, bone->GetName() + "_collision",
-                       math::Pose(), 0.02);
+                       ignition::math::Pose3d::Zero, 0.02);
 
       /// FIXME hardcoded visual to red sphere with radius 0.02
       if (bone->IsRootNode())
       {
         this->AddSphereVisual(linkSdf, bone->GetName() + "__SKELETON_VISUAL__",
-                            math::Pose(), 0.02, "Gazebo/Blue", Color::Blue);
+            ignition::math::Pose3d::Zero, 0.02, "Gazebo/Blue", Color::Blue);
       }
       else
         if (bone->GetChildCount() == 0)
         {
-            this->AddSphereVisual(linkSdf, bone->GetName() +
-                            "__SKELETON_VISUAL__", math::Pose(), 0.02,
-                            "Gazebo/Yellow", Color::Yellow);
+          this->AddSphereVisual(linkSdf, bone->GetName() +
+              "__SKELETON_VISUAL__", ignition::math::Pose3d::Zero, 0.02,
+              "Gazebo/Yellow", Color::Yellow);
         }
         else
+        {
           this->AddSphereVisual(linkSdf, bone->GetName() +
-                            "__SKELETON_VISUAL__", math::Pose(), 0.02,
-                            "Gazebo/Red", Color::Red);
+              "__SKELETON_VISUAL__", ignition::math::Pose3d::Zero, 0.02,
+              "Gazebo/Red", Color::Red);
+        }
 
-      for (unsigned int i = 0; i < bone->GetChildCount(); i++)
+      for (unsigned int i = 0; i < bone->GetChildCount(); ++i)
       {
         SkeletonNode *curChild = bone->GetChild(i);
 
-        math::Vector3 dir = curChild->GetModelTransform().GetTranslation() -
-            bone->GetModelTransform().GetTranslation();
-        double length = dir.GetLength();
+        ignition::math::Vector3d dir =
+          curChild->ModelTransform().Translation() -
+          bone->ModelTransform().Translation();
+        double length = dir.Length();
 
-        if (!math::equal(length, 0.0))
+        if (!ignition::math::equal(length, 0.0))
         {
-          math::Vector3 r = curChild->GetTransform().GetTranslation();
-          math::Vector3 linkPos = math::Vector3(r.x / 2.0,
-                                    r.y / 2.0, r.z / 2.0);
-          double theta = atan2(dir.y, dir.x);
-          double phi = acos(dir.z / length);
+          ignition::math::Vector3d r =
+            curChild->Transform().Translation();
+          ignition::math::Vector3d linkPos =
+            ignition::math::Vector3d(r.X() / 2.0, r.Y() / 2.0, r.Z() / 2.0);
+          double theta = atan2(dir.Y(), dir.X());
+          double phi = acos(dir.Z() / length);
 
-          math::Pose bonePose(linkPos, math::Quaternion(0.0, phi, theta));
-          bonePose.rot = pose.rot.GetInverse() * bonePose.rot;
+          ignition::math::Pose3d bonePose(linkPos,
+              ignition::math::Quaterniond(0.0, phi, theta));
+          bonePose.Rot() = pose.Rot().Inverse() * bonePose.Rot();
 
           this->AddBoxVisual(linkSdf, bone->GetName() + "_" +
             curChild->GetName() + "__SKELETON_VISUAL__", bonePose,
-            math::Vector3(0.02, 0.02, length), "Gazebo/Green", Color::Green);
+            ignition::math::Vector3d(0.02, 0.02, length),
+            "Gazebo/Green", Color::Green);
         }
       }
     }
@@ -249,7 +268,7 @@ void Actor::LoadScript(sdf::ElementPtr _sdf)
       unsigned int idx = iter - this->trajInfo.begin();
       this->trajInfo.insert(iter, tinfo);
 
-      std::map<double, math::Pose> points;
+      std::map<double, ignition::math::Pose3d> points;
 
       if (trajSdf->HasElement("waypoint"))
       {
@@ -257,11 +276,12 @@ void Actor::LoadScript(sdf::ElementPtr _sdf)
         while (wayptSdf)
         {
           points[wayptSdf->Get<double>("time")] =
-                                          wayptSdf->Get<math::Pose>("pose");
+            wayptSdf->Get<ignition::math::Pose3d>("pose");
           wayptSdf = wayptSdf->GetNextElement("waypoint");
         }
 
-        std::map<double, math::Pose>::reverse_iterator last = points.rbegin();
+        std::map<double, ignition::math::Pose3d>::reverse_iterator last =
+          points.rbegin();
         std::stringstream animName;
         animName << tinfo.type << "_" << tinfo.id;
         common::PoseAnimation *anim = new common::PoseAnimation(animName.str(),
@@ -269,19 +289,20 @@ void Actor::LoadScript(sdf::ElementPtr _sdf)
         this->trajInfo[idx].duration = last->first;
         this->trajInfo[idx].translated = true;
 
-        for (std::map<double, math::Pose>::iterator pIter = points.begin();
-              pIter != points.end(); ++pIter)
+        for (std::map<double, ignition::math::Pose3d>::iterator pIter =
+            points.begin(); pIter != points.end(); ++pIter)
         {
           common::PoseKeyFrame *key;
-          if (pIter == points.begin() && !math::equal(pIter->first, 0.0))
+          if (pIter == points.begin() &&
+              !ignition::math::equal(pIter->first, 0.0))
           {
             key = anim->CreateKeyFrame(0.0);
-            key->SetTranslation(pIter->second.pos);
-            key->SetRotation(pIter->second.rot);
+            key->Translation(pIter->second.Pos());
+            key->Rotation(pIter->second.Rot());
           }
           key = anim->CreateKeyFrame(pIter->first);
-          key->SetTranslation(pIter->second.pos);
-          key->SetRotation(pIter->second.rot);
+          key->Translation(pIter->second.Pos());
+          key->Rotation(pIter->second.Rot());
         }
 
         this->trajectories[this->trajInfo[idx].id] = anim;
@@ -331,7 +352,7 @@ void Actor::LoadAnimation(sdf::ElementPtr _sdf)
     this->skelAnimation[this->skinFile] =
         this->skeleton->GetAnimation(0);
     std::map<std::string, std::string> skelMap;
-    for (unsigned int i = 0; i < this->skeleton->GetNumNodes(); i++)
+    for (unsigned int i = 0; i < this->skeleton->GetNumNodes(); ++i)
       skelMap[this->skeleton->GetNodeByHandle(i)->GetName()] =
         this->skeleton->GetNodeByHandle(i)->GetName();
     this->skelNodesMap[this->skinFile] = skelMap;
@@ -373,7 +394,7 @@ void Actor::LoadAnimation(sdf::ElementPtr _sdf)
       if (this->skeleton->GetNumNodes() != skel->GetNumNodes())
         compatible = false;
       else
-        for (unsigned int i = 0; i < this->skeleton->GetNumNodes(); i++)
+        for (unsigned int i = 0; i < this->skeleton->GetNumNodes(); ++i)
         {
           SkeletonNode *skinNode = this->skeleton->GetNodeByHandle(i);
           SkeletonNode *animNode = skel->GetNodeByHandle(i);
@@ -470,7 +491,7 @@ void Actor::Update()
 
   TrajectoryInfo tinfo;
 
-  for (unsigned int i = 0; i < this->trajInfo.size(); i++)
+  for (unsigned int i = 0; i < this->trajInfo.size(); ++i)
     if (this->trajInfo[i].startTime <= scriptTime &&
           this->trajInfo[i].endTime >= scriptTime)
     {
@@ -483,51 +504,54 @@ void Actor::Update()
   SkeletonAnimation *skelAnim = this->skelAnimation[tinfo.type];
   std::map<std::string, std::string> skelMap = this->skelNodesMap[tinfo.type];
 
-  math::Pose modelPose;
-  std::map<std::string, math::Matrix4> frame;
+  ignition::math::Pose3d modelPose;
+  std::map<std::string, ignition::math::Matrix4d> frame;
   if (this->trajectories.find(tinfo.id) != this->trajectories.end())
   {
     common::PoseKeyFrame posFrame(0.0);
     this->trajectories[tinfo.id]->SetTime(scriptTime);
     this->trajectories[tinfo.id]->GetInterpolatedKeyFrame(posFrame);
-    modelPose.pos = posFrame.GetTranslation();
-    modelPose.rot = posFrame.GetRotation();
+
+    modelPose.Pos() = posFrame.Translation();
+    modelPose.Rot() = posFrame.Rotation();
 
     if (this->lastTraj == tinfo.id)
-      this->pathLength += fabs(this->lastPos.Distance(modelPose.pos));
+      this->pathLength += fabs(this->lastPos.Distance(modelPose.Pos()));
     else
     {
       common::PoseKeyFrame *frame0 = dynamic_cast<common::PoseKeyFrame*>
         (this->trajectories[tinfo.id]->GetKeyFrame(0));
-      this->pathLength = fabs(modelPose.pos.Distance(frame0->GetTranslation()));
+      ignition::math::Vector3d vector3Ign;
+      vector3Ign = frame0->Translation();
+      this->pathLength = fabs(modelPose.Pos().Distance(vector3Ign));
     }
-    this->lastPos = modelPose.pos;
+    this->lastPos = modelPose.Pos();
   }
   if (this->interpolateX[tinfo.type] &&
         this->trajectories.find(tinfo.id) != this->trajectories.end())
   {
-    frame = skelAnim->GetPoseAtX(this->pathLength,
+    frame = skelAnim->PoseAtX(this->pathLength,
               skelMap[this->skeleton->GetRootNode()->GetName()]);
   }
   else
-    frame = skelAnim->GetPoseAt(scriptTime);
+    frame = skelAnim->PoseAt(scriptTime);
 
   this->lastTraj = tinfo.id;
 
-  math::Matrix4 rootTrans =
-                  frame[skelMap[this->skeleton->GetRootNode()->GetName()]];
+  ignition::math::Matrix4d rootTrans =
+    frame[skelMap[this->skeleton->GetRootNode()->GetName()]];
 
-  math::Vector3 rootPos = rootTrans.GetTranslation();
-  math::Quaternion rootRot = rootTrans.GetRotation();
+  ignition::math::Vector3d rootPos = rootTrans.Translation();
+  ignition::math::Quaterniond rootRot = rootTrans.Rotation();
 
   if (tinfo.translated)
-    rootPos.x = 0.0;
-  math::Pose actorPose;
-  actorPose.pos = modelPose.pos + modelPose.rot.RotateVector(rootPos);
-  actorPose.rot = modelPose.rot *rootRot;
+    rootPos.X() = 0.0;
+  ignition::math::Pose3d actorPose;
+  actorPose.Pos() = modelPose.Pos() + modelPose.Rot().RotateVector(rootPos);
+  actorPose.Rot() = modelPose.Rot() * rootRot;
 
-  math::Matrix4 rootM(actorPose.rot.GetAsMatrix4());
-  rootM.SetTranslate(actorPose.pos);
+  ignition::math::Matrix4d rootM(actorPose.Rot());
+  rootM.Translate(actorPose.Pos());
 
   frame[skelMap[this->skeleton->GetRootNode()->GetName()]] = rootM;
 
@@ -537,28 +561,28 @@ void Actor::Update()
 }
 
 //////////////////////////////////////////////////
-void Actor::SetPose(std::map<std::string, math::Matrix4> _frame,
+void Actor::SetPose(std::map<std::string, ignition::math::Matrix4d> _frame,
       std::map<std::string, std::string> _skelMap, double _time)
 {
   msgs::PoseAnimation msg;
   msg.set_model_name(this->visualName);
   msg.set_model_id(this->visualId);
 
-  math::Matrix4 modelTrans(math::Matrix4::IDENTITY);
-  math::Pose mainLinkPose;
+  ignition::math::Matrix4d modelTrans(ignition::math::Matrix4d::Identity);
+  ignition::math::Pose3d mainLinkPose;
 
-  for (unsigned int i = 0; i < this->skeleton->GetNumNodes(); i++)
+  for (unsigned int i = 0; i < this->skeleton->GetNumNodes(); ++i)
   {
     SkeletonNode *bone = this->skeleton->GetNodeByHandle(i);
     SkeletonNode *parentBone = bone->GetParent();
-    math::Matrix4 transform(math::Matrix4::IDENTITY);
+    ignition::math::Matrix4d transform(ignition::math::Matrix4d::Identity);
     if (_frame.find(_skelMap[bone->GetName()]) != _frame.end())
       transform = _frame[_skelMap[bone->GetName()]];
     else
-      transform = bone->GetTransform();
+      transform = bone->Transform();
 
     LinkPtr currentLink = this->GetChildLink(bone->GetName());
-    math::Pose bonePose = transform.GetAsPose();
+    ignition::math::Pose3d bonePose = transform.Pose();
 
     if (!bonePose.IsFinite())
     {
@@ -572,29 +596,30 @@ void Actor::SetPose(std::map<std::string, math::Matrix4> _frame,
 
     if (!parentBone)
     {
-      bone_pose->mutable_position()->CopyFrom(msgs::Convert(math::Vector3()));
+      bone_pose->mutable_position()->CopyFrom(
+          msgs::Convert(ignition::math::Vector3d()));
       bone_pose->mutable_orientation()->CopyFrom(msgs::Convert(
-                                                    math::Quaternion()));
+            ignition::math::Quaterniond()));
       mainLinkPose = bonePose;
     }
     else
     {
-      bone_pose->mutable_position()->CopyFrom(msgs::Convert(bonePose.pos));
-      bone_pose->mutable_orientation()->CopyFrom(msgs::Convert(bonePose.rot));
+      bone_pose->mutable_position()->CopyFrom(msgs::Convert(bonePose.Pos()));
+      bone_pose->mutable_orientation()->CopyFrom(msgs::Convert(bonePose.Rot()));
       LinkPtr parentLink = this->GetChildLink(parentBone->GetName());
       math::Pose parentPose = parentLink->GetWorldPose();
       math::Matrix4 parentTrans(parentPose.rot.GetAsMatrix4());
       parentTrans.SetTranslate(parentPose.pos);
-      transform = parentTrans * transform;
+      transform = (parentTrans * transform).Ign();
     }
 
     msgs::Pose *link_pose = msg.add_pose();
     link_pose->set_name(currentLink->GetScopedName());
     link_pose->set_id(currentLink->GetId());
-    math::Pose linkPose = transform.GetAsPose() - mainLinkPose;
-    link_pose->mutable_position()->CopyFrom(msgs::Convert(linkPose.pos));
-    link_pose->mutable_orientation()->CopyFrom(msgs::Convert(linkPose.rot));
-    currentLink->SetWorldPose(transform.GetAsPose(), true, false);
+    ignition::math::Pose3d linkPose = transform.Pose() - mainLinkPose;
+    link_pose->mutable_position()->CopyFrom(msgs::Convert(linkPose.Pos()));
+    link_pose->mutable_orientation()->CopyFrom(msgs::Convert(linkPose.Rot()));
+    currentLink->SetWorldPose(transform.Pose(), true, false);
   }
 
   msgs::Time *stamp = msg.add_time();
@@ -603,8 +628,9 @@ void Actor::SetPose(std::map<std::string, math::Matrix4> _frame,
   msgs::Pose *model_pose = msg.add_pose();
   model_pose->set_name(this->GetScopedName());
   model_pose->set_id(this->GetId());
-  model_pose->mutable_position()->CopyFrom(msgs::Convert(mainLinkPose.pos));
-  model_pose->mutable_orientation()->CopyFrom(msgs::Convert(mainLinkPose.rot));
+  model_pose->mutable_position()->CopyFrom(msgs::Convert(mainLinkPose.Pos()));
+  model_pose->mutable_orientation()->CopyFrom(
+      msgs::Convert(mainLinkPose.Rot()));
 
   if (this->bonePosePub && this->bonePosePub->HasConnections())
     this->bonePosePub->Publish(msg);
@@ -630,9 +656,9 @@ const sdf::ElementPtr Actor::GetSDF()
 }
 
 //////////////////////////////////////////////////
-void Actor::AddSphereInertia(sdf::ElementPtr _linkSdf,
-                             const math::Pose &_pose,
-                             double _mass, double _radius)
+void Actor::AddSphereInertia(const sdf::ElementPtr &_linkSdf,
+                             const ignition::math::Pose3d &_pose,
+                             const double _mass, const double _radius)
 {
   double ixx = 2.0 * _mass * _radius * _radius / 5.0;
   sdf::ElementPtr inertialSdf = _linkSdf->GetElement("inertial");
@@ -649,10 +675,10 @@ void Actor::AddSphereInertia(sdf::ElementPtr _linkSdf,
 }
 
 //////////////////////////////////////////////////
-void Actor::AddSphereCollision(sdf::ElementPtr _linkSdf,
+void Actor::AddSphereCollision(const sdf::ElementPtr &_linkSdf,
                                const std::string &_name,
-                               const math::Pose &_pose,
-                               double _radius)
+                               const ignition::math::Pose3d &_pose,
+                               const double _radius)
 {
   sdf::ElementPtr collisionSdf = _linkSdf->GetElement("collision");
   collisionSdf->GetAttribute("name")->Set(_name);
@@ -664,9 +690,10 @@ void Actor::AddSphereCollision(sdf::ElementPtr _linkSdf,
 }
 
 //////////////////////////////////////////////////
-void Actor::AddSphereVisual(sdf::ElementPtr _linkSdf, const std::string &_name,
-            const math::Pose &_pose, double _radius,
-            const std::string &_material, const common::Color &_ambient)
+void Actor::AddSphereVisual(const sdf::ElementPtr &_linkSdf,
+    const std::string &_name, const ignition::math::Pose3d &_pose,
+    const double _radius, const std::string &_material,
+    const common::Color &_ambient)
 {
   sdf::ElementPtr visualSdf = _linkSdf->GetElement("visual");
   visualSdf->GetAttribute("name")->Set(_name);
@@ -682,9 +709,10 @@ void Actor::AddSphereVisual(sdf::ElementPtr _linkSdf, const std::string &_name,
 }
 
 //////////////////////////////////////////////////
-void Actor::AddBoxVisual(sdf::ElementPtr _linkSdf, const std::string &_name,
-    const math::Pose &_pose, const math::Vector3 &_size,
-    const std::string &_material, const common::Color &_ambient)
+void Actor::AddBoxVisual(const sdf::ElementPtr &_linkSdf,
+    const std::string &_name, const ignition::math::Pose3d &_pose,
+    const ignition::math::Vector3d &_size, const std::string &_material,
+    const common::Color &_ambient)
 {
   sdf::ElementPtr visualSdf = _linkSdf->AddElement("visual");
   visualSdf->GetAttribute("name")->Set(_name);
@@ -700,8 +728,8 @@ void Actor::AddBoxVisual(sdf::ElementPtr _linkSdf, const std::string &_name,
 }
 
 //////////////////////////////////////////////////
-void Actor::AddActorVisual(sdf::ElementPtr _linkSdf, const std::string &_name,
-                           const math::Pose &_pose)
+void Actor::AddActorVisual(const sdf::ElementPtr &_linkSdf,
+    const std::string &_name, const ignition::math::Pose3d &_pose)
 {
   sdf::ElementPtr visualSdf = _linkSdf->AddElement("visual");
   visualSdf->GetAttribute("name")->Set(_name);
