@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Open Source Robotics Foundation
+ * Copyright (C) 2012-2016 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -19,6 +19,14 @@
  * Date: 03 Apr 2007
  */
 
+#ifdef _WIN32
+  // Ensure that Winsock2.h is included before Windows.h, which can get
+  // pulled in by anybody (e.g., Boost).
+  #include <Winsock2.h>
+#endif
+
+#include <boost/bind.hpp>
+#include <boost/function.hpp>
 #include <boost/thread/recursive_mutex.hpp>
 
 #include "gazebo/msgs/msgs.hh"
@@ -72,7 +80,7 @@ Entity::Entity(BasePtr _parent)
 
   this->setWorldPoseFunc = &Entity::SetWorldPoseDefault;
 
-  this->scale = math::Vector3::One;
+  this->scale = ignition::math::Vector3d::One;
 }
 
 //////////////////////////////////////////////////
@@ -123,7 +131,14 @@ void Entity::Load(sdf::ElementPtr _sdf)
     this->visualMsg->set_parent_name(this->world->GetName());
     this->visualMsg->set_parent_id(0);
   }
-  msgs::Set(this->visualMsg->mutable_pose(), this->GetRelativePose());
+  msgs::Set(this->visualMsg->mutable_pose(), this->GetRelativePose().Ign());
+
+  if (this->HasType(Base::MODEL))
+    this->visualMsg->set_type(msgs::Visual::MODEL);
+  if (this->HasType(Base::LINK))
+    this->visualMsg->set_type(msgs::Visual::LINK);
+  if (this->HasType(Base::COLLISION))
+    this->visualMsg->set_type(msgs::Visual::COLLISION);
 
   this->visPub->Publish(*this->visualMsg);
 
@@ -322,29 +337,44 @@ void Entity::SetWorldPoseModel(const math::Pose &_pose, bool _notify,
     {
       EntityPtr entity = boost::static_pointer_cast<Entity>(*iter);
 
-      if (entity->IsCanonicalLink())
-        entity->worldPose = (entity->initialRelativePose + _pose);
+      if (entity->HasType(LINK))
+      {
+        if (entity->IsCanonicalLink())
+          entity->worldPose = (entity->initialRelativePose + _pose);
+        else
+        {
+          entity->worldPose = ((entity->worldPose - oldModelWorldPose) + _pose);
+          if (_publish)
+            entity->PublishPose();
+        }
+
+        if (_notify)
+          entity->UpdatePhysicsPose(false);
+
+        // Tell collisions that their current world pose is dirty (needs
+        // updating). We set a dirty flag instead of directly updating the
+        // value to improve performance.
+        for (Base_V::iterator iterC = (*iter)->children.begin();
+             iterC != (*iter)->children.end(); ++iterC)
+        {
+          if ((*iterC)->HasType(COLLISION))
+          {
+            CollisionPtr entityC =
+                boost::static_pointer_cast<Collision>(*iterC);
+            entityC->SetWorldPoseDirty();
+          }
+        }
+      }
+      else if (entity->HasType(MODEL))
+      {
+        // set pose of nested models
+        entity->SetWorldPoseModel(
+            (entity->worldPose - oldModelWorldPose) + _pose, _notify, _publish);
+      }
       else
       {
-        entity->worldPose = ((entity->worldPose - oldModelWorldPose) + _pose);
-        if (_publish)
-          entity->PublishPose();
-      }
-
-      if (_notify)
-        entity->UpdatePhysicsPose(false);
-
-      // Tell collisions that their current world pose is dirty (needs
-      // updating). We set a dirty flag instead of directly updating the
-      // value to improve performance.
-      for (Base_V::iterator iterC = (*iter)->children.begin();
-           iterC != (*iter)->children.end(); ++iterC)
-      {
-        if ((*iterC)->HasType(COLLISION))
-        {
-          CollisionPtr entityC = boost::static_pointer_cast<Collision>(*iterC);
-          entityC->SetWorldPoseDirty();
-        }
+        gzerr << "SetWorldPoseModel error: unknown type of entity in Model."
+          << std::endl;
       }
     }
   }
@@ -360,37 +390,51 @@ void Entity::SetWorldPoseCanonicalLink(const math::Pose &_pose, bool _notify,
   if (_notify)
     this->UpdatePhysicsPose(true);
 
-  // also update parent model's pose
-  if (this->parentEntity->HasType(MODEL))
+  if (!this->parentEntity->HasType(MODEL))
+  {
+    gzerr << "SetWorldPose for Canonical Body [" << this->GetName()
+        << "] but parent[" << this->parentEntity->GetName()
+        << "] is not a MODEL!" << std::endl;
+    return;
+  }
+
+  EntityPtr parentEnt = this->parentEntity;
+  ignition::math::Pose3d relativePose = this->initialRelativePose.Ign();
+  math::Pose updatePose = _pose;
+
+  // recursively update parent model pose based on new canonical link pose
+  while (parentEnt && parentEnt->HasType(MODEL))
   {
     // setting parent Model world pose from canonical link world pose
     // where _pose is the canonical link's world pose
-    this->parentEntity->worldPose = (-this->initialRelativePose) + _pose;
+    parentEnt->worldPose = math::Pose(-relativePose) + updatePose;
 
-    this->parentEntity->worldPose.Correct();
+    parentEnt->worldPose.Correct();
 
     if (_notify)
-      this->parentEntity->UpdatePhysicsPose(false);
+      parentEnt->UpdatePhysicsPose(false);
 
     if (_publish)
       this->parentEntity->PublishPose();
 
-    // Tell collisions that their current world pose is dirty (needs
-    // updating). We set a dirty flag instead of directly updating the
-    // value to improve performance.
-    for (Base_V::iterator iterC = this->children.begin();
-        iterC != this->children.end(); ++iterC)
+    updatePose = parentEnt->worldPose;
+    relativePose = parentEnt->GetInitialRelativePose().Ign();
+
+    parentEnt = boost::dynamic_pointer_cast<Entity>(parentEnt->GetParent());
+  }
+
+  // Tell collisions that their current world pose is dirty (needs
+  // updating). We set a dirty flag instead of directly updating the
+  // value to improve performance.
+  for (Base_V::iterator iterC = this->children.begin();
+      iterC != this->children.end(); ++iterC)
+  {
+    if ((*iterC)->HasType(COLLISION))
     {
-      if ((*iterC)->HasType(COLLISION))
-      {
-        CollisionPtr entityC = boost::static_pointer_cast<Collision>(*iterC);
-        entityC->SetWorldPoseDirty();
-      }
+      CollisionPtr entityC = boost::static_pointer_cast<Collision>(*iterC);
+      entityC->SetWorldPoseDirty();
     }
   }
-  else
-    gzerr << "SWP for CB[" << this->GetName() << "] but parent["
-      << this->parentEntity->GetName() << "] is not a MODEL!\n";
 }
 
 //////////////////////////////////////////////////
@@ -532,7 +576,7 @@ void Entity::OnPoseMsg(ConstPosePtr &_msg)
 {
   if (_msg->name() == this->GetScopedName())
   {
-    math::Pose p = msgs::Convert(*_msg);
+    ignition::math::Pose3d p = msgs::ConvertIgn(*_msg);
     this->SetWorldPose(p);
   }
 }
@@ -557,10 +601,7 @@ void Entity::Fini()
 //////////////////////////////////////////////////
 void Entity::Reset()
 {
-  if (this->HasType(Base::MODEL))
-    this->SetWorldPose(this->initialRelativePose);
-  else
-    this->SetRelativePose(this->initialRelativePose);
+  this->SetRelativePose(this->initialRelativePose);
 }
 
 //////////////////////////////////////////////////
@@ -588,8 +629,8 @@ void Entity::UpdateAnimation(const common::UpdateInfo &_info)
   this->animation->GetInterpolatedKeyFrame(kf);
 
   math::Pose offset;
-  offset.pos = kf.GetTranslation();
-  offset.rot = kf.GetRotation();
+  offset.pos = kf.Translation();
+  offset.rot = kf.Rotation();
 
   this->SetWorldPose(offset);
   this->prevAnimationTime = _info.simTime;
