@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Open Source Robotics Foundation
+ * Copyright (C) 2012-2016 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,7 +15,16 @@
  *
 */
 
+#ifdef _WIN32
+  // Ensure that Winsock2.h is included before Windows.h, which can get
+  // pulled in by anybody (e.g., Boost).
+  #include <Winsock2.h>
+#endif
+
+#include <boost/algorithm/string.hpp>
+#include <boost/bind.hpp>
 #include <sstream>
+#include <functional>
 
 #include "gazebo/msgs/msgs.hh"
 
@@ -29,9 +38,7 @@
 #include "gazebo/common/Console.hh"
 #include "gazebo/common/Exception.hh"
 #include "gazebo/common/Assert.hh"
-
-#include "gazebo/sensors/SensorsIface.hh"
-#include "gazebo/sensors/Sensor.hh"
+#include "gazebo/common/Battery.hh"
 
 #include "gazebo/physics/PhysicsIface.hh"
 #include "gazebo/physics/Model.hh"
@@ -56,7 +63,6 @@ Link::Link(EntityPtr _parent)
   this->publishDataMutex = new boost::recursive_mutex();
 }
 
-
 //////////////////////////////////////////////////
 Link::~Link()
 {
@@ -104,26 +110,33 @@ Link::~Link()
 
   this->requestPub.reset();
   this->dataPub.reset();
+  this->wrenchSub.reset();
   this->connections.clear();
 
   delete this->publishDataMutex;
   this->publishDataMutex = NULL;
 
   this->collisions.clear();
+  this->batteries.clear();
 }
 
 //////////////////////////////////////////////////
 void Link::Load(sdf::ElementPtr _sdf)
 {
-  bool needUpdate = false;
-
   Entity::Load(_sdf);
 
-  // before loading child collsion, we have to figure out of selfCollide is true
-  // and modify parent class Entity so this body has its own spaceId
-  this->SetSelfCollide(this->sdf->Get<bool>("self_collide"));
+  // before loading child collision, we have to figure out if selfCollide is
+  // true and modify parent class Entity so this body has its own spaceId
+  if (this->sdf->HasElement("self_collide"))
+  {
+    this->SetSelfCollide(this->sdf->Get<bool>("self_collide"));
+  }
+  else
+  {
+    this->SetSelfCollide(this->GetModel()->GetSelfCollide());
+  }
   this->sdf->GetElement("self_collide")->GetValue()->SetUpdateFunc(
-      boost::bind(&Link::GetSelfCollide, this));
+      std::bind(&Link::GetSelfCollide, this));
 
   // Parse visuals from SDF
   this->ParseVisuals();
@@ -154,9 +167,14 @@ void Link::Load(sdf::ElementPtr _sdf)
       }
       else if (sensorElem->Get<std::string>("type") != "__default__")
       {
-        std::string sensorName =
-          sensors::create_sensor(sensorElem, this->GetWorld()->GetName(),
-              this->GetScopedName(), this->GetId());
+        // This must match the implementation in Sensors::GetScopedName
+        std::string sensorName = this->GetScopedName(true) + "::" +
+          sensorElem->Get<std::string>("name");
+
+        // Tell the sensor library to create a sensor.
+        event::Events::createSensor(sensorElem,
+            this->GetWorld()->GetName(), this->GetScopedName(), this->GetId());
+
         this->sensors.push_back(sensorName);
       }
       sensorElem = sensorElem->GetNextElement("sensor");
@@ -180,7 +198,7 @@ void Link::Load(sdf::ElementPtr _sdf)
       util::OpenALSourcePtr source = util::OpenAL::Instance()->CreateSource(
           audioElem);
 
-      std::vector<std::string> names = source->GetCollisionNames();
+      std::vector<std::string> names = source->CollisionNames();
       std::copy(names.begin(), names.end(), std::back_inserter(collisionNames));
 
       audioElem = audioElem->GetNextElement("audio_source");
@@ -201,21 +219,31 @@ void Link::Load(sdf::ElementPtr _sdf)
       this->audioContactsSub = this->node->Subscribe(topic,
           &Link::OnCollision, this);
     }
-
-    needUpdate = true;
   }
 
   if (_sdf->HasElement("audio_sink"))
   {
-    needUpdate = true;
     this->audioSink = util::OpenAL::Instance()->CreateSink(
         _sdf->GetElement("audio_sink"));
   }
 #endif
 
-  if (needUpdate)
-    this->connections.push_back(event::Events::ConnectWorldUpdateBegin(
-          boost::bind(&Link::Update, this, _1)));
+  if (this->sdf->HasElement("battery"))
+  {
+    sdf::ElementPtr batteryElem = this->sdf->GetElement("battery");
+    while (batteryElem)
+    {
+      this->LoadBattery(batteryElem);
+      batteryElem = batteryElem->GetNextElement("battery");
+    }
+  }
+
+  this->connections.push_back(event::Events::ConnectWorldUpdateBegin(
+      boost::bind(&Link::Update, this, _1)));
+
+  std::string topicName = "~/" + this->GetScopedName() + "/wrench";
+  boost::replace_all(topicName, "::", "/");
+  this->wrenchSub = this->node->Subscribe(topicName, &Link::OnWrenchMsg, this);
 }
 
 //////////////////////////////////////////////////
@@ -242,6 +270,12 @@ void Link::Init()
     }
   }
 
+  // Initialize all the batteries
+  for (auto &battery : this->batteries)
+  {
+    battery->Init();
+  }
+
   this->initialized = true;
 }
 
@@ -252,12 +286,14 @@ void Link::Fini()
   this->childJoints.clear();
   this->collisions.clear();
   this->inertial.reset();
+  this->batteries.clear();
 
-  for (std::vector<std::string>::iterator iter = this->sensors.begin();
-       iter != this->sensors.end(); ++iter)
+  // Remove all the sensors attached to the link
+  for (auto const &sensor : this->sensors)
   {
-    sensors::remove_sensor(*iter);
+    event::Events::removeSensor(sensor);
   }
+
   this->sensors.clear();
 
   for (Visuals_M::iterator iter = this->visuals.begin();
@@ -324,7 +360,7 @@ void Link::UpdateParameters(sdf::ElementPtr _sdf)
   if (this->sdf->Get<bool>("gravity") != this->GetGravityMode())
     this->SetGravityMode(this->sdf->Get<bool>("gravity"));
 
-  // before loading child collsiion, we have to figure out if
+  // before loading child collision, we have to figure out if
   // selfCollide is true and modify parent class Entity so this
   // body has its own spaceId
   this->SetSelfCollide(this->sdf->Get<bool>("self_collide"));
@@ -341,6 +377,7 @@ void Link::UpdateParameters(sdf::ElementPtr _sdf)
       msg.set_name(this->GetScopedName() + "::" + msg.name());
       msg.set_parent_name(this->GetScopedName());
       msg.set_is_static(this->IsStatic());
+      msg.set_type(msgs::Visual::VISUAL);
 
       this->visPub->Publish(msg);
 
@@ -361,6 +398,21 @@ void Link::UpdateParameters(sdf::ElementPtr _sdf)
       collisionElem = collisionElem->GetNextElement("collision");
     }
   }
+
+  // Update the battery information
+  if (this->sdf->HasElement("battery"))
+  {
+    sdf::ElementPtr batteryElem = this->sdf->GetElement("battery");
+    while (batteryElem)
+    {
+      common::BatteryPtr battery = this->Battery(
+          batteryElem->Get<std::string>("name"));
+
+      if (battery)
+        battery->UpdateParameters(batteryElem);
+      batteryElem = batteryElem->GetNextElement("battery");
+    }
+  }
 }
 
 //////////////////////////////////////////////////
@@ -437,16 +489,16 @@ void Link::Update(const common::UpdateInfo & /*_info*/)
 #ifdef HAVE_OPENAL
   if (this->audioSink)
   {
-    this->audioSink->SetPose(this->GetWorldPose());
-    this->audioSink->SetVelocity(this->GetWorldLinearVel());
+    this->audioSink->SetPose(this->GetWorldPose().Ign());
+    this->audioSink->SetVelocity(this->GetWorldLinearVel().Ign());
   }
 
   // Update all the audio sources
   for (std::vector<util::OpenALSourcePtr>::iterator iter =
       this->audioSources.begin(); iter != this->audioSources.end(); ++iter)
   {
-    (*iter)->SetPose(this->GetWorldPose());
-    (*iter)->SetVelocity(this->GetWorldLinearVel());
+    (*iter)->SetPose(this->GetWorldPose().Ign());
+    (*iter)->SetVelocity(this->GetWorldLinearVel().Ign());
   }
 #endif
 
@@ -456,6 +508,27 @@ void Link::Update(const common::UpdateInfo & /*_info*/)
      this->enabled = this->GetEnabled();
      this->enabledSignal(this->enabled);
    }*/
+
+  if (!this->wrenchMsgs.empty())
+  {
+    std::vector<msgs::Wrench> messages;
+    {
+      boost::mutex::scoped_lock lock(this->wrenchMsgMutex);
+      messages = this->wrenchMsgs;
+      this->wrenchMsgs.clear();
+    }
+
+    for (auto it : messages)
+    {
+      this->ProcessWrenchMsg(it);
+    }
+  }
+
+  // Update the batteries.
+  for (auto &battery : this->batteries)
+  {
+    battery->Update();
+  }
 }
 
 /////////////////////////////////////////////////
@@ -609,13 +682,26 @@ math::Vector3 Link::GetWorldLinearAccel() const
 //////////////////////////////////////////////////
 math::Vector3 Link::GetRelativeAngularAccel() const
 {
-  return this->GetRelativeTorque() / this->inertial->GetMass();
+  return this->GetWorldPose().rot.RotateVectorReverse(
+    this->GetWorldAngularAccel());
 }
 
 //////////////////////////////////////////////////
 math::Vector3 Link::GetWorldAngularAccel() const
 {
-  return this->GetWorldTorque() / this->inertial->GetMass();
+  // I: inertia matrix in world frame
+  // T: sum of external torques in world frame
+  // L: angular momentum of CoG in world frame
+  // w: angular velocity in world frame
+  // return I^-1 * (T - w x L)
+  return this->GetWorldInertiaMatrix().Inverse() * (this->GetWorldTorque()
+    - this->GetWorldAngularVel().Cross(this->GetWorldAngularMomentum()));
+}
+
+//////////////////////////////////////////////////
+math::Vector3 Link::GetWorldAngularMomentum() const
+{
+  return this->GetWorldInertiaMatrix() * this->GetWorldAngularVel();
 }
 
 //////////////////////////////////////////////////
@@ -747,9 +833,9 @@ void Link::FillMsg(msgs::Link &_msg)
   _msg.set_gravity(this->GetGravityMode());
   _msg.set_kinematic(this->GetKinematic());
   _msg.set_enabled(this->GetEnabled());
-  msgs::Set(_msg.mutable_pose(), relPose);
+  msgs::Set(_msg.mutable_pose(), relPose.Ign());
 
-  msgs::Set(this->visualMsg->mutable_pose(), relPose);
+  msgs::Set(this->visualMsg->mutable_pose(), relPose.Ign());
   _msg.add_visual()->CopyFrom(*this->visualMsg);
 
   _msg.mutable_inertial()->set_mass(this->inertial->GetMass());
@@ -759,25 +845,36 @@ void Link::FillMsg(msgs::Link &_msg)
   _msg.mutable_inertial()->set_iyy(this->inertial->GetIYY());
   _msg.mutable_inertial()->set_iyz(this->inertial->GetIYZ());
   _msg.mutable_inertial()->set_izz(this->inertial->GetIZZ());
-  msgs::Set(_msg.mutable_inertial()->mutable_pose(), this->inertial->GetPose());
+  msgs::Set(_msg.mutable_inertial()->mutable_pose(),
+      this->inertial->GetPose().Ign());
 
-  for (Collision_V::iterator iter = this->collisions.begin();
-      iter != this->collisions.end(); ++iter)
+  for (auto &child : this->children)
   {
-    (*iter)->FillMsg(*_msg.add_collision());
+    if (child->HasType(Base::COLLISION))
+    {
+      CollisionPtr collision = boost::static_pointer_cast<Collision>(child);
+      collision->FillMsg(*_msg.add_collision());
+    }
   }
 
-  for (std::vector<std::string>::iterator iter = this->sensors.begin();
-      iter != this->sensors.end(); ++iter)
+  // Add in the sensor data.
+  if (this->sdf->HasElement("sensor"))
   {
-    sensors::SensorPtr sensor = sensors::get_sensor(*iter);
-    if (sensor)
-      sensor->FillMsg(*_msg.add_sensor());
+    sdf::ElementPtr sensorElem = this->sdf->GetElement("sensor");
+    while (sensorElem)
+    {
+      msgs::Sensor *msg = _msg.add_sensor();
+      msg->CopyFrom(msgs::SensorFromSDF(sensorElem));
+      msg->set_parent(this->GetScopedName());
+      msg->set_parent_id(this->GetId());
+      sensorElem = sensorElem->GetNextElement("sensor");
+    }
   }
 
-  // Parse visuals from SDF
   if (this->visuals.empty())
     this->ParseVisuals();
+  else
+    this->UpdateVisualMsg();
 
   for (Visuals_M::iterator iter = this->visuals.begin();
       iter != this->visuals.end(); ++iter)
@@ -797,11 +894,19 @@ void Link::FillMsg(msgs::Link &_msg)
     proj->set_fov(elem->Get<double>("fov"));
     proj->set_near_clip(elem->Get<double>("near_clip"));
     proj->set_far_clip(elem->Get<double>("far_clip"));
-    msgs::Set(proj->mutable_pose(), elem->Get<math::Pose>("pose"));
+    msgs::Set(proj->mutable_pose(), elem->Get<ignition::math::Pose3d>("pose"));
   }
 
   if (this->IsCanonicalLink())
     _msg.set_canonical(true);
+
+  // Fill message with battery information
+  for (auto &battery : this->batteries)
+  {
+    msgs::Battery *bat = _msg.add_battery();
+    bat->set_name(battery->Name());
+    bat->set_voltage(battery->Voltage());
+  }
 }
 
 //////////////////////////////////////////////////
@@ -830,13 +935,15 @@ void Link::ProcessMsg(const msgs::Link &_msg)
   {
     this->inertial->ProcessMsg(_msg.inertial());
     this->SetEnabled(true);
-    this->UpdateMass();
+    // Only update the Center of Mass if object is dynamic
+    if (!this->GetKinematic())
+      this->UpdateMass();
   }
 
   if (_msg.has_pose())
   {
     this->SetEnabled(true);
-    this->SetRelativePose(msgs::Convert(_msg.pose()));
+    this->SetRelativePose(msgs::ConvertIgn(_msg.pose()));
   }
 
   for (int i = 0; i < _msg.collision_size(); i++)
@@ -916,6 +1023,12 @@ void Link::OnPoseChange()
 void Link::SetState(const LinkState &_state)
 {
   this->SetWorldPose(_state.GetPose());
+  this->SetLinearVel(_state.GetVelocity().pos);
+  this->SetAngularVel(_state.GetVelocity().rot.GetAsEuler());
+  this->SetLinearAccel(_state.GetAcceleration().pos);
+  this->SetAngularAccel(_state.GetAcceleration().rot.GetAsEuler());
+  this->SetForce(_state.GetWrench().pos);
+  this->SetTorque(_state.GetWrench().rot.GetAsEuler());
 
   /*
   for (unsigned int i = 0; i < _state.GetCollisionStateCount(); ++i)
@@ -985,13 +1098,127 @@ void Link::PublishData()
     msgs::Set(this->linkDataMsg.mutable_time(), this->world->GetSimTime());
     linkDataMsg.set_name(this->GetScopedName());
     msgs::Set(this->linkDataMsg.mutable_linear_velocity(),
-        this->GetWorldLinearVel());
+        this->GetWorldLinearVel().Ign());
     msgs::Set(this->linkDataMsg.mutable_angular_velocity(),
-        this->GetWorldAngularVel());
+        this->GetWorldAngularVel().Ign());
     this->dataPub->Publish(this->linkDataMsg);
   }
 }
 
+//////////////////////////////////////////////////
+common::BatteryPtr Link::Battery(const std::string &_name) const
+{
+  common::BatteryPtr result;
+
+  for (auto &battery : this->batteries)
+  {
+    if (battery->Name() == _name)
+    {
+      result = battery;
+      break;
+    }
+  }
+
+  return result;
+}
+
+/////////////////////////////////////////////////
+common::BatteryPtr Link::Battery(const size_t _index) const
+{
+  if (_index < this->batteries.size())
+    return this->batteries[_index];
+  else
+    return common::BatteryPtr();
+}
+
+/////////////////////////////////////////////////
+size_t Link::BatteryCount() const
+{
+  return this->batteries.size();
+}
+
+//////////////////////////////////////////////////
+bool Link::VisualId(const std::string &_visName, uint32_t &_visualId) const
+{
+  for (auto &iter : this->visuals)
+  {
+    if (iter.second.name() == _visName ||
+        iter.second.name() == this->GetScopedName() + "::" + _visName)
+    {
+      _visualId = iter.first;
+      return true;
+    }
+  }
+  gzerr << "Trying to get unique ID of visual from invalid visual name["
+        << _visName << "] for link [" << this->GetScopedName() << "]\n";
+  return false;
+}
+
+//////////////////////////////////////////////////
+bool Link::VisualPose(const uint32_t _id, ignition::math::Pose3d &_pose) const
+{
+  auto iter = this->visuals.find(_id);
+  if (iter == this->visuals.end())
+  {
+    gzerr << "Trying to get pose of visual from invalid visual id[" << _id
+          << "] for link [" << this->GetScopedName() << "]\n";
+    return false;
+  }
+  const msgs::Visual &msg = iter->second;
+  if (msg.has_pose())
+  {
+    _pose = msgs::ConvertIgn(msg.pose());
+  }
+  else
+  {
+    // Pose wasn't specified on SDF, use default value
+    _pose = ignition::math::Pose3d::Zero;
+  }
+  return true;
+}
+
+//////////////////////////////////////////////////
+bool Link::SetVisualPose(const uint32_t _id,
+                         const ignition::math::Pose3d &_pose)
+{
+  auto iter = this->visuals.find(_id);
+  if (iter == this->visuals.end())
+  {
+    gzerr << "Trying to set pose of visual from invalid visual id[" << _id
+          << "] for link [" << this->GetScopedName() << "]\n";
+    return false;
+  }
+  msgs::Visual &msg = iter->second;
+  msgs::Set(msg.mutable_pose(), _pose);
+  std::string linkName = this->GetScopedName();
+  if (this->sdf->HasElement("visual"))
+  {
+    sdf::ElementPtr visualElem = this->sdf->GetElement("visual");
+    while (visualElem)
+    {
+      std::string visName = linkName + "::" +
+        visualElem->Get<std::string>("name");
+
+      // update visual msg if it exists
+      if (msg.name() == visName)
+      {
+        visualElem->GetElement("pose")->Set(_pose);
+        break;
+      }
+
+      visualElem = visualElem->GetNextElement("visual");
+    }
+  }
+  msgs::Visual visual;
+  visual.set_name(msg.name());
+  visual.set_id(_id);
+  visual.set_parent_name(linkName);
+  visual.set_parent_id(this->GetId());
+  msgs::Set(visual.mutable_pose(), _pose);
+  this->visPub->Publish(visual);
+  return true;
+}
+
 //////////////////////////////////////////////////
 void Link::OnCollision(ConstContactsPtr &_msg)
 {
@@ -1027,32 +1254,10 @@ void Link::OnCollision(ConstContactsPtr &_msg)
 /////////////////////////////////////////////////
 void Link::ParseVisuals()
 {
-  // TODO: this shouldn't be in the physics sim
-  if (this->sdf->HasElement("visual"))
-  {
-    sdf::ElementPtr visualElem = this->sdf->GetElement("visual");
-    while (visualElem)
-    {
-      msgs::Visual msg = msgs::VisualFromSDF(visualElem);
+  this->UpdateVisualMsg();
 
-      std::string visName = this->GetScopedName() + "::" + msg.name();
-      msg.set_name(visName);
-      msg.set_id(physics::getUniqueId());
-      msg.set_parent_name(this->GetScopedName());
-      msg.set_parent_id(this->GetId());
-      msg.set_is_static(this->IsStatic());
-
-      this->visPub->Publish(msg);
-
-      Visuals_M::iterator iter = this->visuals.find(msg.id());
-      if (iter != this->visuals.end())
-        gzthrow(std::string("Duplicate visual name[")+msg.name()+"]\n");
-
-      this->visuals[msg.id()] = msg;
-
-      visualElem = visualElem->GetNextElement("visual");
-    }
-  }
+  for (auto const it : this->visuals)
+    this->visPub->Publish(it.second);
 }
 
 /////////////////////////////////////////////////
@@ -1094,14 +1299,14 @@ void Link::SetScale(const math::Vector3 &_scale)
     }
   }
 
-  this->scale = _scale;
+  // update the visual sdf to ensure cloning and saving has the correct values.
+  this->UpdateVisualGeomSDF(_scale);
 
-  // update the visual sdf to ensure cloning gets the correct values.
-  this->UpdateVisualSDF();
+  this->scale = _scale.Ign();
 }
 
 //////////////////////////////////////////////////
-void Link::UpdateVisualSDF()
+void Link::UpdateVisualGeomSDF(const math::Vector3 &_scale)
 {
   // TODO: this shouldn't be in the physics sim
   if (this->sdf->HasElement("visual"))
@@ -1113,22 +1318,85 @@ void Link::UpdateVisualSDF()
 
       if (geomElem->HasElement("box"))
       {
-        geomElem->GetElement("box")->GetElement("size")->Set(this->scale);
+        math::Vector3 size =
+            geomElem->GetElement("box")->Get<math::Vector3>("size");
+        geomElem->GetElement("box")->GetElement("size")->Set(
+            _scale/this->scale*size);
       }
       else if (geomElem->HasElement("sphere"))
       {
+        // update radius the same way as collision shapes
+        double radius = geomElem->GetElement("sphere")->Get<double>("radius");
+        double newRadius = std::max(_scale.z, std::max(_scale.x, _scale.y));
+        double oldRadius = std::max(this->scale.Z(),
+            std::max(this->scale.X(), this->scale.Y()));
         geomElem->GetElement("sphere")->GetElement("radius")->Set(
-            this->scale.x/2.0);
+            newRadius/oldRadius*radius);
       }
       else if (geomElem->HasElement("cylinder"))
       {
-        geomElem->GetElement("cylinder")->GetElement("radius")
-            ->Set(this->scale.x/2.0);
+        // update radius the same way as collision shapes
+        double radius = geomElem->GetElement("cylinder")->Get<double>("radius");
+        double newRadius = std::max(_scale.x, _scale.y);
+        double oldRadius = std::max(this->scale.X(), this->scale.Y());
+
+        double length = geomElem->GetElement("cylinder")->Get<double>("length");
+        geomElem->GetElement("cylinder")->GetElement("radius")->Set(
+            newRadius/oldRadius*radius);
         geomElem->GetElement("cylinder")->GetElement("length")->Set(
-            this->scale.z);
+            _scale.z/this->scale.Z()*length);
       }
       else if (geomElem->HasElement("mesh"))
-        geomElem->GetElement("mesh")->GetElement("scale")->Set(this->scale);
+        geomElem->GetElement("mesh")->GetElement("scale")->Set(_scale);
+
+      visualElem = visualElem->GetNextElement("visual");
+    }
+  }
+}
+
+//////////////////////////////////////////////////
+void Link::UpdateVisualMsg()
+{
+  // TODO: this shouldn't be in the physics sim
+  if (this->sdf->HasElement("visual"))
+  {
+    sdf::ElementPtr visualElem = this->sdf->GetElement("visual");
+    while (visualElem)
+    {
+      msgs::Visual msg = msgs::VisualFromSDF(visualElem);
+
+      bool newVis = true;
+      std::string linkName = this->GetScopedName();
+
+      // update visual msg if it exists
+      for (auto &iter : this->visuals)
+      {
+        std::string visName = linkName + "::" +
+            visualElem->Get<std::string>("name");
+        if (iter.second.name() == visName)
+        {
+          iter.second.mutable_geometry()->CopyFrom(msg.geometry());
+          newVis = false;
+          break;
+        }
+      }
+
+      // add to visual msgs if not found.
+      if (newVis)
+      {
+        std::string visName = this->GetScopedName() + "::" + msg.name();
+        msg.set_name(visName);
+        msg.set_id(physics::getUniqueId());
+        msg.set_parent_name(this->GetScopedName());
+        msg.set_parent_id(this->GetId());
+        msg.set_is_static(this->IsStatic());
+        msg.set_type(msgs::Visual::VISUAL);
+
+        auto iter = this->visuals.find(msg.id());
+        if (iter != this->visuals.end())
+          gzthrow(std::string("Duplicate visual name[")+msg.name()+"]\n");
+        this->visuals[msg.id()] = msg;
+      }
 
       visualElem = visualElem->GetNextElement("visual");
     }
@@ -1326,3 +1594,34 @@ msgs::Visual Link::GetVisualMessage(const std::string &_name) const
 
   return result;
 }
+
+//////////////////////////////////////////////////
+void Link::OnWrenchMsg(ConstWrenchPtr &_msg)
+{
+  boost::mutex::scoped_lock lock(this->wrenchMsgMutex);
+  this->wrenchMsgs.push_back(*_msg);
+}
+
+//////////////////////////////////////////////////
+void Link::ProcessWrenchMsg(const msgs::Wrench &_msg)
+{
+  math::Vector3 pos = math::Vector3::Zero;
+  if (_msg.has_force_offset())
+  {
+    pos = msgs::ConvertIgn(_msg.force_offset());
+  }
+
+  const ignition::math::Vector3d force = msgs::ConvertIgn(_msg.force());
+  this->AddLinkForce(force, pos);
+
+  const ignition::math::Vector3d torque = msgs::ConvertIgn(_msg.torque());
+  this->AddRelativeTorque(torque);
+}
+
+//////////////////////////////////////////////////
+void Link::LoadBattery(sdf::ElementPtr _sdf)
+{
+  common::BatteryPtr battery(new common::Battery());
+  battery->Load(_sdf);
+  this->batteries.push_back(battery);
+}
