@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2015 Open Source Robotics Foundation
+ * Copyright (C) 2014-2016 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -23,37 +23,37 @@
 #include "gazebo/physics/WorldPrivate.hh"
 
 #include "gazebo/physics/dart/dart_inc.h"
+#include "gazebo/physics/dart/DARTCollision.hh"
 #include "gazebo/physics/dart/DARTPhysics.hh"
 #include "gazebo/physics/dart/DARTModel.hh"
 #include "gazebo/physics/dart/DARTLink.hh"
 #include "gazebo/physics/dart/DARTJoint.hh"
+#include "gazebo/physics/dart/DARTSurfaceParams.hh"
+
+#include "gazebo/physics/dart/DARTLinkPrivate.hh"
 
 using namespace gazebo;
 using namespace physics;
 
 //////////////////////////////////////////////////
 DARTLink::DARTLink(EntityPtr _parent)
-  : Link(_parent),
-    dtBodyNode(NULL),
-    staticLink(false),
-    weldJointConst(NULL)
+  : Link(_parent), dataPtr(new DARTLinkPrivate())
 {
 }
 
 //////////////////////////////////////////////////
 DARTLink::~DARTLink()
 {
-  // We don't need to delete dartBodyNode because skeletone will delete
-  // dartBodyNode if this is registered to the skeletone.
+  delete this->dataPtr;
 }
 
 //////////////////////////////////////////////////
 void DARTLink::Load(sdf::ElementPtr _sdf)
 {
-  this->dartPhysics = boost::dynamic_pointer_cast<DARTPhysics>(
+  this->dataPtr->dartPhysics = boost::dynamic_pointer_cast<DARTPhysics>(
       this->GetWorld()->GetPhysicsEngine());
 
-  if (this->dartPhysics == NULL)
+  if (this->dataPtr->dartPhysics == NULL)
     gzthrow("Not using the dart physics engine");
 
   // Check if soft_contact element is contained in this link. If so,
@@ -103,7 +103,7 @@ void DARTLink::Load(sdf::ElementPtr _sdf)
   if (dartElem != NULL)
   {
     // Create DART SoftBodyNode
-    dart::dynamics::SoftBodyNode* dtSoftBodyNode
+    dart::dynamics::SoftBodyNode *dtSoftBodyNode
         = new dart::dynamics::SoftBodyNode();
 
     // Mass
@@ -166,12 +166,12 @@ void DARTLink::Load(sdf::ElementPtr _sdf)
       gzerr << "Unknown soft shape" << std::endl;
     }
 
-    dtBodyNode = dtSoftBodyNode;
+    this->dataPtr->dtBodyNode = dtSoftBodyNode;
   }
   else
   {
     // Create DART BodyNode
-    dtBodyNode = new dart::dynamics::BodyNode();
+    this->dataPtr->dtBodyNode = new dart::dynamics::BodyNode();
   }
 
   Link::Load(_sdf);
@@ -184,11 +184,11 @@ void DARTLink::Init()
 
   // Name
   std::string bodyName = this->GetName();
-  this->dtBodyNode->setName(bodyName);
+  this->dataPtr->dtBodyNode->setName(bodyName);
 
   // Mass
   double mass = this->inertial->GetMass();
-  this->dtBodyNode->setMass(mass);
+  this->dataPtr->dtBodyNode->setMass(mass);
 
   // Inertia
   double Ixx = this->inertial->GetIXX();
@@ -197,18 +197,58 @@ void DARTLink::Init()
   double Ixy = this->inertial->GetIXY();
   double Ixz = this->inertial->GetIXZ();
   double Iyz = this->inertial->GetIYZ();
-  this->dtBodyNode->setMomentOfInertia(Ixx, Iyy, Izz, Ixy, Ixz, Iyz);
+  this->dataPtr->dtBodyNode->setMomentOfInertia(Ixx, Iyy, Izz, Ixy, Ixz, Iyz);
 
   // Visual
   this->visuals;
 
   // COG offset
   math::Vector3 cog = this->inertial->GetCoG();
-  this->dtBodyNode->setLocalCOM(DARTTypes::ConvVec3(cog));
+  this->dataPtr->dtBodyNode->setLocalCOM(DARTTypes::ConvVec3(cog));
 
   // Gravity mode
   this->SetGravityMode(this->sdf->Get<bool>("gravity"));
 
+  // Friction coefficient
+
+  /// \todo FIXME: Friction Parameters
+  /// Gazebo allows different friction parameters per collision objects,
+  /// while DART stores the friction parameter per link (BodyNode in DART). For
+  /// now, the average friction parameter of all the child collision objects is
+  /// stored in this->dataPtr->dtBodyNode.
+  /// Final friction coefficient is applied in DART's constraint solver by
+  /// taking the lower of the 2 colliding rigidLink's.
+  /// See also:
+  /// - https://github.com/dartsim/dart/issues/141
+  /// - https://github.com/dartsim/dart/issues/266
+
+  double hackAvgMu1 = 0;
+  double hackAvgMu2 = 0;
+  int numCollisions = 0;
+
+  for (auto const &child : this->children)
+  {
+    if (child->HasType(Base::COLLISION))
+    {
+      CollisionPtr collision =
+          boost::static_pointer_cast<Collision>(child);
+
+      SurfaceParamsPtr surface = collision->GetSurface();
+      GZ_ASSERT(surface, "Surface pointer for is invalid");
+      FrictionPyramidPtr friction = surface->FrictionPyramid();
+      GZ_ASSERT(friction, "Friction pointer for is invalid");
+
+      numCollisions++;
+      hackAvgMu1 += friction->MuPrimary();
+      hackAvgMu2 += friction->MuSecondary();
+    }
+  }
+
+  hackAvgMu1 /= static_cast<double>(numCollisions);
+  hackAvgMu2 /= static_cast<double>(numCollisions);
+
+  this->dataPtr->dtBodyNode->setFrictionCoeff(0.5 * (hackAvgMu1 + hackAvgMu2));
+
   // We don't add dart body node to the skeleton here because dart body node
   // should be set its parent joint before being added. This body node will be
   // added to the skeleton in DARTModel::Init().
@@ -220,20 +260,36 @@ void DARTLink::Fini()
   Link::Fini();
 }
 
+/////////////////////////////////////////////////////////////////////
+void DARTLink::UpdateMass()
+{
+  if (this->dataPtr->dtBodyNode && this->inertial)
+  {
+    this->dataPtr->dtBodyNode->setMass(this->inertial->GetMass());
+    auto Ixxyyzz = this->inertial->GetPrincipalMoments();
+    auto Ixyxzyz = this->inertial->GetProductsofInertia();
+    this->dataPtr->dtBodyNode->setMomentOfInertia(
+        Ixxyyzz[0], Ixxyyzz[1], Ixxyyzz[2],
+        Ixyxzyz[0], Ixyxzyz[1], Ixyxzyz[2]);
+    auto cog = DARTTypes::ConvVec3(this->inertial->GetCoG());
+    this->dataPtr->dtBodyNode->setLocalCOM(cog);
+  }
+}
+
 //////////////////////////////////////////////////
 void DARTLink::OnPoseChange()
 {
   Link::OnPoseChange();
 
   // DART body node always have its parent joint.
-  dart::dynamics::Joint* joint = this->dtBodyNode->getParentJoint();
+  dart::dynamics::Joint *joint = this->dataPtr->dtBodyNode->getParentJoint();
 
   // This is for the case this function called before DARTModel::Init() is
   // called.
   if (joint == NULL)
     return;
 
-  dart::dynamics::FreeJoint* freeJoint =
+  dart::dynamics::FreeJoint *freeJoint =
       dynamic_cast<dart::dynamics::FreeJoint*>(joint);
   if (freeJoint)
   {
@@ -243,13 +299,32 @@ void DARTLink::OnPoseChange()
     const Eigen::Isometry3d &InvT2 = joint->getTransformFromChildBodyNode();
     Eigen::Isometry3d P = Eigen::Isometry3d::Identity();
 
-    if (this->dtBodyNode->getParentBodyNode())
-      P = this->dtBodyNode->getParentBodyNode()->getTransform();
+    if (this->dataPtr->dtBodyNode->getParentBodyNode())
+      P = this->dataPtr->dtBodyNode->getParentBodyNode()->getTransform();
 
     Eigen::Isometry3d Q = T1.inverse() * P.inverse() * W * InvT2;
-    // Set generalized coordinate and update the transformations only
-    freeJoint->setPositions(dart::math::logMap(Q));
+
+    // Convert homogeneous transformation matrix to 6-dimensional generalized
+    // coordinates. There are several ways of conversions. Here is the way of
+    // DART. The orientation part is converted by using logarithm map, which
+    // maps SO(3) to so(3), and it takes the first three components of the
+    // generalized coordinates. On the other hand, the position part just takes
+    // the last three components of the generalized coordinates without any
+    // conversion.
+    Eigen::Vector6d q;
+    q.head<3>() = dart::math::logMap(Q.linear());
+    q.tail<3>() = Q.translation();
+    freeJoint->setPositions(q);
+    // TODO: The above 4 lines will be reduced to single line as:
+    // freeJoint->setPositions(FreeJoint::convertToPositions(Q));
+    // after the following PR is merged:
+    // https://github.com/dartsim/dart/pull/322
+
+    // Update all the transformations of the links in the parent model.
     freeJoint->getSkeleton()->computeForwardKinematics(true, false, false);
+    // TODO: This kinematic updating will be done automatically after pull
+    // request (https://github.com/dartsim/dart/pull/319) is merged so that
+    // we don't need this line anymore.
   }
   else
   {
@@ -275,7 +350,7 @@ bool DARTLink::GetEnabled() const
 void DARTLink::SetLinearVel(const math::Vector3 &_vel)
 {
   // DART body node always have its parent joint.
-  dart::dynamics::Joint* joint = this->dtBodyNode->getParentJoint();
+  dart::dynamics::Joint *joint = this->dataPtr->dtBodyNode->getParentJoint();
 
   // This is for the case this function called before DARTModel::Init() is
   // called.
@@ -296,18 +371,20 @@ void DARTLink::SetLinearVel(const math::Vector3 &_vel)
     // Generalized velocities
     Eigen::Vector3d genVel = DARTTypes::ConvVec3(_vel);
 
+    dart::dynamics::BodyNode *dtBodyNode = this->dataPtr->dtBodyNode;
+
     // If this link has parent link then subtract the effect of parent link's
     // linear and angular velocities
-    if (this->dtBodyNode->getParentBodyNode())
+    if (dtBodyNode->getParentBodyNode())
     {
       // Local transformation from the parent link frame to this link frame
       Eigen::Isometry3d T = freeJoint->getLocalTransform();
 
       // Parent link's linear and angular velocities
       Eigen::Vector3d parentLinVel =
-          this->dtBodyNode->getParentBodyNode()->getBodyLinearVelocity();
+          dtBodyNode->getParentBodyNode()->getBodyLinearVelocity();
       Eigen::Vector3d parentAngVel =
-          this->dtBodyNode->getParentBodyNode()->getBodyAngularVelocity();
+          dtBodyNode->getParentBodyNode()->getBodyAngularVelocity();
 
       // The effect of the parent link's velocities
       Eigen::Vector3d propagatedLinVel =
@@ -319,7 +396,7 @@ void DARTLink::SetLinearVel(const math::Vector3 &_vel)
     }
 
     // Rotation matrix from world frame to this link frame
-    Eigen::Matrix3d R = this->dtBodyNode->getTransform().linear();
+    Eigen::Matrix3d R = dtBodyNode->getTransform().linear();
 
     // Change the reference frame to world
     genVel = R * genVel;
@@ -343,7 +420,7 @@ void DARTLink::SetLinearVel(const math::Vector3 &_vel)
 void DARTLink::SetAngularVel(const math::Vector3 &_vel)
 {
   // DART body node always have its parent joint.
-  dart::dynamics::Joint *joint = this->dtBodyNode->getParentJoint();
+  dart::dynamics::Joint *joint = this->dataPtr->dtBodyNode->getParentJoint();
 
   // This is for the case this function called before DARTModel::Init() is
   // called.
@@ -364,16 +441,18 @@ void DARTLink::SetAngularVel(const math::Vector3 &_vel)
     // Generalized velocities
     Eigen::Vector3d genVel = DARTTypes::ConvVec3(_vel);
 
+    dart::dynamics::BodyNode *dtBodyNode = this->dataPtr->dtBodyNode;
+
     // If this link has parent link then subtract the effect of parent link's
     // linear and angular velocities
-    if (this->dtBodyNode->getParentBodyNode())
+    if (dtBodyNode->getParentBodyNode())
     {
       // Local transformation from the parent link frame to this link frame
       Eigen::Isometry3d T = freeJoint->getLocalTransform();
 
       // Parent link's linear and angular velocities
       Eigen::Vector3d parentAngVel =
-          this->dtBodyNode->getParentBodyNode()->getBodyAngularVelocity();
+          dtBodyNode->getParentBodyNode()->getBodyAngularVelocity();
 
       // The effect of the parent link's velocities
       Eigen::Vector3d propagatedAngVel = T.linear().transpose() * parentAngVel;
@@ -383,7 +462,7 @@ void DARTLink::SetAngularVel(const math::Vector3 &_vel)
     }
 
     // Rotation matrix from world frame to this link frame
-    Eigen::Matrix3d R = this->dtBodyNode->getTransform().linear();
+    Eigen::Matrix3d R = dtBodyNode->getTransform().linear();
 
     // Change the reference frame to world
     genVel = R * genVel;
@@ -407,26 +486,26 @@ void DARTLink::SetAngularVel(const math::Vector3 &_vel)
 void DARTLink::SetForce(const math::Vector3 &_force)
 {
   // DART assume that _force is external force.
-  this->dtBodyNode->setExtForce(DARTTypes::ConvVec3(_force));
+  this->dataPtr->dtBodyNode->setExtForce(DARTTypes::ConvVec3(_force));
 }
 
 //////////////////////////////////////////////////
 void DARTLink::SetTorque(const math::Vector3 &_torque)
 {
   // DART assume that _torque is external torque.
-  this->dtBodyNode->setExtTorque(DARTTypes::ConvVec3(_torque));
+  this->dataPtr->dtBodyNode->setExtTorque(DARTTypes::ConvVec3(_torque));
 }
 
 //////////////////////////////////////////////////
 void DARTLink::AddForce(const math::Vector3 &_force)
 {
-  this->dtBodyNode->addExtForce(DARTTypes::ConvVec3(_force));
+  this->dataPtr->dtBodyNode->addExtForce(DARTTypes::ConvVec3(_force));
 }
 
 /////////////////////////////////////////////////
 void DARTLink::AddRelativeForce(const math::Vector3 &_force)
 {
-  this->dtBodyNode->addExtForce(DARTTypes::ConvVec3(_force),
+  this->dataPtr->dtBodyNode->addExtForce(DARTTypes::ConvVec3(_force),
                                 Eigen::Vector3d::Zero(),
                                 true, true);
 }
@@ -435,7 +514,7 @@ void DARTLink::AddRelativeForce(const math::Vector3 &_force)
 void DARTLink::AddForceAtWorldPosition(const math::Vector3 &_force,
                                         const math::Vector3 &_pos)
 {
-  this->dtBodyNode->addExtForce(DARTTypes::ConvVec3(_pos),
+  this->dataPtr->dtBodyNode->addExtForce(DARTTypes::ConvVec3(_pos),
                                 DARTTypes::ConvVec3(_force),
                                 false, false);
 }
@@ -444,21 +523,29 @@ void DARTLink::AddForceAtWorldPosition(const math::Vector3 &_force,
 void DARTLink::AddForceAtRelativePosition(const math::Vector3 &_force,
                                           const math::Vector3 &_relpos)
 {
-  this->dtBodyNode->addExtForce(DARTTypes::ConvVec3(_force),
+  this->dataPtr->dtBodyNode->addExtForce(DARTTypes::ConvVec3(_force),
                                 DARTTypes::ConvVec3(_relpos),
                                 true, true);
 }
 
+//////////////////////////////////////////////////
+void DARTLink::AddLinkForce(const math::Vector3 &/*_force*/,
+    const math::Vector3 &/*_offset*/)
+{
+  gzlog << "DARTLink::AddLinkForce not yet implemented (issue #1477)."
+        << std::endl;
+}
+
 /////////////////////////////////////////////////
 void DARTLink::AddTorque(const math::Vector3 &_torque)
 {
-  this->dtBodyNode->addExtTorque(DARTTypes::ConvVec3(_torque));
+  this->dataPtr->dtBodyNode->addExtTorque(DARTTypes::ConvVec3(_torque));
 }
 
 /////////////////////////////////////////////////
 void DARTLink::AddRelativeTorque(const math::Vector3 &_torque)
 {
-  this->dtBodyNode->addExtTorque(DARTTypes::ConvVec3(_torque), true);
+  this->dataPtr->dtBodyNode->addExtTorque(DARTTypes::ConvVec3(_torque), true);
 }
 
 //////////////////////////////////////////////////
@@ -466,7 +553,8 @@ gazebo::math::Vector3 DARTLink::GetWorldLinearVel(
     const math::Vector3 &_offset) const
 {
   Eigen::Vector3d linVel =
-      this->dtBodyNode->getWorldLinearVelocity(DARTTypes::ConvVec3(_offset));
+      this->dataPtr->dtBodyNode->getWorldLinearVelocity(
+        DARTTypes::ConvVec3(_offset));
 
   return DARTTypes::ConvVec3(linVel);
 }
@@ -479,9 +567,10 @@ math::Vector3 DARTLink::GetWorldLinearVel(
   Eigen::Matrix3d R1 = Eigen::Matrix3d(DARTTypes::ConvQuat(_q));
   Eigen::Vector3d worldOffset = R1 * DARTTypes::ConvVec3(_offset);
   Eigen::Vector3d bodyOffset =
-      this->dtBodyNode->getTransform().linear().transpose() * worldOffset;
+      this->dataPtr->dtBodyNode->getTransform().linear().transpose() *
+      worldOffset;
   Eigen::Vector3d linVel =
-    this->dtBodyNode->getWorldLinearVelocity(bodyOffset);
+      this->dataPtr->dtBodyNode->getWorldLinearVelocity(bodyOffset);
 
   return DARTTypes::ConvVec3(linVel);
 }
@@ -489,7 +578,7 @@ math::Vector3 DARTLink::GetWorldLinearVel(
 //////////////////////////////////////////////////
 math::Vector3 DARTLink::GetWorldCoGLinearVel() const
 {
-  Eigen::Vector3d linVel = this->dtBodyNode->getWorldCOMVelocity();
+  Eigen::Vector3d linVel = this->dataPtr->dtBodyNode->getWorldCOMVelocity();
 
   return DARTTypes::ConvVec3(linVel);
 }
@@ -497,7 +586,7 @@ math::Vector3 DARTLink::GetWorldCoGLinearVel() const
 //////////////////////////////////////////////////
 math::Vector3 DARTLink::GetWorldAngularVel() const
 {
-  Eigen::Vector3d angVel = this->dtBodyNode->getWorldAngularVelocity();
+  Eigen::Vector3d angVel = this->dataPtr->dtBodyNode->getWorldAngularVelocity();
 
   return DARTTypes::ConvVec3(angVel);
 }
@@ -505,7 +594,7 @@ math::Vector3 DARTLink::GetWorldAngularVel() const
 /////////////////////////////////////////////////
 math::Vector3 DARTLink::GetWorldForce() const
 {
-  Eigen::Vector6d F = this->dtBodyNode->getExternalForceGlobal();
+  Eigen::Vector6d F = this->dataPtr->dtBodyNode->getExternalForceGlobal();
   return DARTTypes::ConvVec3(F.tail<3>());
 }
 
@@ -515,10 +604,10 @@ math::Vector3 DARTLink::GetWorldTorque() const
   // TODO: Need verification
   math::Vector3 torque;
 
-  Eigen::Isometry3d W = this->dtBodyNode->getTransform();
-  Eigen::Matrix6d G   = this->dtBodyNode->getSpatialInertia();
-  Eigen::VectorXd V   = this->dtBodyNode->getBodyVelocity();
-  Eigen::VectorXd dV  = this->dtBodyNode->getBodyAcceleration();
+  Eigen::Isometry3d W = this->dataPtr->dtBodyNode->getTransform();
+  Eigen::Matrix6d G   = this->dataPtr->dtBodyNode->getSpatialInertia();
+  Eigen::VectorXd V   = this->dataPtr->dtBodyNode->getBodyVelocity();
+  Eigen::VectorXd dV  = this->dataPtr->dtBodyNode->getBodyAcceleration();
   Eigen::Vector6d F   = G * dV - dart::math::dad(V, G * V);
 
   torque = DARTTypes::ConvVec3(W.linear() * F.head<3>());
@@ -530,13 +619,13 @@ math::Vector3 DARTLink::GetWorldTorque() const
 void DARTLink::SetGravityMode(bool _mode)
 {
   this->sdf->GetElement("gravity")->Set(_mode);
-  this->dtBodyNode->setGravityMode(_mode);
+  this->dataPtr->dtBodyNode->setGravityMode(_mode);
 }
 
 //////////////////////////////////////////////////
 bool DARTLink::GetGravityMode() const
 {
-  return this->dtBodyNode->getGravityMode();
+  return this->dataPtr->dtBodyNode->getGravityMode();
 }
 
 //////////////////////////////////////////////////
@@ -544,21 +633,23 @@ void DARTLink::SetSelfCollide(bool _collide)
 {
   this->sdf->GetElement("self_collide")->Set(_collide);
 
+  dart::dynamics::BodyNode *dtBodyNode = this->dataPtr->dtBodyNode;
+
   // If this function is called before the body node is not added to a skeleton,
   // the body node does not have parent skeleton. So we just return here. Self
   // collision setting will be done later in DARTModel::Init().
   if (dtBodyNode->getSkeleton() == NULL)
     return;
 
-  dart::simulation::World *dtWorld = this->dartPhysics->GetDARTWorld();
-  dart::dynamics::Skeleton *dtSkeleton = this->dtBodyNode->getSkeleton();
+  dart::simulation::World *dtWorld = this->dataPtr->dartPhysics->GetDARTWorld();
+  dart::dynamics::Skeleton *dtSkeleton = dtBodyNode->getSkeleton();
   dart::collision::CollisionDetector *dtCollDet =
       dtWorld->getConstraintSolver()->getCollisionDetector();
 
   Link_V links = this->GetModel()->GetLinks();
 
   bool isSkeletonSelfCollidable =
-      this->dtBodyNode->getSkeleton()->isEnabledSelfCollisionCheck();
+      dtBodyNode->getSkeleton()->isEnabledSelfCollisionCheck();
 
   if (_collide)
   {
@@ -574,14 +665,14 @@ void DARTLink::SetSelfCollide(bool _collide)
           dart::dynamics::BodyNode *itdtBodyNode =
             boost::dynamic_pointer_cast<DARTLink>(links[i])->GetDARTBodyNode();
 
-          // If this->dtBodyNode and itdtBodyNode are connected then don't
-          // enable the pair.
+          // If this->dataPtr->dtBodyNode and itdtBodyNode are connected then
+          // don't enable the pair.
           // Please see: https://bitbucket.org/osrf/gazebo/issue/899
-          if ((this->dtBodyNode->getParentBodyNode() == itdtBodyNode) ||
-              itdtBodyNode->getParentBodyNode() == this->dtBodyNode)
+          if ((dtBodyNode->getParentBodyNode() == itdtBodyNode) ||
+              itdtBodyNode->getParentBodyNode() == this->dataPtr->dtBodyNode)
             continue;
 
-          dtCollDet->enablePair(this->dtBodyNode, itdtBodyNode);
+          dtCollDet->enablePair(dtBodyNode, itdtBodyNode);
         }
       }
     }
@@ -602,8 +693,8 @@ void DARTLink::SetSelfCollide(bool _collide)
           dart::dynamics::BodyNode *itdtBodyNode2 =
             boost::dynamic_pointer_cast<DARTLink>(links[j])->GetDARTBodyNode();
 
-          // If this->dtBodyNode and itdtBodyNode are connected then don't
-          // enable the pair.
+          // If this->dataPtr->dtBodyNode and itdtBodyNode are connected then
+          // don't enable the pair.
           // Please see: https://bitbucket.org/osrf/gazebo/issue/899
           if ((itdtBodyNode1->getParentBodyNode() == itdtBodyNode2) ||
               itdtBodyNode2->getParentBodyNode() == itdtBodyNode1)
@@ -627,7 +718,7 @@ void DARTLink::SetSelfCollide(bool _collide)
         {
           dart::dynamics::BodyNode *itdtBodyNode =
             boost::dynamic_pointer_cast<DARTLink>(links[i])->GetDARTBodyNode();
-          dtCollDet->disablePair(this->dtBodyNode, itdtBodyNode);
+          dtCollDet->disablePair(dtBodyNode, itdtBodyNode);
         }
       }
     }
@@ -686,23 +777,27 @@ void DARTLink::SetAutoDisable(bool /*_disable*/)
 //////////////////////////////////////////////////
 void DARTLink::SetLinkStatic(bool _static)
 {
-  if (_static == staticLink)
+  if (_static == this->dataPtr->staticLink)
     return;
 
   if (_static == true)
   {
     // Add weld joint constraint to DART
-    this->weldJointConst =
-        new dart::constraint::WeldJointConstraint(this->dtBodyNode);
-    GetDARTWorld()->getConstraintSolver()->addConstraint(weldJointConst);
+    this->dataPtr->dtWeldJointConst =
+        new dart::constraint::WeldJointConstraint(this->dataPtr->dtBodyNode);
+    GetDARTWorld()->getConstraintSolver()->addConstraint(
+        this->dataPtr->dtWeldJointConst);
   }
   else
   {
     // Remove ball and revolute joint constraints from DART
-    GetDARTWorld()->getConstraintSolver()->removeConstraint(weldJointConst);
+    GetDARTWorld()->getConstraintSolver()->removeConstraint(
+        this->dataPtr->dtWeldJointConst);
+    delete this->dataPtr->dtWeldJointConst;
+    this->dataPtr->dtWeldJointConst = NULL;
   }
 
-  staticLink = _static;
+  this->dataPtr->staticLink = _static;
 }
 
 //////////////////////////////////////////////////
@@ -711,7 +806,7 @@ void DARTLink::updateDirtyPoseFromDARTTransformation()
   // Step 1: get dart body's transformation
   // Step 2: set gazebo link's pose using the transformation
   math::Pose newPose = DARTTypes::ConvPose(
-                         this->dtBodyNode->getTransform());
+                         this->dataPtr->dtBodyNode->getTransform());
 
   // Set the new pose to this link
   this->dirtyPose = newPose;
@@ -743,17 +838,17 @@ DARTModelPtr DARTLink::GetDARTModel() const
 //////////////////////////////////////////////////
 dart::dynamics::BodyNode *DARTLink::GetDARTBodyNode() const
 {
-  return dtBodyNode;
+  return this->dataPtr->dtBodyNode;
 }
 
 //////////////////////////////////////////////////
 void DARTLink::SetDARTParentJoint(DARTJointPtr _dartParentJoint)
 {
-  dartParentJoint = _dartParentJoint;
+  this->dataPtr->dartParentJoint = _dartParentJoint;
 }
 
 //////////////////////////////////////////////////
 void DARTLink::AddDARTChildJoint(DARTJointPtr _dartChildJoint)
 {
-  dartChildJoints.push_back(_dartChildJoint);
+  this->dataPtr->dartChildJoints.push_back(_dartChildJoint);
 }
