@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Open Source Robotics Foundation
+ * Copyright (C) 2012-2016 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -95,40 +95,14 @@ void ODELink::Init()
   {
     GZ_ASSERT(this->inertial != NULL, "Inertial pointer is NULL");
     math::Vector3 cogVec = this->inertial->GetCoG();
-    Base_V::iterator iter;
-    for (iter = this->children.begin(); iter != this->children.end(); ++iter)
+    for (auto const &child : this->children)
     {
-      if ((*iter)->HasType(Base::COLLISION))
+      if (child->HasType(Base::COLLISION))
       {
-        ODECollisionPtr g = boost::static_pointer_cast<ODECollision>(*iter);
+        ODECollisionPtr g = boost::static_pointer_cast<ODECollision>(child);
         if (g->IsPlaceable() && g->GetCollisionId())
         {
           dGeomSetBody(g->GetCollisionId(), this->linkId);
-
-          // update pose immediately
-          math::Pose localPose = g->GetRelativePose();
-          localPose.pos -= cogVec;
-
-          dQuaternion q;
-          q[0] = localPose.rot.w;
-          q[1] = localPose.rot.x;
-          q[2] = localPose.rot.y;
-          q[3] = localPose.rot.z;
-
-          // Set the pose of the encapsulated collision; this is always relative
-          // to the CoM
-          dGeomSetOffsetPosition(g->GetCollisionId(), localPose.pos.x,
-              localPose.pos.y, localPose.pos.z);
-          dGeomSetOffsetQuaternion(g->GetCollisionId(), q);
-
-          // Set max_vel and min_depth
-          if (g->GetODESurface()->maxVel < 0)
-          {
-            g->GetODESurface()->maxVel =
-             this->GetWorld()->GetPhysicsEngine()->GetContactMaxCorrectingVel();
-          }
-          dBodySetMaxVel(this->linkId, g->GetODESurface()->maxVel);
-          dBodySetMinDepth(this->linkId, g->GetODESurface()->minDepth);
         }
       }
     }
@@ -140,8 +114,12 @@ void ODELink::Init()
           << " in ODELink::Init" << std::endl;
   }
 
-  // Update the Center of Mass.
-  this->UpdateMass();
+  // Update the Collision Offsets, Surface, and Center of Mass.
+  this->UpdateCollisionOffsets();
+  this->UpdateSurface();
+  // Only update the Center of Mass if object is dynamic
+  if (!this->GetKinematic())
+    this->UpdateMass();
 
   if (this->linkId)
   {
@@ -182,8 +160,8 @@ void ODELink::MoveCallback(dBodyID _id)
 
   self->dirtyPose.pos -= cog;
 
-  // TODO: this is an ugly line of code. It's like this for speed.
-  self->world->dataPtr->dirtyPoses.push_back(self);
+  // Tell the world that our pose has changed.
+  self->world->_AddDirty(self);
 
   // self->poseMutex->unlock();
 
@@ -328,18 +306,70 @@ bool ODELink::GetEnabled() const
   return result;
 }
 
+/////////////////////////////////////////////////////////////////////
+void ODELink::UpdateCollisionOffsets()
+{
+  if (this->linkId)
+  {
+    GZ_ASSERT(this->inertial != NULL, "Inertial pointer is NULL");
+    math::Vector3 cogVec = this->inertial->GetCoG();
+    for (auto const &child : this->children)
+    {
+      if (child->HasType(Base::COLLISION))
+      {
+        ODECollisionPtr g = boost::static_pointer_cast<ODECollision>(child);
+        if (g->IsPlaceable() && g->GetCollisionId())
+        {
+          // update pose immediately
+          math::Pose localPose = g->GetRelativePose();
+          localPose.pos -= cogVec;
+
+          dQuaternion q;
+          q[0] = localPose.rot.w;
+          q[1] = localPose.rot.x;
+          q[2] = localPose.rot.y;
+          q[3] = localPose.rot.z;
+
+          // Set the pose of the encapsulated collision; this is always relative
+          // to the CoM
+          dGeomSetOffsetPosition(g->GetCollisionId(), localPose.pos.x,
+              localPose.pos.y, localPose.pos.z);
+          dGeomSetOffsetQuaternion(g->GetCollisionId(), q);
+        }
+      }
+    }
+  }
+}
+
 /////////////////////////////////////////////////////////////////////
 void ODELink::UpdateSurface()
 {
-  Base_V::iterator iter;
-  Base_V::iterator iter_end = this->children.end();
-  for (iter = this->children.begin(); iter != iter_end; ++iter)
+  if (!this->linkId)
   {
-    if ((*iter)->HasType(Base::COLLISION))
+    return;
+  }
+
+  for (auto const &child : this->children)
+  {
+    if (child->HasType(Base::COLLISION))
     {
-      ODECollisionPtr g = boost::static_pointer_cast<ODECollision>(*iter);
+      ODECollisionPtr g = boost::static_pointer_cast<ODECollision>(child);
       if (g->IsPlaceable() && g->GetCollisionId())
       {
+        // Set max_vel and min_depth
+        boost::any value;
+        if (g->GetODESurface()->maxVel < 0 && this->GetWorld()->
+            GetPhysicsEngine()->GetParam("contact_max_correcting_vel", value))
+        {
+          try
+          {
+            g->GetODESurface()->maxVel = boost::any_cast<double>(value);
+          }
+          catch(boost::bad_any_cast &_e)
+          {
+            gzerr << "Failed boost::any_cast in ODELink.cc: " << _e.what();
+          }
+        }
         // Set surface properties max_vel and min_depth
         dBodySetMaxVel(this->linkId, g->GetODESurface()->maxVel);
         dBodySetMinDepth(this->linkId, g->GetODESurface()->minDepth);
@@ -347,6 +377,7 @@ void ODELink::UpdateSurface()
     }
   }
 }
+
 /////////////////////////////////////////////////////////////////////
 void ODELink::UpdateMass()
 {
@@ -380,6 +411,10 @@ void ODELink::UpdateMass()
     dBodySetMass(this->linkId, &odeMass);
   else
     gzthrow("Setting custom link " + this->GetScopedName() + "mass to zero!");
+
+  // In case the center of mass changed:
+  this->UpdateCollisionOffsets();
+  this->OnPoseChange();
 }
 
 //////////////////////////////////////////////////
@@ -589,6 +624,31 @@ void ODELink::AddForceAtWorldPosition(const math::Vector3 &_force,
   }
 }
 
+//////////////////////////////////////////////////
+void ODELink::AddLinkForce(const math::Vector3 &_force,
+    const math::Vector3 &_offset)
+{
+  if (this->linkId)
+  {
+    // Force vector represents a direction only, so it should be rotated but
+    // not translated
+    math::Vector3 forceWorld = this->GetWorldPose().rot.RotateVector(_force);
+    // Does this need to be rotated?
+    math::Vector3 offsetCoG = _offset - this->inertial->GetCoG();
+
+    this->SetEnabled(true);
+    dBodyAddForceAtRelPos(this->linkId,
+        forceWorld.x, forceWorld.y, forceWorld.z,
+        offsetCoG.x, offsetCoG.y, offsetCoG.z);
+  }
+  else if (!this->IsStatic())
+  {
+    gzlog << "ODE body for link [" << this->GetScopedName() << "]"
+          << " does not exist, unable to AddLinkForce"
+          << std::endl;
+  }
+}
+
 /////////////////////////////////////////////////
 void ODELink::AddTorque(const math::Vector3 &_torque)
 {
