@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Open Source Robotics Foundation
+ * Copyright (C) 2012-2016 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -19,6 +19,7 @@
  * Date: 13 Feb 2006
  */
 
+#include <boost/bind.hpp>
 #include <boost/thread.hpp>
 
 #include "gazebo/common/Assert.hh"
@@ -111,10 +112,16 @@ void SimbodyLink::Init()
 //////////////////////////////////////////////////
 void SimbodyLink::Fini()
 {
+  event::Events::DisconnectWorldUpdateBegin(this->gravityModeConnection);
   event::Events::DisconnectWorldUpdateEnd(this->staticLinkConnection);
   Link::Fini();
 }
 
+/////////////////////////////////////////////////////////////////////
+void SimbodyLink::UpdateMass()
+{
+}
+
 //////////////////////////////////////////////////
 void SimbodyLink::SetGravityMode(bool _mode)
 {
@@ -525,6 +532,14 @@ void SimbodyLink::AddForceAtRelativePosition(const math::Vector3 &/*_force*/,
   gzerr << "Not implemented.\n";
 }
 
+//////////////////////////////////////////////////
+void SimbodyLink::AddLinkForce(const math::Vector3 &/*_force*/,
+    const math::Vector3 &/*_offset*/)
+{
+  gzlog << "SimbodyLink::AddLinkForce not yet implemented (issue #1478)."
+        << std::endl;
+}
+
 /////////////////////////////////////////////////
 void SimbodyLink::AddTorque(const math::Vector3 &/*_torque*/)
 {
