@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Open Source Robotics Foundation
+ * Copyright (C) 2012-2016 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -24,6 +24,11 @@
  * @update  2007 by independentCreations see independentCreations@gmail.com
  */
 
+#ifdef _WIN32
+  // Ensure that windows types like LONGLONG are defined
+  #include <windows.h>
+#endif
+
 #include <boost/thread/recursive_mutex.hpp>
 
 #include "gazebo/common/common.hh"
