@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Open Source Robotics Foundation
+ * Copyright (C) 2012-2016 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,15 +15,22 @@
  *
 */
 
+#ifdef _WIN32
+  // Ensure that Winsock2.h is included before Windows.h, which can get
+  // pulled in by anybody (e.g., Boost).
+  #include <Winsock2.h>
+#endif
+
 #include <sys/stat.h>
-#include <boost/bind.hpp>
 
 #include "gazebo/common/Console.hh"
 #include "gazebo/common/Exception.hh"
 #include "gazebo/common/SystemPaths.hh"
+#include "gazebo/rendering/ogre_gazebo.h"
 #include "gazebo/rendering/RenderEngine.hh"
 #include "gazebo/rendering/Scene.hh"
 #include "gazebo/rendering/Visual.hh"
+#include "gazebo/rendering/RTShaderSystemPrivate.hh"
 #include "gazebo/rendering/RTShaderSystem.hh"
 
 #define MINOR_VERSION 7
@@ -32,18 +39,20 @@ using namespace rendering;
 
 //////////////////////////////////////////////////
 RTShaderSystem::RTShaderSystem()
+  : dataPtr(new RTShaderSystemPrivate)
 {
-  this->entityMutex = new boost::mutex();
-  this->initialized = false;
-  this->shadowsApplied = false;
-  this->pssmSetup.setNull();
+  this->dataPtr->initialized = false;
+  this->dataPtr->shadowsApplied = false;
+  this->dataPtr->pssmSetup.setNull();
+  this->dataPtr->updateShaders = false;
 }
 
 //////////////////////////////////////////////////
 RTShaderSystem::~RTShaderSystem()
 {
   this->Fini();
-  delete this->entityMutex;
+  delete this->dataPtr;
+  this->dataPtr = NULL;
 }
 
 //////////////////////////////////////////////////
@@ -58,33 +67,33 @@ void RTShaderSystem::Init()
 
   if (Ogre::RTShader::ShaderGenerator::initialize())
   {
-    this->initialized = true;
+    this->dataPtr->initialized = true;
 
     std::string coreLibsPath, cachePath;
     this->GetPaths(coreLibsPath, cachePath);
 
     // Get the shader generator pointer
-    this->shaderGenerator = Ogre::RTShader::ShaderGenerator::getSingletonPtr();
+    this->dataPtr->shaderGenerator =
+        Ogre::RTShader::ShaderGenerator::getSingletonPtr();
 
     // Add the shader libs resource location
     Ogre::ResourceGroupManager::getSingleton().addResourceLocation(
         coreLibsPath, "FileSystem");
 
     // Set shader cache path.
-    this->shaderGenerator->setShaderCachePath(cachePath);
+    this->dataPtr->shaderGenerator->setShaderCachePath(cachePath);
 
-    this->shaderGenerator->setTargetLanguage("glsl");
+    this->dataPtr->shaderGenerator->setTargetLanguage("glsl");
   }
   else
     gzerr << "RT Shader system failed to initialize\n";
-
 #endif
 }
 
 //////////////////////////////////////////////////
 void RTShaderSystem::Fini()
 {
-  if (!this->initialized)
+  if (!this->dataPtr->initialized)
     return;
 
   // Restore default scheme.
@@ -92,20 +101,20 @@ void RTShaderSystem::Fini()
       Ogre::MaterialManager::DEFAULT_SCHEME_NAME);
 
   // Finalize RTShader system.
-  if (this->shaderGenerator != NULL)
+  if (this->dataPtr->shaderGenerator != NULL)
   {
-#if (OGRE_VERSION < ((1 << 16) | (9 << 8) | 0))
+    // On Windows, we're using 1.9RC1, which doesn't have a bunch of changes.
+#if (OGRE_VERSION < ((1 << 16) | (9 << 8) | 0)) || defined(_WIN32)
     Ogre::RTShader::ShaderGenerator::finalize();
 #else
     Ogre::RTShader::ShaderGenerator::destroy();
 #endif
-    this->shaderGenerator = NULL;
+    this->dataPtr->shaderGenerator = NULL;
   }
 
-  this->pssmSetup.setNull();
-  this->entities.clear();
-  this->scenes.clear();
-  this->initialized = false;
+  this->dataPtr->pssmSetup.setNull();
+  this->dataPtr->scenes.clear();
+  this->dataPtr->initialized = false;
 }
 
 #if INCLUDE_RTSHADER && OGRE_VERSION_MAJOR >= 1 &&\
@@ -113,14 +122,14 @@ void RTShaderSystem::Fini()
 //////////////////////////////////////////////////
 void RTShaderSystem::AddScene(ScenePtr _scene)
 {
-  if (!this->initialized)
+  if (!this->dataPtr->initialized)
     return;
 
   // Set the scene manager
-  this->shaderGenerator->addSceneManager(_scene->GetManager());
-  this->shaderGenerator->createScheme(_scene->GetName() +
+  this->dataPtr->shaderGenerator->addSceneManager(_scene->OgreSceneManager());
+  this->dataPtr->shaderGenerator->createScheme(_scene->Name() +
       Ogre::RTShader::ShaderGenerator::DEFAULT_SCHEME_NAME);
-  this->scenes.push_back(_scene);
+  this->dataPtr->scenes.push_back(_scene);
 }
 #else
 void RTShaderSystem::AddScene(ScenePtr /*_scene*/)
@@ -131,60 +140,64 @@ void RTShaderSystem::AddScene(ScenePtr /*_scene*/)
 //////////////////////////////////////////////////
 void RTShaderSystem::RemoveScene(ScenePtr _scene)
 {
-  if (!this->initialized)
+  if (!this->dataPtr->initialized)
     return;
 
-  std::vector<ScenePtr>::iterator iter;
-  for (iter = this->scenes.begin(); iter != scenes.end(); ++iter)
+  auto iter = this->dataPtr->scenes.begin();
+  for (; iter != this->dataPtr->scenes.end(); ++iter)
     if ((*iter) == _scene)
       break;
 
-  if (iter != this->scenes.end())
+  if (iter != this->dataPtr->scenes.end())
   {
-    this->scenes.erase(iter);
-    this->shaderGenerator->invalidateScheme(_scene->GetName() +
+    this->dataPtr->scenes.erase(iter);
+    this->dataPtr->shaderGenerator->invalidateScheme(_scene->Name() +
         Ogre::RTShader::ShaderGenerator::DEFAULT_SCHEME_NAME);
-    this->shaderGenerator->removeSceneManager(_scene->GetManager());
-    this->shaderGenerator->removeAllShaderBasedTechniques();
-    this->shaderGenerator->flushShaderCache();
-    // this->UpdateShaders();
+    this->dataPtr->shaderGenerator->removeSceneManager(
+        _scene->OgreSceneManager());
+    this->dataPtr->shaderGenerator->removeAllShaderBasedTechniques();
+    this->dataPtr->shaderGenerator->flushShaderCache();
   }
 }
 
 //////////////////////////////////////////////////
-void RTShaderSystem::AttachEntity(Visual *vis)
+void RTShaderSystem::RemoveScene(const std::string &_scene)
 {
-  if (!this->initialized)
+  if (!this->dataPtr->initialized)
     return;
 
-  this->entityMutex->lock();
-  this->entities.push_back(vis);
-  this->entityMutex->unlock();
-  this->GenerateShaders(vis);
+  for (auto iter : this->dataPtr->scenes)
+  {
+    if (iter->Name() == _scene)
+    {
+      this->RemoveScene(iter);
+      return;
+    }
+  }
 }
 
 //////////////////////////////////////////////////
-void RTShaderSystem::DetachEntity(Visual *_vis)
+void RTShaderSystem::AttachEntity(Visual * /*_vis*/)
 {
-  if (!this->initialized)
-    return;
+  return;
+}
 
-  boost::mutex::scoped_lock lock(*this->entityMutex);
-  this->entities.remove(_vis);
+//////////////////////////////////////////////////
+void RTShaderSystem::DetachEntity(Visual * /*_vis*/)
+{
+  return;
 }
 
 //////////////////////////////////////////////////
 void RTShaderSystem::Clear()
 {
-  boost::mutex::scoped_lock lock(*this->entityMutex);
-  this->entities.clear();
 }
 
 //////////////////////////////////////////////////
 void RTShaderSystem::AttachViewport(Ogre::Viewport *_viewport, ScenePtr _scene)
 {
 #if OGRE_VERSION_MAJOR == 1 && OGRE_VERSION_MINOR >= 7
-  _viewport->setMaterialScheme(_scene->GetName() +
+  _viewport->setMaterialScheme(_scene->Name() +
       Ogre::RTShader::ShaderGenerator::DEFAULT_SCHEME_NAME);
 #endif
 }
@@ -193,35 +206,47 @@ void RTShaderSystem::AttachViewport(Ogre::Viewport *_viewport, ScenePtr _scene)
 void RTShaderSystem::DetachViewport(Ogre::Viewport *_viewport, ScenePtr _scene)
 {
 #if OGRE_VERSION_MAJOR == 1 && OGRE_VERSION_MINOR >= 7
-  if (_viewport && _scene && _scene->GetInitialized())
-    _viewport->setMaterialScheme(_scene->GetName());
+  if (_viewport && _scene && _scene->Initialized())
+    _viewport->setMaterialScheme(_scene->Name());
 #endif
 }
 
 //////////////////////////////////////////////////
 void RTShaderSystem::UpdateShaders()
 {
-  if (!this->initialized)
-    return;
-
-  std::list<Visual*>::iterator iter;
-
-  boost::mutex::scoped_lock lock(*this->entityMutex);
-
-  // Update all the shaders
-  for (iter = this->entities.begin(); iter != this->entities.end(); ++iter)
-    this->GenerateShaders(*iter);
+  // shaders will be updated in the Update call on pre-render event.
+  this->dataPtr->updateShaders = true;
 }
 
 //////////////////////////////////////////////////
-void RTShaderSystem::GenerateShaders(Visual *vis)
+void RTShaderSystem::UpdateShaders(VisualPtr _vis)
 {
-  if (!this->initialized)
+  if (_vis)
+  {
+    if (_vis->UseRTShader())
+      this->GenerateShaders(_vis);
+    for (unsigned int i = 0; i < _vis->GetChildCount(); ++i)
+      this->UpdateShaders(_vis->GetChild(i));
+  }
+}
+
+//////////////////////////////////////////////////
+void RTShaderSystem::GenerateShaders(Visual *_vis)
+{
+  VisualPtr vis(_vis);
+  this->GenerateShaders(vis);
+}
+
+//////////////////////////////////////////////////
+void RTShaderSystem::GenerateShaders(const VisualPtr &_vis)
+{
+  if (!this->dataPtr->initialized || !_vis)
     return;
 
-  for (unsigned int k = 0; k < vis->GetSceneNode()->numAttachedObjects(); k++)
+  for (unsigned int k = 0; _vis->GetSceneNode() &&
+      k < _vis->GetSceneNode()->numAttachedObjects(); ++k)
   {
-    Ogre::MovableObject *obj = vis->GetSceneNode()->getAttachedObject(k);
+    Ogre::MovableObject *obj = _vis->GetSceneNode()->getAttachedObject(k);
     Ogre::Entity *entity = dynamic_cast<Ogre::Entity*>(obj);
     if (!entity)
       continue;
@@ -232,14 +257,14 @@ void RTShaderSystem::GenerateShaders(Visual *vis)
       const Ogre::String& curMaterialName = curSubEntity->getMaterialName();
       bool success = false;
 
-      for (unsigned int s = 0; s < this->scenes.size(); s++)
+      for (unsigned int s = 0; s < this->dataPtr->scenes.size(); s++)
       {
         try
         {
-          success = this->shaderGenerator->createShaderBasedTechnique(
+          success = this->dataPtr->shaderGenerator->createShaderBasedTechnique(
               curMaterialName,
               Ogre::MaterialManager::DEFAULT_SCHEME_NAME,
-              this->scenes[s]->GetName() +
+              this->dataPtr->scenes[s]->Name() +
               Ogre::RTShader::ShaderGenerator::DEFAULT_SCHEME_NAME);
         }
         catch(Ogre::Exception &e)
@@ -259,8 +284,8 @@ void RTShaderSystem::GenerateShaders(Visual *vis)
           // NOTE:For more complicated samples iterate over the passes and build
           // each one of them as desired.
           Ogre::RTShader::RenderState* renderState =
-            this->shaderGenerator->getRenderState(
-                this->scenes[s]->GetName() +
+            this->dataPtr->shaderGenerator->getRenderState(
+                this->dataPtr->scenes[s]->Name() +
                 Ogre::RTShader::ShaderGenerator::DEFAULT_SCHEME_NAME,
                 curMaterialName, 0);
 
@@ -268,10 +293,10 @@ void RTShaderSystem::GenerateShaders(Visual *vis)
           renderState->reset();
 
           /// This doesn't seem to work properly.
-          if (vis->GetShaderType() == "normal_map_object_space")
+          if (_vis->GetShaderType() == "normal_map_object_space")
           {
             Ogre::RTShader::SubRenderState* subRenderState =
-              this->shaderGenerator->createSubRenderState(
+              this->dataPtr->shaderGenerator->createSubRenderState(
                   Ogre::RTShader::NormalMapLighting::Type);
 
             Ogre::RTShader::NormalMapLighting* normalMapSubRS =
@@ -280,13 +305,13 @@ void RTShaderSystem::GenerateShaders(Visual *vis)
             normalMapSubRS->setNormalMapSpace(
                 Ogre::RTShader::NormalMapLighting::NMS_OBJECT);
 
-            normalMapSubRS->setNormalMapTextureName(vis->GetNormalMap());
+            normalMapSubRS->setNormalMapTextureName(_vis->GetNormalMap());
             renderState->addTemplateSubRenderState(normalMapSubRS);
           }
-          else if (vis->GetShaderType() == "normal_map_tangent_space")
+          else if (_vis->GetShaderType() == "normal_map_tangent_space")
           {
             Ogre::RTShader::SubRenderState* subRenderState =
-              this->shaderGenerator->createSubRenderState(
+              this->dataPtr->shaderGenerator->createSubRenderState(
                   Ogre::RTShader::NormalMapLighting::Type);
 
             Ogre::RTShader::NormalMapLighting* normalMapSubRS =
@@ -295,14 +320,14 @@ void RTShaderSystem::GenerateShaders(Visual *vis)
             normalMapSubRS->setNormalMapSpace(
                 Ogre::RTShader::NormalMapLighting::NMS_TANGENT);
 
-            normalMapSubRS->setNormalMapTextureName(vis->GetNormalMap());
+            normalMapSubRS->setNormalMapTextureName(_vis->GetNormalMap());
 
             renderState->addTemplateSubRenderState(normalMapSubRS);
           }
-          else if (vis->GetShaderType() == "vertex")
+          else if (_vis->GetShaderType() == "vertex")
           {
             Ogre::RTShader::SubRenderState *perPerVertexLightModel =
-              this->shaderGenerator->createSubRenderState(
+              this->dataPtr->shaderGenerator->createSubRenderState(
                   Ogre::RTShader::FFPLighting::Type);
 
             renderState->addTemplateSubRenderState(perPerVertexLightModel);
@@ -310,7 +335,7 @@ void RTShaderSystem::GenerateShaders(Visual *vis)
           else
           {
             Ogre::RTShader::SubRenderState *perPixelLightModel =
-              this->shaderGenerator->createSubRenderState(
+              this->dataPtr->shaderGenerator->createSubRenderState(
                   Ogre::RTShader::PerPixelLighting::Type);
 
             renderState->addTemplateSubRenderState(perPixelLightModel);
@@ -318,8 +343,8 @@ void RTShaderSystem::GenerateShaders(Visual *vis)
 
 
           // Invalidate this material in order to re-generate its shaders.
-          this->shaderGenerator->invalidateMaterial(
-              this->scenes[s]->GetName() +
+          this->dataPtr->shaderGenerator->invalidateMaterial(
+              this->dataPtr->scenes[s]->Name() +
               Ogre::RTShader::ShaderGenerator::DEFAULT_SCHEME_NAME,
               curMaterialName);
         }
@@ -331,7 +356,7 @@ void RTShaderSystem::GenerateShaders(Visual *vis)
 //////////////////////////////////////////////////
 bool RTShaderSystem::GetPaths(std::string &coreLibsPath, std::string &cachePath)
 {
-  if (!this->initialized)
+  if (!this->dataPtr->initialized)
     return false;
 
   Ogre::StringVector groupVector;
@@ -386,7 +411,11 @@ bool RTShaderSystem::GetPaths(std::string &coreLibsPath, std::string &cachePath)
           stream << tmpdir << "/gazebo-" << user << "-rtshaderlibcache" << "/";
           cachePath = stream.str();
           // Create the directory
+#ifdef _WIN32
+          if (mkdir(cachePath.c_str()) != 0)
+#else
           if (mkdir(cachePath.c_str(), S_IRUSR | S_IWUSR | S_IXUSR) != 0)
+#endif
           {
             if (errno != EEXIST)
             {
@@ -420,37 +449,39 @@ bool RTShaderSystem::GetPaths(std::string &coreLibsPath, std::string &cachePath)
 /////////////////////////////////////////////////
 void RTShaderSystem::RemoveShadows(ScenePtr _scene)
 {
-  if (!this->initialized || !this->shadowsApplied)
+  if (!this->dataPtr->initialized || !this->dataPtr->shadowsApplied)
     return;
 
-  _scene->GetManager()->setShadowTechnique(Ogre::SHADOWTYPE_NONE);
-  _scene->GetManager()->setShadowCameraSetup(Ogre::ShadowCameraSetupPtr());
+  _scene->OgreSceneManager()->setShadowTechnique(Ogre::SHADOWTYPE_NONE);
+  _scene->OgreSceneManager()->setShadowCameraSetup(
+      Ogre::ShadowCameraSetupPtr());
 
   Ogre::RTShader::RenderState* schemeRenderState =
-    this->shaderGenerator->getRenderState(
-        _scene->GetName() +
+    this->dataPtr->shaderGenerator->getRenderState(
+        _scene->Name() +
         Ogre::RTShader::ShaderGenerator::DEFAULT_SCHEME_NAME);
 
-  schemeRenderState->removeTemplateSubRenderState(this->shadowRenderState);
+  schemeRenderState->removeTemplateSubRenderState(
+      this->dataPtr->shadowRenderState);
 
-  this->shaderGenerator->invalidateScheme(_scene->GetName() +
+  this->dataPtr->shaderGenerator->invalidateScheme(_scene->Name() +
       Ogre::RTShader::ShaderGenerator::DEFAULT_SCHEME_NAME);
   this->UpdateShaders();
 
-  this->shadowsApplied = false;
+  this->dataPtr->shadowsApplied = false;
 }
 
 /////////////////////////////////////////////////
 void RTShaderSystem::ApplyShadows(ScenePtr _scene)
 {
-  if (!this->initialized || this->shadowsApplied)
+  if (!this->dataPtr->initialized || this->dataPtr->shadowsApplied)
     return;
 
-  Ogre::SceneManager *sceneMgr = _scene->GetManager();
+  Ogre::SceneManager *sceneMgr = _scene->OgreSceneManager();
 
   // Grab the scheme render state.
   Ogre::RTShader::RenderState* schemRenderState =
-    this->shaderGenerator->getRenderState(_scene->GetName() +
+    this->dataPtr->shaderGenerator->getRenderState(_scene->Name() +
         Ogre::RTShader::ShaderGenerator::DEFAULT_SCHEME_NAME);
 
   sceneMgr->setShadowTechnique(Ogre::SHADOWTYPE_TEXTURE_ADDITIVE_INTEGRATED);
@@ -480,18 +511,19 @@ void RTShaderSystem::ApplyShadows(ScenePtr _scene)
   // pssmCasterPass->setFog(true);
 
   // shadow camera setup
-  if (this->pssmSetup.isNull())
+  if (this->dataPtr->pssmSetup.isNull())
   {
-    this->pssmSetup =
+    this->dataPtr->pssmSetup =
         Ogre::ShadowCameraSetupPtr(new Ogre::PSSMShadowCameraSetup());
   }
 
   double shadowFarDistance = 500;
-  double cameraNearClip = 0.1;
+  double cameraNearClip = 0.01;
   sceneMgr->setShadowFarDistance(shadowFarDistance);
 
   Ogre::PSSMShadowCameraSetup *cameraSetup =
-      dynamic_cast<Ogre::PSSMShadowCameraSetup*>(this->pssmSetup.get());
+      dynamic_cast<Ogre::PSSMShadowCameraSetup*>(
+      this->dataPtr->pssmSetup.get());
 
   cameraSetup->calculateSplitPoints(3, cameraNearClip, shadowFarDistance);
   cameraSetup->setSplitPadding(4);
@@ -499,7 +531,7 @@ void RTShaderSystem::ApplyShadows(ScenePtr _scene)
   cameraSetup->setOptimalAdjustFactor(1, 1);
   cameraSetup->setOptimalAdjustFactor(2, .5);
 
-  sceneMgr->setShadowCameraSetup(this->pssmSetup);
+  sceneMgr->setShadowCameraSetup(this->dataPtr->pssmSetup);
 
   // These values do not seem to help at all. Leaving here until I have time
   // to properly fix shadow z-fighting.
@@ -507,10 +539,12 @@ void RTShaderSystem::ApplyShadows(ScenePtr _scene)
   // cameraSetup->setOptimalAdjustFactor(1, 1);
   // cameraSetup->setOptimalAdjustFactor(2, 0.5);
 
-  this->shadowRenderState = this->shaderGenerator->createSubRenderState(
+  this->dataPtr->shadowRenderState =
+      this->dataPtr->shaderGenerator->createSubRenderState(
       Ogre::RTShader::IntegratedPSSM3::Type);
   Ogre::RTShader::IntegratedPSSM3 *pssm3SubRenderState =
-    static_cast<Ogre::RTShader::IntegratedPSSM3*>(this->shadowRenderState);
+      static_cast<Ogre::RTShader::IntegratedPSSM3*>(
+      this->dataPtr->shadowRenderState);
 
   const Ogre::PSSMShadowCameraSetup::SplitPointList &srcSplitPoints =
     cameraSetup->getSplitPoints();
@@ -523,18 +557,36 @@ void RTShaderSystem::ApplyShadows(ScenePtr _scene)
   }
 
   pssm3SubRenderState->setSplitPoints(dstSplitPoints);
-  schemRenderState->addTemplateSubRenderState(this->shadowRenderState);
+  schemRenderState->addTemplateSubRenderState(this->dataPtr->shadowRenderState);
 
-  this->shaderGenerator->invalidateScheme(_scene->GetName() +
+  this->dataPtr->shaderGenerator->invalidateScheme(_scene->Name() +
       Ogre::RTShader::ShaderGenerator::DEFAULT_SCHEME_NAME);
 
   this->UpdateShaders();
 
-  this->shadowsApplied = true;
+  this->dataPtr->shadowsApplied = true;
 }
 
 /////////////////////////////////////////////////
 Ogre::PSSMShadowCameraSetup *RTShaderSystem::GetPSSMShadowCameraSetup() const
 {
-  return dynamic_cast<Ogre::PSSMShadowCameraSetup *>(this->pssmSetup.get());
+  return dynamic_cast<Ogre::PSSMShadowCameraSetup *>(
+      this->dataPtr->pssmSetup.get());
+}
+
+/////////////////////////////////////////////////
+void RTShaderSystem::Update()
+{
+  if (!this->dataPtr->initialized || !this->dataPtr->updateShaders)
+    return;
+
+  for (const auto &scene : this->dataPtr->scenes)
+  {
+    VisualPtr vis = scene->WorldVisual();
+    if (vis)
+    {
+      this->UpdateShaders(vis);
+    }
+  }
+  this->dataPtr->updateShaders = false;
 }
