@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Open Source Robotics Foundation
+ * Copyright (C) 2012-2016 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -22,3 +22,5 @@ using namespace rendering;
 
 event::EventT<void (const std::string &)> Events::createScene;
 event::EventT<void (const std::string &)> Events::removeScene;
+event::EventT<void (const int32_t)> Events::toggleLayer;
+event::EventT<void (const int32_t)> Events::newLayer;
