@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Open Source Robotics Foundation
+ * Copyright (C) 2012-2016 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,11 +14,14 @@
  * limitations under the License.
  *
 */
-/*
- * Desc: Class to manager all sensors
- * Author: Nate Koenig
- * Date: 18 Dec 2009
- */
+#ifdef _WIN32
+  // Ensure that Winsock2.h is included before Windows.h, which can get
+  // pulled in by anybody (e.g., Boost).
+  #include <Winsock2.h>
+#endif
+
+#include <functional>
+#include <boost/bind.hpp>
 #include "gazebo/common/Assert.hh"
 #include "gazebo/common/Time.hh"
 
@@ -101,20 +104,31 @@ void SensorManager::Update(bool _force)
   {
     boost::recursive_mutex::scoped_lock lock(this->mutex);
 
-    // in case things are spawn, sensors length changes
-    for (Sensor_V::iterator iter = this->initSensors.begin();
-         iter != this->initSensors.end(); ++iter)
+    if (this->worlds.empty() && physics::worlds_running() && this->initialized)
     {
-      GZ_ASSERT((*iter) != NULL, "Sensor pointer is NULL");
-      GZ_ASSERT((*iter)->GetCategory() < 0 ||
-          (*iter)->GetCategory() < CATEGORY_COUNT, "Sensor category is empty");
-      GZ_ASSERT(this->sensorContainers[(*iter)->GetCategory()] != NULL,
-                "Sensor container is NULL");
+      auto world = physics::get_world();
+      this->worlds[world->GetName()] = world;
+      world->_SetSensorsInitialized(true);
+    }
 
-      (*iter)->Init();
-      this->sensorContainers[(*iter)->GetCategory()]->AddSensor(*iter);
+    if (!this->initSensors.empty())
+    {
+      // in case things are spawned, sensors length changes
+      for (auto &sensor : this->initSensors)
+      {
+        GZ_ASSERT(sensor != NULL, "Sensor pointer is NULL");
+        GZ_ASSERT(sensor->Category() < 0 ||
+            sensor->Category() < CATEGORY_COUNT, "Sensor category is empty");
+        GZ_ASSERT(this->sensorContainers[sensor->Category()] != NULL,
+            "Sensor container is NULL");
+
+        sensor->Init();
+        this->sensorContainers[sensor->Category()]->AddSensor(sensor);
+      }
+      this->initSensors.clear();
+      for (auto &worldName_worldPtr : this->worlds)
+        worldName_worldPtr.second->_SetSensorsInitialized(true);
     }
-    this->initSensors.clear();
 
     for (std::vector<std::string>::iterator iter = this->removeSensors.begin();
          iter != this->removeSensors.end(); ++iter)
@@ -192,6 +206,20 @@ void SensorManager::Init()
     (*iter)->Init();
   }
 
+  // Connect to the time reset event.
+  this->timeResetConnection = event::Events::ConnectTimeReset(
+      std::bind(&SensorManager::ResetLastUpdateTimes, this));
+
+  // Connect to the remove sensor event.
+  this->removeSensorConnection = event::Events::ConnectRemoveSensor(
+      std::bind(&SensorManager::RemoveSensor, this, std::placeholders::_1));
+
+  // Connect to the create sensor event.
+  this->createSensorConnection = event::Events::ConnectCreateSensor(
+      std::bind(&SensorManager::OnCreateSensor, this,
+        std::placeholders::_1, std::placeholders::_2,
+        std::placeholders::_3, std::placeholders::_4));
+
   this->initialized = true;
 }
 
@@ -210,6 +238,8 @@ void SensorManager::Fini()
   }
 
   this->removeSensors.clear();
+  this->initSensors.clear();
+  this->worlds.clear();
 
   delete this->simTimeEventHandler;
   this->simTimeEventHandler = NULL;
@@ -223,6 +253,15 @@ void SensorManager::GetSensorTypes(std::vector<std::string> &_types) const
   sensors::SensorFactory::GetSensorTypes(_types);
 }
 
+//////////////////////////////////////////////////
+void SensorManager::OnCreateSensor(sdf::ElementPtr _elem,
+    const std::string &_worldName,
+    const std::string &_parentName,
+    const uint32_t _parentId)
+{
+  this->CreateSensor(_elem, _worldName, _parentName, _parentId);
+}
+
 //////////////////////////////////////////////////
 std::string SensorManager::CreateSensor(sdf::ElementPtr _elem,
                                         const std::string &_worldName,
@@ -243,23 +282,25 @@ std::string SensorManager::CreateSensor(sdf::ElementPtr _elem,
 
   // Load the sensor
   sensor->Load(_worldName, _elem);
+  this->worlds[_worldName] = physics::get_world(_worldName);
 
   // If the SensorManager has not been initialized, then it's okay to push
   // the sensor into one of the sensor vectors because the sensor will get
   // initialized in SensorManager::Init
   if (!this->initialized)
   {
-    this->sensorContainers[sensor->GetCategory()]->AddSensor(sensor);
+    this->sensorContainers[sensor->Category()]->AddSensor(sensor);
   }
   // Otherwise the SensorManager is already running, and the sensor will get
   // initialized during the next SensorManager::Update call.
   else
   {
     boost::recursive_mutex::scoped_lock lock(this->mutex);
+    this->worlds[_worldName]->_SetSensorsInitialized(false);
     this->initSensors.push_back(sensor);
   }
 
-  return sensor->GetScopedName();
+  return sensor->ScopedName();
 }
 
 //////////////////////////////////////////////////
@@ -346,7 +387,7 @@ void SensorManager::RemoveSensor(const std::string &_name)
   {
     // Push it on the list, to be removed by the main sensor thread,
     // to ensure correct access to rendering resources.
-    this->removeSensors.push_back(sensor->GetScopedName());
+    this->removeSensors.push_back(sensor->ScopedName());
   }
 }
 
@@ -376,11 +417,10 @@ void SensorManager::SensorContainer::Init()
 {
   boost::recursive_mutex::scoped_lock lock(this->mutex);
 
-  Sensor_V::iterator iter;
-  for (iter = this->sensors.begin(); iter != this->sensors.end(); ++iter)
+  for (auto &sensor : this->sensors)
   {
-    GZ_ASSERT((*iter) != NULL, "Sensor is NULL");
-    (*iter)->Init();
+    GZ_ASSERT(sensor != NULL, "Sensor is NULL");
+    sensor->Init();
   }
 
   this->initialized = true;
@@ -467,7 +507,7 @@ void SensorManager::SensorContainer::RunLoop()
         iter != this->sensors.end() && !this->stop; ++iter)
     {
       GZ_ASSERT((*iter) != NULL, "Sensor is NULL");
-      maxUpdateRate = std::max((*iter)->GetUpdateRate(), maxUpdateRate);
+      maxUpdateRate = std::max((*iter)->UpdateRate(), maxUpdateRate);
     }
   }
 
@@ -555,9 +595,9 @@ SensorPtr SensorManager::SensorContainer::GetSensor(const std::string &_name,
     GZ_ASSERT((*iter) != NULL, "Sensor is NULL");
 
     // We match on the scoped name (model::link::sensor) because multiple
-    // sensors with the name leaf name make exists in a world.
-    if ((_useLeafName && (*iter)->GetName() == _name) ||
-        (!_useLeafName && (*iter)->GetScopedName() == _name))
+    // sensors with the same leaf name may exist in a world.
+    if ((_useLeafName && (*iter)->Name() == _name) ||
+        (!_useLeafName && (*iter)->ScopedName() == _name))
     {
       result = (*iter);
       break;
@@ -595,7 +635,7 @@ bool SensorManager::SensorContainer::RemoveSensor(const std::string &_name)
   {
     GZ_ASSERT((*iter) != NULL, "Sensor is NULL");
 
-    if ((*iter)->GetScopedName() == _name)
+    if ((*iter)->ScopedName() == _name)
     {
       (*iter)->Fini();
       this->sensors.erase(iter);
