@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Open Source Robotics Foundation
+ * Copyright (C) 2012-2016 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -18,17 +18,15 @@
 
 #include "gazebo/msgs/msgs.hh"
 #include "gazebo/physics/physics.hh"
-#include "gazebo/physics/ode/ODESurfaceParams.hh"
-#include "gazebo/physics/ode/ODETypes.hh"
 
 #ifdef HAVE_BULLET
-#include "gazebo/physics/bullet/BulletSurfaceParams.hh"
-#include "gazebo/physics/bullet/BulletTypes.hh"
+#include "gazebo/physics/bullet/bullet_math_inc.h"
 #endif
 
 #include "gazebo/transport/transport.hh"
-#include "ServerFixture.hh"
-#include "helper_physics_generator.hh"
+#include "gazebo/test/ServerFixture.hh"
+#include "gazebo/test/helper_physics_generator.hh"
+#include "gazebo/gazebo_config.h"
 
 using namespace gazebo;
 
@@ -37,7 +35,7 @@ const double g_friction_tolerance = 1e-3;
 class PhysicsFrictionTest : public ServerFixture,
                         public testing::WithParamInterface<const char*>
 {
-  protected: PhysicsFrictionTest() : ServerFixture(), spawnCount(0)
+  protected: PhysicsFrictionTest() : ServerFixture()
              {
              }
 
@@ -57,29 +55,10 @@ class PhysicsFrictionTest : public ServerFixture,
               physics::Collision_V::iterator iter = collisions.begin();
               if (iter != collisions.end())
               {
-                physics::PhysicsEnginePtr physics = world->GetPhysicsEngine();
-                if (physics->GetType() == "ode")
-                {
-                  physics::ODESurfaceParamsPtr surface =
-                    boost::dynamic_pointer_cast<physics::ODESurfaceParams>(
-                    (*iter)->GetSurface());
-                  // Average the mu1 and mu2 values
-                  this->friction = (surface->frictionPyramid.GetMuPrimary()
-                                  + surface->frictionPyramid.GetMuSecondary())
-                                  / 2.0;
-                }
-#ifdef HAVE_BULLET
-                else if (physics->GetType() == "bullet")
-                {
-                  physics::BulletSurfaceParamsPtr surface =
-                    boost::dynamic_pointer_cast<physics::BulletSurfaceParams>(
-                    (*iter)->GetSurface());
-                  // Average the mu1 and mu2 values
-                  this->friction = (surface->frictionPyramid.GetMuPrimary()
-                                  + surface->frictionPyramid.GetMuSecondary())
-                                  / 2.0;
-                }
-#endif
+                physics::SurfaceParamsPtr surf = (*iter)->GetSurface();
+                // Use the Secondary friction value,
+                // since gravity has a non-zero component in the y direction
+                this->friction = surf->FrictionPyramid()->MuSecondary();
               }
             }
     public: ~FrictionDemoBox() {}
@@ -104,22 +83,22 @@ class PhysicsFrictionTest : public ServerFixture,
             }
 
     /// \brief Size of box to spawn.
-    public: math::Vector3 size;
+    public: ignition::math::Vector3d size;
 
     /// \brief Mass of box to spawn (inertia computed automatically).
     public: double mass;
 
     /// \brief Model pose.
-    public: math::Pose modelPose;
+    public: ignition::math::Pose3d modelPose;
 
     /// \brief Link pose.
-    public: math::Pose linkPose;
+    public: ignition::math::Pose3d linkPose;
 
     /// \brief Inertial pose.
-    public: math::Pose inertialPose;
+    public: ignition::math::Pose3d inertialPose;
 
     /// \brief Collision pose.
-    public: math::Pose collisionPose;
+    public: ignition::math::Pose3d collisionPose;
 
     /// \brief Friction coefficient in primary direction.
     public: double friction1;
@@ -128,97 +107,50 @@ class PhysicsFrictionTest : public ServerFixture,
     public: double friction2;
 
     /// \brief Primary friction direction.
-    public: math::Vector3 direction1;
+    public: ignition::math::Vector3d direction1;
   };
 
   /// \brief Spawn a box with friction coefficients and direction.
   /// \param[in] _opt Options for friction box.
   public: physics::ModelPtr SpawnBox(const SpawnFrictionBoxOptions &_opt)
           {
-            msgs::Factory msg;
-            std::ostringstream modelStr;
-            std::ostringstream modelName;
-            modelName << "box_model" << this->spawnCount++;
+            std::string modelName = this->GetUniqueString("box_model");
 
-            double dx = _opt.size.x;
-            double dy = _opt.size.y;
-            double dz = _opt.size.z;
-            double ixx = _opt.mass/12.0 * (dy*dy + dz*dz);
-            double iyy = _opt.mass/12.0 * (dz*dz + dx*dx);
-            double izz = _opt.mass/12.0 * (dx*dx + dy*dy);
+            msgs::Model model;
+            model.set_name(modelName);
+            msgs::Set(model.mutable_pose(), _opt.modelPose);
 
-            modelStr
-              << "<sdf version='" << SDF_VERSION << "'>"
-              << "<model name ='" << modelName.str() << "'>"
-              << "  <pose>" << _opt.modelPose << "</pose>"
-              << "  <link name='link'>"
-              << "    <pose>" << _opt.linkPose << "</pose>"
-              << "    <inertial>"
-              << "      <pose>" << _opt.inertialPose << "</pose>"
-              << "      <mass>" << _opt.mass << "</mass>"
-              << "      <inertia>"
-              << "        <ixx>" << ixx << "</ixx>"
-              << "        <iyy>" << iyy << "</iyy>"
-              << "        <izz>" << izz << "</izz>"
-              << "        <ixy>" << 0.0 << "</ixy>"
-              << "        <ixz>" << 0.0 << "</ixz>"
-              << "        <iyz>" << 0.0 << "</iyz>"
-              << "      </inertia>"
-              << "    </inertial>"
-              << "    <collision name='collision'>"
-              << "      <pose>" << _opt.collisionPose << "</pose>"
-              << "      <geometry>"
-              << "        <box><size>" << _opt.size << "</size></box>"
-              << "      </geometry>"
-              << "      <surface>"
-              << "        <friction>"
-              << "          <ode>"
-              << "            <mu>" << _opt.friction1 << "</mu>"
-              << "            <mu2>" << _opt.friction2 << "</mu2>"
-              << "            <fdir1>" << _opt.direction1 << "</fdir1>"
-              << "          </ode>"
-              << "        </friction>"
-              << "      </surface>"
-              << "    </collision>"
-              << "    <visual name='visual'>"
-              << "      <pose>" << _opt.collisionPose << "</pose>"
-              << "      <geometry>"
-              << "        <box><size>" << _opt.size << "</size></box>"
-              << "      </geometry>"
-              << "    </visual>"
-              << "  </link>"
-              << "</model>";
+            msgs::AddBoxLink(model, _opt.mass, _opt.size);
+            auto link = model.mutable_link(0);
+            msgs::Set(link->mutable_pose(), _opt.linkPose);
 
-            physics::WorldPtr world = physics::get_world("default");
-            world->InsertModelString(modelStr.str());
-
-            physics::ModelPtr model;
-            common::Time wait(100, 0);
-
-            common::Time wallStart = common::Time::GetWallTime();
-            unsigned int waitCount = 0;
-            while (wait > (common::Time::GetWallTime() - wallStart) &&
-                   !this->HasEntity(modelName.str()))
             {
-              common::Time::MSleep(10);
-              if (++waitCount % 100 == 0)
-              {
-                gzwarn << "Waiting " << waitCount / 100 << " seconds for "
-                       << "box to spawn." << std::endl;
-              }
+              auto inertial = link->mutable_inertial();
+              msgs::Set(inertial->mutable_pose(), _opt.inertialPose);
             }
-            if (this->HasEntity(modelName.str()) && waitCount >= 100)
-              gzwarn << "box has spawned." << std::endl;
 
-            if (world != NULL)
-              model = world->GetModel(modelName.str());
+            auto collision = link->mutable_collision(0);
+            msgs::Set(collision->mutable_pose(), _opt.collisionPose);
 
-            return model;
+            auto friction = collision->mutable_surface()->mutable_friction();
+            friction->set_mu(_opt.friction1);
+            friction->set_mu2(_opt.friction2);
+            msgs::Set(friction->mutable_fdir1(), _opt.direction1);
+
+            return ServerFixture::SpawnModel(model);
           }
 
   /// \brief Use the friction_demo world.
   /// \param[in] _physicsEngine Physics engine to use.
-  public: void FrictionDemo(const std::string &_physicsEngine);
+  public: void FrictionDemo(const std::string &_physicsEngine,
+                            const std::string &_solverType="quick",
+                            const std::string &_worldSolverType="ODE_DANTZIG");
+
+  /// \brief Friction test of maximum dissipation principle.
+  /// Basically test that friction force vector is aligned with
+  /// and opposes velocity vector.
+  /// \param[in] _physicsEngine Physics engine to use.
+  public: void MaximumDissipation(const std::string &_physicsEngine);
 
   /// \brief Test friction directions for friction pyramid with boxes.
   /// \param[in] _physicsEngine Physics engine to use.
@@ -228,15 +160,10 @@ class PhysicsFrictionTest : public ServerFixture,
   /// no NaN's are generated.
   /// \param[in] _physicsEngine Physics engine to use.
   public: void DirectionNaN(const std::string &_physicsEngine);
+};
 
-  /// \brief Test Link::GetWorldInertia* functions.
-  /// \TODO: move the SpawnBox function to ServerFixture,
-  /// and then move this test to a different file.
-  /// \param[in] _physicsEngine Physics engine to use.
-  public: void LinkGetWorldInertia(const std::string &_physicsEngine);
-
-  /// \brief Count of spawned models, used to ensure unique model names.
-  private: unsigned int spawnCount;
+class WorldStepFrictionTest : public PhysicsFrictionTest
+{
 };
 
 /////////////////////////////////////////////////
@@ -246,7 +173,9 @@ class PhysicsFrictionTest : public ServerFixture,
 // box has a different coefficient of friction. These friction coefficients
 // are chosen to be close to the value that would prevent sliding according
 // to the Coulomb model.
-void PhysicsFrictionTest::FrictionDemo(const std::string &_physicsEngine)
+void PhysicsFrictionTest::FrictionDemo(const std::string &_physicsEngine,
+                                       const std::string &_solverType,
+                                       const std::string &_worldSolverType)
 {
   if (_physicsEngine == "simbody")
   {
@@ -255,13 +184,6 @@ void PhysicsFrictionTest::FrictionDemo(const std::string &_physicsEngine)
           << std::endl;
     return;
   }
-  if (_physicsEngine == "dart")
-  {
-    gzerr << "Aborting test since there's an issue with dart's friction"
-          << " parameters (#1000)"
-          << std::endl;
-    return;
-  }
 
   Load("worlds/friction_demo.world", true, _physicsEngine);
   physics::WorldPtr world = physics::get_world("default");
@@ -278,6 +200,15 @@ void PhysicsFrictionTest::FrictionDemo(const std::string &_physicsEngine)
   EXPECT_DOUBLE_EQ(g.y, -1.0);
   EXPECT_DOUBLE_EQ(g.z, -1.0);
 
+  if (_physicsEngine == "ode")
+  {
+    // Set solver type
+    physics->SetParam("solver_type", _solverType);
+
+    // Set world step solver type
+    physics->SetParam("world_step_solver", _worldSolverType);
+  }
+
   std::vector<PhysicsFrictionTest::FrictionDemoBox> boxes;
   std::vector<PhysicsFrictionTest::FrictionDemoBox>::iterator box;
   boxes.push_back(PhysicsFrictionTest::FrictionDemoBox(world, "box_01_model"));
@@ -300,24 +231,40 @@ void PhysicsFrictionTest::FrictionDemo(const std::string &_physicsEngine)
     world->Step(500);
     t = world->GetSimTime();
 
+    double yTolerance = g_friction_tolerance;
+    if (_solverType == "world")
+    {
+      if (_worldSolverType == "DART_PGS")
+        yTolerance *= 2;
+      else if (_worldSolverType == "ODE_DANTZIG")
+        yTolerance = 0.84;
+    }
+
     for (box = boxes.begin(); box != boxes.end(); ++box)
     {
       math::Vector3 vel = box->model->GetWorldLinearVel();
       EXPECT_NEAR(vel.x, 0, g_friction_tolerance);
-      EXPECT_NEAR(vel.z, 0, g_friction_tolerance);
+      EXPECT_NEAR(vel.z, 0, yTolerance);
 
       // Coulomb friction model
       if (box->friction >= 1.0)
       {
         // Friction is large enough to prevent motion
-        EXPECT_NEAR(vel.y, 0, g_friction_tolerance);
+        EXPECT_NEAR(vel.y, 0, yTolerance);
       }
       else
       {
         // Friction is small enough to allow motion
         // Expect velocity = acceleration * time
+        double vyTolerance = yTolerance;
+#ifdef HAVE_BULLET
+        if (_physicsEngine == "bullet" && sizeof(btScalar) == 4)
+        {
+          vyTolerance *= 22;
+        }
+#endif
         EXPECT_NEAR(vel.y, (g.y + box->friction) * t.Double(),
-                    g_friction_tolerance);
+                    vyTolerance);
       }
     }
   }
@@ -327,6 +274,96 @@ void PhysicsFrictionTest::FrictionDemo(const std::string &_physicsEngine)
   }
 }
 
+/////////////////////////////////////////////////
+// MaximumDissipation test:
+// Start with empty world,
+// spawn a bunch of boxes,
+// sets box velocities to different angles,
+// expect velocity unit vectors to stay constant while in motion.
+void PhysicsFrictionTest::MaximumDissipation(const std::string &_physicsEngine)
+{
+  // Load an empty world
+  Load("worlds/empty.world", true, _physicsEngine);
+  physics::WorldPtr world = physics::get_world("default");
+  ASSERT_TRUE(world != NULL);
+
+  // Verify physics engine type
+  physics::PhysicsEnginePtr physics = world->GetPhysicsEngine();
+  ASSERT_TRUE(physics != NULL);
+  EXPECT_EQ(physics->GetType(), _physicsEngine);
+
+  // get the gravity vector
+  // small positive y component
+  math::Vector3 g = physics->GetGravity();
+
+  // Set friction model
+  // "cone_model", "pyramid_model", "box_model"
+  const std::string frictionModel = "cone_model";
+  physics->SetParam("friction_model", frictionModel);
+
+  // Spawn concentric semi-circles of boxes
+  int boxes = 32;
+  double dx = 0.5;
+  double dy = 0.5;
+  double dz = 0.2;
+  std::map<physics::ModelPtr, double> modelAngles;
+
+  for (int ring = 0; ring < 5; ++ring)
+  {
+    gzdbg << "Spawn ring " << ring+1 << " of boxes" << std::endl;
+    for (int i = 0; i < boxes; ++i)
+    {
+      // Set box size and anisotropic friction
+      SpawnFrictionBoxOptions opt;
+      opt.size.Set(dx, dy, dz);
+      opt.friction1 = 0.3;
+      opt.friction2 = opt.friction1;
+
+      // Compute angle for each box
+      double radius = 9.0 + ring;
+      double angle = 2*M_PI*static_cast<double>(i) / static_cast<double>(boxes);
+      opt.modelPose.Pos().Set(radius*cos(angle), radius*sin(angle), dz/2);
+
+      if (ring == 0)
+        opt.direction1 = ignition::math::Vector3d(-sin(angle), cos(angle), 0);
+      else if (ring < 4)
+        opt.direction1 = ignition::math::Vector3d(0.0, 1.0, 0.0);
+
+      if (ring == 1)
+        opt.collisionPose.Rot().Euler(0.0, 0.0, angle);
+
+      if (ring == 2)
+        opt.linkPose.Rot().Euler(0.0, 0.0, angle);
+
+      if (ring == 3)
+        opt.modelPose.Rot().Euler(0.0, 0.0, angle);
+
+      physics::ModelPtr model = SpawnBox(opt);
+      ASSERT_TRUE(model != NULL);
+      modelAngles[model] = angle;
+
+      // Set velocity, larger for outer rings.
+      model->SetLinearVel(
+          radius * ignition::math::Vector3d(cos(angle), sin(angle), 0));
+    }
+  }
+
+  world->Step(1500);
+
+  gzdbg << "Checking position of boxes" << std::endl;
+  std::map<physics::ModelPtr, double>::iterator iter;
+  for (iter = modelAngles.begin(); iter != modelAngles.end(); ++iter)
+  {
+    double cosAngle = cos(iter->second);
+    double sinAngle = sin(iter->second);
+    math::Vector3 pos = iter->first->GetWorldPose().pos;
+    double cosPosAngle = pos.x / pos.GetLength();
+    double sinPosAngle = pos.y / pos.GetLength();
+    EXPECT_NEAR(cosAngle, cosPosAngle, 1e-2);
+    EXPECT_NEAR(sinAngle, sinPosAngle, 1e-2);
+  }
+}
+
 /////////////////////////////////////////////////
 // BoxDirectionRing:
 // Spawn several boxes with different friction direction parameters.
@@ -365,7 +402,6 @@ void PhysicsFrictionTest::BoxDirectionRing(const std::string &_physicsEngine)
   EXPECT_EQ(physics->GetType(), _physicsEngine);
 
   // set the gravity vector
-  // small positive y component
   math::Vector3 g(0.0, 1.0, -9.81);
   physics->SetGravity(g);
 
@@ -390,21 +426,21 @@ void PhysicsFrictionTest::BoxDirectionRing(const std::string &_physicsEngine)
       // Compute angle for each box
       double radius = 5.0 + ring;
       double angle = M_PI*static_cast<double>(i) / static_cast<double>(boxes);
-      opt.modelPose.pos.Set(radius*cos(angle), radius*sin(angle), dz/2);
+      opt.modelPose.Pos().Set(radius*cos(angle), radius*sin(angle), dz/2);
 
       if (ring == 0)
-        opt.direction1 = math::Vector3(-sin(angle), cos(angle), 0);
+        opt.direction1 = ignition::math::Vector3d(-sin(angle), cos(angle), 0);
       else
-        opt.direction1 = math::Vector3(0.0, 1.0, 0.0);
+        opt.direction1 = ignition::math::Vector3d(0.0, 1.0, 0.0);
 
       if (ring == 1)
-        opt.collisionPose.rot.SetFromEuler(0.0, 0.0, angle);
+        opt.collisionPose.Rot().Euler(0.0, 0.0, angle);
 
       if (ring == 2)
-        opt.linkPose.rot.SetFromEuler(0.0, 0.0, angle);
+        opt.linkPose.Rot().Euler(0.0, 0.0, angle);
 
       if (ring == 3)
-        opt.modelPose.rot.SetFromEuler(0.0, 0.0, angle);
+        opt.modelPose.Rot().Euler(0.0, 0.0, angle);
 
       physics::ModelPtr model = SpawnBox(opt);
       ASSERT_TRUE(model != NULL);
@@ -479,8 +515,8 @@ void PhysicsFrictionTest::DirectionNaN(const std::string &_physicsEngine)
   // Set box size and anisotropic friction
   SpawnFrictionBoxOptions opt;
   opt.size.Set(dx, dy, dz);
-  opt.direction1 = math::Vector3(0.0, 0.0, 1.0);
-  opt.modelPose.pos.z = dz/2;
+  opt.direction1 = ignition::math::Vector3d(0.0, 0.0, 1.0);
+  opt.modelPose.Pos().Z(dz/2);
 
   physics::ModelPtr model = SpawnBox(opt);
   ASSERT_TRUE(model != NULL);
@@ -496,160 +532,43 @@ void PhysicsFrictionTest::DirectionNaN(const std::string &_physicsEngine)
   EXPECT_NEAR(velMag, vel.y, g_friction_tolerance);
 }
 
-/////////////////////////////////////////////////
-// LinkGetWorldInertia:
-// Spawn boxes and verify Link::GetWorldInertia* functions
-void PhysicsFrictionTest::LinkGetWorldInertia(const std::string &_physicsEngine)
-{
-  // Load a blank world (no ground plane)
-  Load("worlds/blank.world", true, _physicsEngine);
-  physics::WorldPtr world = physics::get_world("default");
-  ASSERT_TRUE(world != NULL);
-
-  // Verify physics engine type
-  physics::PhysicsEnginePtr physics = world->GetPhysicsEngine();
-  ASSERT_TRUE(physics != NULL);
-  EXPECT_EQ(physics->GetType(), _physicsEngine);
-
-  // disable gravity
-  physics->SetGravity(math::Vector3::Zero);
-
-  // Box size
-  double dx = 1.0;
-  double dy = 4.0;
-  double dz = 9.0;
-  double mass = 10.0;
-  double angle = M_PI / 3.0;
-
-  const unsigned int testCases = 4;
-  for (unsigned int i = 0; i < testCases; ++i)
-  {
-    // Set box size and position
-    SpawnFrictionBoxOptions opt;
-    opt.size.Set(dx, dy, dz);
-    opt.mass = mass;
-    opt.modelPose.pos.x = i * dz;
-    opt.modelPose.pos.z = dz;
-
-    // i=0: rotated model pose
-    //  expect inertial pose to match model pose
-    if (i == 0)
-    {
-      opt.modelPose.rot.SetFromEuler(0.0, 0.0, angle);
-    }
-    // i=1: rotated link pose
-    //  expect inertial pose to match link pose
-    else if (i == 1)
-    {
-      opt.linkPose.rot.SetFromEuler(0.0, 0.0, angle);
-    }
-    // i=2: rotated inertial pose
-    //  expect inertial pose to differ from link pose
-    else if (i == 2)
-    {
-      opt.inertialPose.rot.SetFromEuler(0.0, 0.0, angle);
-    }
-    // i=3: offset inertial pose
-    //  expect inertial pose to differ from link pose
-    else if (i == 3)
-    {
-      opt.inertialPose.pos.Set(1, 1, 1);
-    }
-
-    physics::ModelPtr model = SpawnBox(opt);
-    ASSERT_TRUE(model != NULL);
-
-    physics::LinkPtr link = model->GetLink();
-    ASSERT_TRUE(link != NULL);
-
-    EXPECT_EQ(model->GetWorldPose(), opt.modelPose);
-    EXPECT_EQ(link->GetWorldPose(), opt.linkPose + opt.modelPose);
-    EXPECT_EQ(link->GetWorldInertialPose(),
-              opt.inertialPose + opt.linkPose + opt.modelPose);
-
-    // i=0: rotated model pose
-    //  expect inertial pose to match model pose
-    if (i == 0)
-    {
-      EXPECT_EQ(model->GetWorldPose(),
-                link->GetWorldInertialPose());
-    }
-    // i=1: rotated link pose
-    //  expect inertial pose to match link pose
-    else if (i == 1)
-    {
-      EXPECT_EQ(link->GetWorldPose(),
-                link->GetWorldInertialPose());
-    }
-    // i=2: offset and rotated inertial pose
-    //  expect inertial pose to differ from link pose
-    else if (i == 2)
-    {
-      EXPECT_EQ(link->GetWorldPose().pos,
-                link->GetWorldInertialPose().pos);
-    }
-    // i=3: offset inertial pose
-    //  expect inertial pose to differ from link pose
-    else if (i == 3)
-    {
-      EXPECT_EQ(link->GetWorldPose().pos + opt.inertialPose.pos,
-                link->GetWorldInertialPose().pos);
-    }
-
-    // Expect rotated inertia matrix
-    math::Matrix3 inertia = link->GetWorldInertiaMatrix();
-    if (i == 3)
-    {
-      EXPECT_NEAR(inertia[0][0], 80.8333, 1e-4);
-      EXPECT_NEAR(inertia[1][1], 68.3333, 1e-4);
-      EXPECT_NEAR(inertia[2][2], 14.1667, 1e-4);
-      for (int row = 0; row < 3; ++row)
-        for (int col = 0; col < 3; ++col)
-          if (row != col)
-            EXPECT_NEAR(inertia[row][col], 0.0, g_friction_tolerance);
-    }
-    else
-    {
-      EXPECT_NEAR(inertia[0][0], 71.4583, 1e-4);
-      EXPECT_NEAR(inertia[1][1], 77.7083, 1e-4);
-      EXPECT_NEAR(inertia[2][2], 14.1667, 1e-4);
-      EXPECT_NEAR(inertia[0][1],  5.4126, 1e-4);
-      EXPECT_NEAR(inertia[1][0],  5.4126, 1e-4);
-      EXPECT_NEAR(inertia[0][2], 0, g_friction_tolerance);
-      EXPECT_NEAR(inertia[2][0], 0, g_friction_tolerance);
-      EXPECT_NEAR(inertia[1][2], 0, g_friction_tolerance);
-      EXPECT_NEAR(inertia[2][1], 0, g_friction_tolerance);
-    }
-
-    // For 0-2, apply torque and expect equivalent response
-    if (i <= 2)
-    {
-      for (int step = 0; step < 50; ++step)
-      {
-        link->SetTorque(math::Vector3(100, 0, 0));
-        world->Step(1);
-      }
-      if (_physicsEngine.compare("dart") == 0)
-      {
-        gzerr << "Dart fails this portion of the test (#1090)" << std::endl;
-      }
-      else
-      {
-        math::Vector3 vel = link->GetWorldAngularVel();
-        EXPECT_NEAR(vel.x,  0.0703, g_friction_tolerance);
-        EXPECT_NEAR(vel.y, -0.0049, g_friction_tolerance);
-        EXPECT_NEAR(vel.z,  0.0000, g_friction_tolerance);
-      }
-    }
-  }
-}
-
 /////////////////////////////////////////////////
 TEST_P(PhysicsFrictionTest, FrictionDemo)
 {
   FrictionDemo(GetParam());
 }
 
+/////////////////////////////////////////////////
+TEST_P(WorldStepFrictionTest, FrictionDemoWorldStep)
+{
+  std::string worldStepSolver = GetParam();
+  if (worldStepSolver.compare("BULLET_PGS") == 0 ||
+      worldStepSolver.compare("BULLET_LEMKE") == 0)
+  {
+    gzerr << "Solver ["
+          << worldStepSolver
+          << "] doesn't yet work with this test."
+          << std::endl;
+    return;
+  }
+  FrictionDemo("ode", "world", worldStepSolver);
+}
+
+/////////////////////////////////////////////////
+TEST_P(PhysicsFrictionTest, MaximumDissipation)
+{
+  if (std::string("ode").compare(GetParam()) == 0)
+  {
+    MaximumDissipation(GetParam());
+  }
+  else
+  {
+    gzerr << "Skipping test for physics engine "
+          << GetParam()
+          << std::endl;
+  }
+}
+
 /////////////////////////////////////////////////
 TEST_P(PhysicsFrictionTest, BoxDirectionRing)
 {
@@ -662,15 +581,12 @@ TEST_P(PhysicsFrictionTest, DirectionNaN)
   DirectionNaN(GetParam());
 }
 
-/////////////////////////////////////////////////
-TEST_P(PhysicsFrictionTest, LinkGetWorldInertia)
-{
-  LinkGetWorldInertia(GetParam());
-}
-
 INSTANTIATE_TEST_CASE_P(PhysicsEngines, PhysicsFrictionTest,
                         PHYSICS_ENGINE_VALUES);
 
+INSTANTIATE_TEST_CASE_P(WorldStepSolvers, WorldStepFrictionTest,
+                        WORLD_STEP_SOLVERS);
+
 /////////////////////////////////////////////////
 int main(int argc, char **argv)
 {
