@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2015 Open Source Robotics Foundation
+ * Copyright (C) 2014-2016 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,6 +14,14 @@
  * limitations under the License.
  *
 */
+
+#ifdef _WIN32
+  // Ensure that Winsock2.h is included before Windows.h, which can get
+  // pulled in by anybody (e.g., Boost).
+  #include <Winsock2.h>
+#endif
+
+#include <stdio.h>
 #include <signal.h>
 #include <boost/filesystem.hpp>
 #include <boost/algorithm/string.hpp>
@@ -137,7 +145,11 @@ bool Command::Run(int _argc, char **_argv)
   {
     int argvLen = strlen(_argv[i]) + 1;
     this->argv[i] = new char[argvLen];
+    #ifndef _WIN32
     snprintf(this->argv[i], argvLen, "%s", _argv[i]);
+    #else
+    sprintf_s(this->argv[i], argvLen, "%s", _argv[i]);
+    #endif
   }
 
   // The SDF find file callback.
@@ -289,7 +301,8 @@ PhysicsCommand::PhysicsCommand()
      "eg: -g 0,0,-9.8")
     ("step-size,s", po::value<double>(), "Maximum step size (seconds).")
     ("iters,i", po::value<double>(), "Number of iterations.")
-    ("update-rate,u", po::value<double>(), "Target real-time update rate.");
+    ("update-rate,u", po::value<double>(), "Target real-time update rate.")
+    ("profile,o", po::value<std::string>(), "Preset physics profile.");
 }
 
 /////////////////////////////////////////////////
@@ -351,6 +364,13 @@ bool PhysicsCommand::RunImpl()
     good = true;
   }
 
+  if (this->vm.count("profile") &&
+      this->vm["profile"].as<std::string>().size() > 0)
+  {
+    msg.set_profile_name(this->vm["profile"].as<std::string>());
+    good = true;
+  }
+
   if (good)
     pub->Publish(msg, true);
   else
@@ -449,7 +469,7 @@ bool ModelCommand::RunImpl()
       return false;
     }
 
-    boost::shared_ptr<sdf::SDF> sdf(new sdf::SDF());
+    sdf::SDFPtr sdf(new sdf::SDF());
     if (!sdf::init(sdf))
     {
       std::cerr << "Error: SDF parsing the xml failed" << std::endl;
@@ -475,7 +495,7 @@ bool ModelCommand::RunImpl()
       sdfString += input;
     }
 
-    boost::shared_ptr<sdf::SDF> sdf(new sdf::SDF());
+    sdf::SDFPtr sdf(new sdf::SDF());
     if (!sdf::init(sdf))
     {
       std::cerr << "Error: SDF parsing the xml failed" << std::endl;
@@ -503,7 +523,7 @@ bool ModelCommand::RunImpl()
       if (this->vm.count("info"))
         std::cout << modelMsg.DebugString() << std::endl;
       else if (this->vm.count("pose"))
-        std::cout << gazebo::msgs::Convert(modelMsg.pose()) << std::endl;
+        std::cout << gazebo::msgs::ConvertIgn(modelMsg.pose()) << std::endl;
     }
     else
     {
@@ -520,7 +540,7 @@ bool ModelCommand::RunImpl()
 
     msgs::Model msg;
     msg.set_name(modelName);
-    msgs::Set(msg.mutable_pose(), pose);
+    msgs::Set(msg.mutable_pose(), pose.Ign());
     pub->Publish(msg, true);
   }
 
@@ -528,10 +548,10 @@ bool ModelCommand::RunImpl()
 }
 
 /////////////////////////////////////////////////
-bool ModelCommand::ProcessSpawn(boost::shared_ptr<sdf::SDF> _sdf,
+bool ModelCommand::ProcessSpawn(sdf::SDFPtr _sdf,
     const std::string &_name, const math::Pose &_pose, transport::NodePtr _node)
 {
-  sdf::ElementPtr modelElem = _sdf->root->GetElement("model");
+  sdf::ElementPtr modelElem = _sdf->Root()->GetElement("model");
 
   if (!modelElem)
   {
@@ -548,7 +568,7 @@ bool ModelCommand::ProcessSpawn(boost::shared_ptr<sdf::SDF> _sdf,
 
   msgs::Factory msg;
   msg.set_sdf(_sdf->ToString());
-  msgs::Set(msg.mutable_pose(), _pose);
+  msgs::Set(msg.mutable_pose(), _pose.Ign());
   pub->Publish(msg, true);
 
   return true;
@@ -770,6 +790,8 @@ bool CameraCommand::RunImpl()
   transport::NodePtr node(new transport::Node());
   node->Init(worldName);
 
+  boost::replace_all(cameraName, "::", "/");
+
   transport::PublisherPtr pub =
     node->Advertise<msgs::CameraCmd>(
         std::string("~/") + cameraName + "/cmd");
@@ -929,7 +951,7 @@ bool SDFCommand::TransportRequired()
 /////////////////////////////////////////////////
 bool SDFCommand::RunImpl()
 {
-  sdf::SDF::version = SDF_VERSION;
+  sdf::SDF::Version(SDF_VERSION);
 
   try
   {
@@ -942,14 +964,14 @@ bool SDFCommand::RunImpl()
     std::cerr << "Error initializing log file" << std::endl;
   }
 
-  boost::shared_ptr<sdf::SDF> sdf(new sdf::SDF());
+  sdf::SDFPtr sdf(new sdf::SDF());
 
   if (this->vm.count("version"))
   {
     try
     {
-      sdf::SDF::version = boost::lexical_cast<std::string>(
-          this->vm["version"].as<std::string>());
+      sdf::SDF::Version(boost::lexical_cast<std::string>(
+          this->vm["version"].as<std::string>()));
     }
     catch(...)
     {
@@ -998,7 +1020,7 @@ bool SDFCommand::RunImpl()
     TiXmlDocument xmlDoc;
     if (xmlDoc.LoadFile(path.string()))
     {
-      if (sdf::Converter::Convert(&xmlDoc, sdf::SDF::version, true))
+      if (sdf::Converter::Convert(&xmlDoc, sdf::SDF::Version(), true))
       {
         // Create an XML printer to control formatting
         TiXmlPrinter printer;
