@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Open Source Robotics Foundation
+ * Copyright (C) 2012-2016 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,7 +15,13 @@
  *
 */
 
-#include <boost/algorithm/string/regex.hpp>
+#ifdef _WIN32
+  // Ensure that Winsock2.h is included before Windows.h, which can get
+  // pulled in by anybody (e.g., Boost).
+  #include <Winsock2.h>
+#endif
+
+#include <boost/algorithm/string.hpp>
 #include <boost/date_time/posix_time/posix_time.hpp>
 #include <boost/date_time/posix_time/posix_time_io.hpp>
 
@@ -47,6 +53,8 @@ std::ostringstream &FilterBase::Out(std::ostringstream &_stream,
       _stream << _state.GetRealTime().Double() << " ";
     else if (this->stamp == "wall")
       _stream << _state.GetWallTime().Double() << " ";
+    else if (this->stamp == "iterations")
+      _stream << _state.GetIterations() << " ";
     _stream.setf(flags);
   }
 
@@ -547,7 +555,8 @@ std::string StateFilter::Filter(const std::string &_stateString)
       << "<state world_name='" << state.GetName() << "'>\n"
       << "<sim_time>" << state.GetSimTime() << "</sim_time>\n"
       << "<real_time>" << state.GetRealTime() << "</real_time>\n"
-      << "<wall_time>" << state.GetWallTime() << "</wall_time>\n";
+      << "<wall_time>" << state.GetWallTime() << "</wall_time>\n"
+      << "<iterations>" << state.GetIterations() << "</iterations>\n";
   }
 
   result << this->filter.Filter(state);
@@ -720,10 +729,10 @@ void LogCommand::Info(const std::string &_filename)
     }
 
       // Get the last chunk for the endTime
-    if (play->GetChunkCount() > 1)
+    if (play->ChunkCount() > 1)
     {
       std::string stateString;
-      play->GetChunk(play->GetChunkCount()-1, stateString);
+      play->Chunk(play->ChunkCount()-1, stateString);
 
       g_stateSdf->ClearElements();
       sdf::readString(stateString, g_stateSdf);
@@ -748,9 +757,9 @@ void LogCommand::Info(const std::string &_filename)
 
   // Output info
   std::cout
-    << "Log Version:    " << play->GetLogVersion() << "\n"
-    << "Gazebo Version: " << play->GetGazeboVersion() << "\n"
-    << "Random Seed:    " << play->GetRandSeed() << "\n"
+    << "Log Version:    " << play->LogVersion() << "\n"
+    << "Gazebo Version: " << play->GazeboVersion() << "\n"
+    << "Random Seed:    " << play->RandSeed() << "\n"
     // << "Start:          " << boost::posix_time::from_time_t(startTime.sec)
     // << "." << startTime.nsec << "\n"
     // << "End:            " << boost::posix_time::from_time_t(endTime.sec)
@@ -764,7 +773,7 @@ void LogCommand::Info(const std::string &_filename)
     //                       << deltaTime.nsec << "\n"
     // << "Steps:          " << play->GetChunkCount() << "\n"
     << "Size:           " << this->GetFileSizeStr(_filename) << "\n"
-    << "Encoding:       " << play->GetEncoding() << "\n"
+    << "Encoding:       " << play->Encoding() << "\n"
     // << "Model Count:    " << modelCount << "\n"
     << "\n";
 }
@@ -778,7 +787,7 @@ void LogCommand::Echo(const std::string &_filter, bool _raw,
 
   // Output the header
   if (!_raw)
-    std::cout << play->GetHeader() << std::endl;
+    std::cout << play->Header() << std::endl;
 
   StateFilter filter(!_raw, _stamp, _hz);
   filter.Init(_filter);
@@ -817,7 +826,7 @@ void LogCommand::Step(const std::string &_filter, bool _raw,
   gazebo::util::LogPlay *play = gazebo::util::LogPlay::Instance();
 
   if (!_raw)
-    std::cout << play->GetHeader() << std::endl;
+    std::cout << play->Header() << std::endl;
 
   char c = '\0';
 
@@ -908,16 +917,18 @@ std::string LogCommand::GetFileSizeStr(const std::string &_filename)
 /////////////////////////////////////////////////
 int LogCommand::GetChar()
 {
+# ifndef _WIN32
   struct termios oldt, newt;
-  int ch;
   tcgetattr(STDIN_FILENO, &oldt);
   newt = oldt;
   newt.c_lflag &= ~(ICANON | ECHO);
   tcsetattr(STDIN_FILENO, TCSANOW, &newt);
-  ch = getchar();
+  int ch = getchar();
   tcsetattr(STDIN_FILENO, TCSANOW, &oldt);
-
   return ch;
+# else
+  return 'q';
+# endif
 }
 
 /////////////////////////////////////////////////
